@@ -20,14 +20,17 @@
 #include <arc/OptionParser.h>
 #include <arc/StringConv.h>
 #include <arc/URL.h>
+#include <arc/UserConfig.h>
 #include <arc/Utils.h>
 #include <arc/XMLNode.h>
-#include <arc/client/Submitter.h>
-#include <arc/client/TargetGenerator.h>
-#include <arc/client/JobDescription.h>
-#include <arc/UserConfig.h>
 #include <arc/client/Broker.h>
+#include <arc/client/JobDescription.h>
+#include <arc/client/TargetGenerator.h>
+#include <arc/client/Submitter.h>
+#include <arc/loader/Plugin.h>
+#include <arc/loader/FinderLoader.h>
 #include <arc/credential/Credential.h>
+
 static Arc::Logger logger(Arc::Logger::getRootLogger(), "arcsub");
 
 int submit(const Arc::UserConfig& usercfg, const std::list<Arc::JobDescription>& jobdescriptionlist, const std::string& jobidfile);
@@ -118,7 +121,7 @@ int RUNSUB(main)(int argc, char **argv) {
 
   std::string broker;
   options.AddOption('b', "broker",
-                    istring("select broker method (Random (default), FastestQueue, or custom)"),
+                    istring("select broker method (list available brokers with --listplugins flag)"),
                     istring("broker"), broker);
 
 /**
@@ -129,6 +132,10 @@ int RUNSUB(main)(int argc, char **argv) {
                     dolocalsandbox);
 */
 
+  bool show_plugins = false;
+  options.AddOption('P', "listplugins", istring("list the available plugins"),
+                    show_plugins);
+
   bool version = false;
   options.AddOption('v', "version", istring("print version information"),
                     version);
@@ -151,25 +158,71 @@ int RUNSUB(main)(int argc, char **argv) {
     return 1;
   }
 
+  if (show_plugins) {
+    std::list<Arc::ModuleDesc> modules;
+    Arc::PluginsFactory pf(Arc::BaseConfig().MakeConfig(Arc::Config()).Parent());
+    
+    pf.scan(Arc::FinderLoader::GetLibrariesList(), modules);
+    Arc::PluginsFactory::FilterByKind("HED:Submitter", modules);
+    std::cout << Arc::IString("Types of execution services arcsub is able to submit jobs to:") << std::endl;
+    for (std::list<Arc::ModuleDesc>::iterator itMod = modules.begin();
+         itMod != modules.end(); itMod++) {
+      for (std::list<Arc::PluginDesc>::iterator itPlug = itMod->plugins.begin();
+           itPlug != itMod->plugins.end(); itPlug++) {
+        std::cout << "  " << itPlug->name << " - " << itPlug->description << std::endl;
+      }
+    }
+    
+    pf.scan(Arc::FinderLoader::GetLibrariesList(), modules);
+    Arc::PluginsFactory::FilterByKind("HED:TargetRetriever", modules);
+    std::cout << Arc::IString("Types of index and information services which arcsub is able collect information from:") << std::endl;
+    for (std::list<Arc::ModuleDesc>::iterator itMod = modules.begin();
+         itMod != modules.end(); itMod++) {
+      for (std::list<Arc::PluginDesc>::iterator itPlug = itMod->plugins.begin();
+           itPlug != itMod->plugins.end(); itPlug++) {
+        std::cout << "  " << itPlug->name << " - " << itPlug->description << std::endl;
+      }
+    }
+    
+    pf.scan(Arc::FinderLoader::GetLibrariesList(), modules);
+    Arc::PluginsFactory::FilterByKind("HED:JobDescriptionParser", modules);
+    std::cout << Arc::IString("Job description languages supported by arcsub:") << std::endl;
+    for (std::list<Arc::ModuleDesc>::iterator itMod = modules.begin();
+         itMod != modules.end(); itMod++) {
+      for (std::list<Arc::PluginDesc>::iterator itPlug = itMod->plugins.begin();
+           itPlug != itMod->plugins.end(); itPlug++) {
+        std::cout << "  " << itPlug->name << " - " << itPlug->description << std::endl;
+      }
+    }
+
+    modules.clear();
+    pf.scan(Arc::FinderLoader::GetLibrariesList(), modules);
+    Arc::PluginsFactory::FilterByKind("HED:Broker", modules);
+    bool isDefaultBrokerLocated = false;
+    std::cout << Arc::IString("Brokers available to arcsub:") << std::endl;
+    for (std::list<Arc::ModuleDesc>::iterator itMod = modules.begin();
+         itMod != modules.end(); itMod++) {
+      for (std::list<Arc::PluginDesc>::iterator itPlug = itMod->plugins.begin();
+           itPlug != itMod->plugins.end(); itPlug++) {
+        std::cout << "  " << itPlug->name;
+        if (itPlug->name == usercfg.Broker().first) {
+          std::cout << " (default)";
+          isDefaultBrokerLocated = true;
+        }
+        std::cout << " - " << itPlug->description << std::endl;
+      }
+    }
+
+    if (!isDefaultBrokerLocated) {
+      logger.msg(Arc::WARNING, "Default broker (%s) is not available. When using arcsub a broker should be specified explicitly (-b option).", usercfg.Broker().first);
+    }
+    return 0;
+  }
+
   if (!usercfg.ProxyPath().empty() ) {
-    if (!usercfg.CACertificatesDirectory().empty() ){
-      Arc::Credential holder(usercfg.ProxyPath(), "", usercfg.CACertificatesDirectory(), "");
-      if(holder.IsValid() ){  
-        if (holder.GetEndTime() < Arc::Time()){
-          std::cout << Arc::IString("Proxy expired. Job submission aborted. Please run 'arcproxy'!") << std::endl;
-          return 1;
-        }
-        else if (holder.GetVerification()) {
-          logger.msg(Arc::INFO, "Proxy successfully verified.");
-        }
-      }
-      else {
-          std::cout << Arc::IString("Proxy not valid. Job submission aborted. Please run 'arcproxy'!") << std::endl;
-          return 1;
-        }
-      }
-      else {
-      std::cout << Arc::IString("Cannot find CA certificates directory. Please specify the location to the directory in the client configuration file.") << std::endl;
+    Arc::Credential holder(usercfg.ProxyPath(), "", "", "");
+    if (holder.GetEndTime() < Arc::Time()){
+      std::cout << Arc::IString("Proxy expired. Job submission aborted. Please run 'arcproxy'!") << std::endl;
       return 1;
     }
   }
@@ -319,7 +372,7 @@ int submit(const Arc::UserConfig& usercfg, const std::list<Arc::JobDescription>&
     bool descriptionSubmitted = false;
     submittedJobs.push_back(Arc::Job());
     if (ChosenBroker->Submit(targen.GetExecutionTargets(), *it, submittedJobs.back())) {
-      printjobid(submittedJobs.back().JobID.str(), jobidfile);
+      printjobid(submittedJobs.back().JobID.fullstr(), jobidfile);
       descriptionSubmitted = true;
     }
     else if (it->HasAlternatives()) {
@@ -327,7 +380,7 @@ int submit(const Arc::UserConfig& usercfg, const std::list<Arc::JobDescription>&
       for (std::list<Arc::JobDescription>::const_iterator itAlt = it->GetAlternatives().begin();
            itAlt != it->GetAlternatives().end(); itAlt++) {
         if (ChosenBroker->Submit(targen.GetExecutionTargets(), *itAlt, submittedJobs.back())) {
-          printjobid(submittedJobs.back().JobID.str(), jobidfile);
+          printjobid(submittedJobs.back().JobID.fullstr(), jobidfile);
           descriptionSubmitted = true;
           break;
         }
@@ -410,7 +463,7 @@ int dumpjobdescription(const Arc::UserConfig& usercfg, const std::list<Arc::JobD
       Arc::Submitter *submitter = target->GetSubmitter(usercfg);
 
       if (!submitter->ModifyJobDescription(jobdescdump, *target)) {
-        std::cout << "Unable to modify job description according to needs of the target resource." << std::endl;
+        std::cout << Arc::IString("Unable to modify job description according to needs of the target resource.") << std::endl;
         retval = 1;
         break;
       }
