@@ -16,6 +16,7 @@
 #include <arc/credential/Credential.h>
 #include <arc/data/DataHandle.h>
 #include <arc/data/DataPoint.h>
+#include <arc/loader/FinderLoader.h>
 #include <arc/OptionParser.h>
 
 static Arc::Logger logger(Arc::Logger::getRootLogger(), "arcls");
@@ -37,7 +38,7 @@ static bool arcls(const Arc::URL& dir_url,
   }
   if (dir_url.Protocol() == "urllist") {
     std::list<Arc::URL> dirs = Arc::ReadURLList(dir_url);
-    if (dirs.size() == 0) {
+    if (dirs.empty()) {
       logger.msg(Arc::ERROR, "Can't read list of locations from file %s",
                  dir_url.Path());
       return false;
@@ -122,7 +123,7 @@ static bool arcls(const Arc::URL& dir_url,
     }
   }
   if (!res) {
-    if (files.size() == 0) {
+    if (files.empty()) {
       logger.msg(Arc::ERROR, "Failed listing files");
       if (res.Retryable())
         logger.msg(Arc::ERROR, "This seems like a temporary error, please try again later");
@@ -143,6 +144,7 @@ static bool arcls(const Arc::URL& dir_url,
     }
     std::cout << std::endl;
   }
+  files.sort();
   for (std::list<Arc::FileInfo>::iterator i = files.begin();
        i != files.end(); i++) {
     std::cout << i->GetName();
@@ -193,6 +195,9 @@ static bool arcls(const Arc::URL& dir_url,
       for (std::map<std::string, std::string>::iterator mi = md.begin(); mi != md.end(); ++mi)
         std::cout<<mi->first<<":"<<mi->second<<std::endl;
     }
+  }
+  for (std::list<Arc::FileInfo>::iterator i = files.begin();
+       i != files.end(); i++) {
     // Do recursion. Recursion has no sense if listing is forbidden.
     if ((recursion > 0) && (!no_list)) {
       if (i->GetType() == Arc::FileInfo::file_type_dir) {
@@ -264,6 +269,11 @@ int main(int argc, char **argv) {
   options.AddOption('c', "checkaccess", istring("check readability of object, does not show any information about object"),
         checkaccess);
 
+  bool show_plugins = false;
+  options.AddOption('P', "listplugins",
+                    istring("list the available plugins (protocols supported)"),
+                    show_plugins);
+
   int timeout = 20;
   options.AddOption('t', "timeout", istring("timeout in seconds (default 20)"),
                     istring("seconds"), timeout);
@@ -293,6 +303,23 @@ int main(int argc, char **argv) {
   if (!debug.empty())
     Arc::Logger::getRootLogger().setThreshold(Arc::string_to_level(debug));
 
+  if (show_plugins) {
+    std::list<Arc::ModuleDesc> modules;
+    Arc::PluginsFactory pf(Arc::BaseConfig().MakeConfig(Arc::Config()).Parent());
+    pf.scan(Arc::FinderLoader::GetLibrariesList(), modules);
+    Arc::PluginsFactory::FilterByKind("HED:DMC", modules);
+
+    std::cout << Arc::IString("Protocol plugins available:") << std::endl;
+    for (std::list<Arc::ModuleDesc>::iterator itMod = modules.begin();
+         itMod != modules.end(); itMod++) {
+      for (std::list<Arc::PluginDesc>::iterator itPlug = itMod->plugins.begin();
+           itPlug != itMod->plugins.end(); itPlug++) {
+        std::cout << "  " << itPlug->name << " - " << itPlug->description << std::endl;
+      }
+    }
+    return 0;
+  }
+
   // credentials will be initialised later if necessary
   Arc::UserConfig usercfg(conffile, Arc::initializeCredentialsType::SkipCredentials);
   if (!usercfg) {
