@@ -4,7 +4,6 @@
 #define __ARC_SUBMITTERARC0_H__
 
 #ifdef WIN32
-#include <arc/win32.h>
 #include <fcntl.h>
 #endif
 
@@ -25,7 +24,6 @@ namespace Arc {
 
   public:
     static Plugin* Instance(PluginArgument *arg);
-    virtual bool GetTestJob(const int& testid, JobDescription& jobdescription);
     virtual bool Submit(const JobDescription& jobdesc, const ExecutionTarget& et, Job& job);
     virtual bool Migrate(const URL& jobid, const JobDescription& jobdesc,
                          const ExecutionTarget& et, bool forcemigration,
