@@ -29,10 +29,6 @@ namespace Arc {
     return new ARCJSDLParser();
   }
 
-  static void XmlErrorHandler(void* /* ctx */, const char* /* msg */) {
-    return;
-  }
-
   bool ARCJSDLParser::parseSoftware(XMLNode xmlSoftware, SoftwareRequirement& sr) const {
     for (int i = 0; (bool)(xmlSoftware["Software"][i]); i++) {
       Software::ComparisonOperator comOp = &Software::operator==;
@@ -78,9 +74,6 @@ namespace Arc {
           xmlVersion.NewAttribute("require") = Software::toString(*itCO);
       }
     }
-
-    if (bool(arcJSDL["Software"]) && sr.isRequiringAll())
-      arcJSDL.NewAttribute("require") = "all";
   }
 
   template<typename T>
@@ -166,23 +159,10 @@ namespace Arc {
     jobdescs.push_back(JobDescription());
     JobDescription& job = jobdescs.back();
 
-    xmlParserCtxtPtr ctxt = xmlNewParserCtxt();
-
-    if (ctxt == NULL) {
-        logger.msg(VERBOSE, "[ARCJSDLParser] Failed to create parser context");
-    }
-    xmlSetGenericErrorFunc(NULL, (xmlGenericErrorFunc)XmlErrorHandler);
     XMLNode node(source);
-    xmlSetGenericErrorFunc(NULL, NULL);
-
     if (!node) {
         logger.msg(VERBOSE, "[ARCJSDLParser] Parsing error: %s\n", (xmlGetLastError())->message);
     }
-    else if (ctxt->valid == 0) {
-        logger.msg(VERBOSE, "[ARCJSDLParser] Validating error");
-    }
-
-    xmlFreeParserCtxt(ctxt);
 
     if (node.Size() == 0) {
       logger.msg(VERBOSE, "[ARCJSDLParser] Wrong XML structure! ");
@@ -448,21 +428,43 @@ namespace Arc {
 
     // Range<int> DiskSpace;
     // If the consolidated element exist parse it, else try to parse the JSDL one.
-    if (bool(resource["DiskSpaceRequirement"]["DiskSpace"]))
-      parseRange<int>(resource["DiskSpaceRequirement"]["DiskSpace"], job.Resources.DiskSpaceRequirement.DiskSpace, -1);
-    else if (bool(resource["FileSystem"]["DiskSpace"]))
-      parseRange<int>(resource["FileSystem"]["DiskSpace"], job.Resources.DiskSpaceRequirement.DiskSpace, -1);
+    if (bool(resource["DiskSpaceRequirement"]["DiskSpace"])) {
+      Range<long long int> diskspace = -1;
+      parseRange<long long int>(resource["DiskSpaceRequirement"]["DiskSpace"], diskspace, -1);
+      if (diskspace > -1) {
+        job.Resources.DiskSpaceRequirement.DiskSpace.max = diskspace.max/(1024*1024);
+        job.Resources.DiskSpaceRequirement.DiskSpace.min = diskspace.min/(1024*1024);
+      }
+    }
+    else if (bool(resource["FileSystem"]["DiskSpace"])) {
+      Range<long long int> diskspace = -1;
+      parseRange<long long int>(resource["FileSystem"]["DiskSpace"], diskspace, -1);
+      if (diskspace > -1) {
+        job.Resources.DiskSpaceRequirement.DiskSpace.max = diskspace.max/(1024*1024);
+        job.Resources.DiskSpaceRequirement.DiskSpace.min = diskspace.min/(1024*1024);
+      }
+    }
 
     // int CacheDiskSpace;
     if (bool(resource["DiskSpaceRequirement"]["CacheDiskSpace"])) {
-      if (!stringto<int>((std::string)resource["DiskSpaceRequirement"]["CacheDiskSpace"], job.Resources.DiskSpaceRequirement.CacheDiskSpace))
+      long long int cachediskspace = -1;
+      if (!stringto<long long int>((std::string)resource["DiskSpaceRequirement"]["CacheDiskSpace"], cachediskspace)) {
          job.Resources.DiskSpaceRequirement.CacheDiskSpace = -1;
+      }
+      else if (cachediskspace > -1) {
+        job.Resources.DiskSpaceRequirement.CacheDiskSpace = cachediskspace/(1024*1024);
+      }
     }
 
     // int SessionDiskSpace;
     if (bool(resource["DiskSpaceRequirement"]["SessionDiskSpace"])) {
-      if (!stringto<int>((std::string)resource["DiskSpaceRequirement"]["SessionDiskSpace"], job.Resources.DiskSpaceRequirement.SessionDiskSpace))
+      long long int sessiondiskspace = -1;
+      if (!stringto<long long int>((std::string)resource["DiskSpaceRequirement"]["SessionDiskSpace"], sessiondiskspace)) {
         job.Resources.DiskSpaceRequirement.SessionDiskSpace = -1;
+      }
+     else if (sessiondiskspace > -1) {
+        job.Resources.DiskSpaceRequirement.SessionDiskSpace = sessiondiskspace/(1024*1024);
+      }
     }
 
     // Period SessionLifeTime;
@@ -530,8 +532,6 @@ namespace Arc {
 
     // SoftwareRequirement RunTimeEnvironment;
     if (bool(resource["RunTimeEnvironment"])) {
-      if (bool(resource["RunTimeEnvironment"].Attribute("require")))
-        job.Resources.RunTimeEnvironment.setRequirement(lower((std::string)resource["RunTimeEnvironment"].Attribute("require")) == "all");
       if (!parseSoftware(resource["RunTimeEnvironment"], job.Resources.RunTimeEnvironment)) {
         jobdescs.clear();
         return false;
@@ -631,6 +631,13 @@ namespace Arc {
         if (ds["DownloadToCache"] && !file.Source.empty()) {
           file.Source.back().AddOption("cache", (std::string)ds["DownloadToCache"]);
         }
+        if (ds["FileSize"]) {
+          stringto<long>((std::string)ds["FileSize"], file.FileSize);
+        }
+        if (ds["Checksum"]) {
+          file.Checksum = (std::string)ds["Checksum"];
+        }
+        
         job.Files.push_back(file);
       }
     }
@@ -896,27 +903,31 @@ namespace Arc {
     {
       // Range<int> DiskSpace;
       XMLNode xmlDiskSpace("<DiskSpace/>");
-      outputARCJSDLRange(job.Resources.DiskSpaceRequirement.DiskSpace, xmlDiskSpace, (int)-1);
+      Range<long long int> diskspace;
+      diskspace.max = job.Resources.DiskSpaceRequirement.DiskSpace.max*1024*1024;
+      diskspace.min = job.Resources.DiskSpaceRequirement.DiskSpace.min*1024*1024;
+      outputARCJSDLRange(diskspace, xmlDiskSpace, (long long int)-1);
 
       if (xmlDiskSpace.Size() > 0) {
         xmlResources.NewChild("DiskSpaceRequirement").NewChild(xmlDiskSpace);
 
         // int CacheDiskSpace;
-        if (job.Resources.DiskSpaceRequirement.CacheDiskSpace != -1)
-          xmlResources["DiskSpaceRequirement"].NewChild("CacheDiskSpace") = tostring(job.Resources.DiskSpaceRequirement.CacheDiskSpace);
+        if (job.Resources.DiskSpaceRequirement.CacheDiskSpace > -1) {
+          xmlResources["DiskSpaceRequirement"].NewChild("CacheDiskSpace") = tostring(job.Resources.DiskSpaceRequirement.CacheDiskSpace*1024*1024);
+        }
 
         // int SessionDiskSpace;
-        if (job.Resources.DiskSpaceRequirement.SessionDiskSpace != -1)
-          xmlResources["DiskSpaceRequirement"].NewChild("SessionDiskSpace") = tostring(job.Resources.DiskSpaceRequirement.SessionDiskSpace);
+        if (job.Resources.DiskSpaceRequirement.SessionDiskSpace > -1) {
+          xmlResources["DiskSpaceRequirement"].NewChild("SessionDiskSpace") = tostring(job.Resources.DiskSpaceRequirement.SessionDiskSpace*1024*1024);
+        }
       }
-    }
 
-    {
       // JSDL Compliance...
-      XMLNode xmlDiskSpace("<FileSystem/>");
-      outputJSDLRange(job.Resources.DiskSpaceRequirement.DiskSpace, xmlDiskSpace, (int)-1);
-      if (xmlDiskSpace.Size() > 0)
-        xmlResources.NewChild("FileSystem").NewChild(xmlDiskSpace);
+      XMLNode xmlFileSystem("<FileSystem/>");
+      outputJSDLRange(diskspace, xmlFileSystem, (long long int)-1);
+      if (xmlFileSystem.Size() > 0) {
+        xmlResources.NewChild("FileSystem").NewChild(xmlFileSystem);
+      }
     }
 
     // Period SessionLifeTime;
@@ -1060,6 +1071,13 @@ namespace Arc {
       if (!it->Source.empty() && !it->Source.front().Option("cache").empty()) {
         datastaging.NewChild("DownloadToCache") = it->Source.front().Option("cache");
       }
+
+      if (it->FileSize > -1) {
+        datastaging.NewChild("FileSize") = tostring(it->FileSize);
+      }
+      if (!it->Checksum.empty()) {
+        datastaging.NewChild("Checksum") = it->Checksum;
+      }
     }
     // End of DataStaging
 
