@@ -5,7 +5,6 @@
 #endif
 
 #ifdef WIN32 
-#include <arc/win32.h>
 #include <fcntl.h>
 #endif
 
@@ -47,7 +46,7 @@ namespace Arc {
 
   static Logger logger(Logger::rootLogger, "Lister");
 
-  bool SetAttributes(FileInfo& fi, const char *facts) {
+  void SetAttributes(FileInfo& fi, const char *facts) {
     const char *name;
     const char *value;
     const char *p = facts;
@@ -96,14 +95,20 @@ namespace Arc {
           fi.SetCreated(tmp_s); // ISO time
       }
     }
-    return true;
   }
 
-  Lister::callback_status_t Lister::wait_for_callback() {
+  Lister::callback_status_t Lister::wait_for_callback(int to) {
     callback_status_t res;
     globus_mutex_lock(&mutex);
-    while (callback_status == CALLBACK_NOTREADY)
-      globus_cond_wait(&cond, &mutex);
+    if(to < 0) {
+      while (callback_status == CALLBACK_NOTREADY) {
+        globus_cond_wait(&cond, &mutex);
+      }
+    } else {
+      globus_abstime_t timeout;
+      GlobusTimeAbstimeSet(timeout,to,0);
+      globus_cond_timedwait(&cond, &mutex, &timeout);
+    }
     res = callback_status;
     callback_status = CALLBACK_NOTREADY;
     globus_mutex_unlock(&mutex);
@@ -113,14 +118,27 @@ namespace Arc {
   Lister::callback_status_t Lister::wait_for_data_callback() {
     callback_status_t res;
     globus_mutex_lock(&mutex);
-    while (data_callback_status == CALLBACK_NOTREADY)
+    while (data_callback_status == CALLBACK_NOTREADY) {
       globus_cond_wait(&cond, &mutex);
+    }
     res = data_callback_status;
     data_callback_status = CALLBACK_NOTREADY;
     globus_mutex_unlock(&mutex);
     return res;
   }
 
+  Lister::callback_status_t Lister::wait_for_close_callback() {
+    callback_status_t res;
+    globus_mutex_lock(&mutex);
+    while (close_callback_status == CALLBACK_NOTREADY) {
+      globus_cond_wait(&cond, &mutex);
+    }
+    res = close_callback_status;
+    close_callback_status = CALLBACK_NOTREADY;
+    globus_mutex_unlock(&mutex);
+    return res;
+  }
+
   void Lister::resp_destroy() {
     globus_mutex_lock(&mutex);
     if (resp_n > 0) {
@@ -140,8 +158,7 @@ namespace Arc {
     globus_mutex_lock(&(it->mutex));
     if (error != GLOBUS_SUCCESS) {
       it->callback_status = CALLBACK_ERROR;
-      std::string tmp = globus_object_to_string(error);
-      logger.msg(INFO, "Failure: %s", tmp);
+      logger.msg(INFO, "Failure: %s", globus_object_to_string(error));
       if (response)
         logger.msg(INFO, "Response: %s", response->response_buffer);
     }
@@ -149,7 +166,7 @@ namespace Arc {
       if (it->resp_n < LISTER_MAX_RESPONSES) {
         memmove((it->resp) + 1, it->resp,
                 sizeof(globus_ftp_control_response_t) * (it->resp_n));
-        if (response->response_buffer) {
+        if (response && response->response_buffer) {
           globus_ftp_control_response_copy(response, it->resp);
         } else {    // invalid reply causes *_copy to segfault
           it->resp->response_buffer = (globus_byte_t*)strdup("000 ");
@@ -161,7 +178,7 @@ namespace Arc {
         (it->resp_n)++;
       }
       it->callback_status = CALLBACK_DONE;
-      if(response->response_buffer) {
+      if(response && response->response_buffer) {
         dos_to_unix((char*)(response->response_buffer));
         logger.msg(VERBOSE, "Response: %s", response->response_buffer);
       }
@@ -170,6 +187,29 @@ namespace Arc {
     globus_mutex_unlock(&(it->mutex));
   }
 
+  void Lister::close_callback(void *arg, globus_ftp_control_handle_t*,
+                             globus_object_t *error,
+                             globus_ftp_control_response_t *response) {
+    if(!arg) return;
+    Lister *it = (Lister*)arg;
+    Arc::Logger::getRootLogger().setThreadContext();
+    Arc::Logger::getRootLogger().removeDestinations();
+    globus_mutex_lock(&(it->mutex));
+    if (error != GLOBUS_SUCCESS) {
+      it->close_callback_status = CALLBACK_ERROR;
+    } else {
+      it->close_callback_status = CALLBACK_DONE;
+    }
+    globus_cond_signal(&(it->cond));
+    globus_mutex_unlock(&(it->mutex));
+    return;
+  }
+
+  void Lister::simple_callback(void *arg, globus_ftp_control_handle_t*,
+                             globus_object_t *error) {
+    resp_callback(arg, NULL, error, NULL);
+  }
+
   void Lister::list_read_callback(void *arg,
                                   globus_ftp_control_handle_t*,
                                   globus_object_t *error,
@@ -183,8 +223,7 @@ namespace Arc {
     if (error != GLOBUS_SUCCESS) {
       /* no such file or connection error - assume no such file */
       logger.msg(INFO, "Error getting list of files (in list)");
-      std::string tmp = globus_object_to_string(error);
-      logger.msg(INFO, "Failure: %s", tmp);
+      logger.msg(INFO, "Failure: %s", globus_object_to_string(error));
       logger.msg(INFO, "Assuming - file not found");
       globus_mutex_lock(&(it->mutex));
       it->data_callback_status = CALLBACK_ERROR;
@@ -242,18 +281,8 @@ namespace Arc {
           name=name_start+1;
         };
       };
-      std::list<FileInfo>::iterator i;
-      if (name[0] == '/') {
-        i = it->fnames.insert(it->fnames.end(), FileInfo(name));
-      } else {
-        std::string name_ = !it->path.empty() ? it->path : "/";
-        // Workaround for bug in our gridftp server!
-        if(name[0]) {
-          name_ += "/";
-          name_ += name;
-        }
-        i = it->fnames.insert(it->fnames.end(), FileInfo(name_));
-      }
+      std::list<FileInfo>::iterator i = it->fnames.insert(it->fnames.end(), FileInfo(name));
+
       if (it->facts)
         SetAttributes(*i, attrs);
       if (nlen == length)
@@ -296,10 +325,11 @@ namespace Arc {
     /* if(!callback_active) return; */
     Lister *it = (Lister*)arg;
     if (error != GLOBUS_SUCCESS) {
-      std::string tmp = globus_object_to_string(error);
-      logger.msg(INFO, "Failure: %s", tmp);
+      logger.msg(INFO, "Failure: %s", globus_object_to_string(error));
       globus_mutex_lock(&(it->mutex));
       it->data_callback_status = CALLBACK_ERROR;
+      // if data failed no reason to wait for control
+      it->callback_status = CALLBACK_ERROR;
       globus_cond_signal(&(it->cond));
       globus_mutex_unlock(&(it->mutex));
       return;
@@ -314,6 +344,8 @@ namespace Arc {
       logger.msg(INFO, "Failed reading data");
       globus_mutex_lock(&(it->mutex));
       it->data_callback_status = CALLBACK_ERROR;
+      // if data failed no reason to wait for control
+      it->callback_status = CALLBACK_ERROR;
       globus_cond_signal(&(it->cond));
       globus_mutex_unlock(&(it->mutex));
       return;
@@ -322,13 +354,13 @@ namespace Arc {
 
   globus_ftp_control_response_class_t Lister::send_command(const char *command, const char *arg, bool wait_for_response, char **sresp, int* code, char delim) {
     char *cmd = NULL;
-    if (sresp)
-      (*sresp) = NULL;
+    if (sresp) (*sresp) = NULL;
     if (code) *code = 0;
     if (command) { /* if no command - waiting for second reply */
       globus_mutex_lock(&mutex);
-      for (int i = 0; i < resp_n; i++)
+      for (int i = 0; i < resp_n; i++) {
         globus_ftp_control_response_destroy(resp + i);
+      }
       resp_n = 0;
       callback_status = CALLBACK_NOTREADY;
       globus_mutex_unlock(&mutex);
@@ -351,8 +383,7 @@ namespace Arc {
       if (globus_ftp_control_send_command(handle, cmd, resp_callback, this)
           != GLOBUS_SUCCESS) {
         logger.msg(VERBOSE, "%s failed", command);
-        if (cmd)
-          free(cmd);
+        if (cmd) free(cmd);
         return GLOBUS_FTP_UNKNOWN_REPLY;
       }
       logger.msg(DEBUG, "Command is being sent");
@@ -435,16 +466,19 @@ namespace Arc {
     /* !!!!!!! Memory LOST - cmd !!!!!!!! */
   }
 
-  Lister::Lister(GSSCredential& credential)
+  Lister::Lister()
     : inited(false),
+      facts(true),
       handle(NULL),
       resp_n(0),
       callback_status(CALLBACK_NOTREADY),
+      data_callback_status(CALLBACK_NOTREADY),
       connected(false),
       pasv_set(false),
       data_activated(false),
+      free_format(false),
       port((unsigned short int)(-1)),
-      credential(credential) {
+      credential(NULL) {
     if (globus_cond_init(&cond, GLOBUS_NULL) != GLOBUS_SUCCESS) {
       logger.msg(ERROR, "Failed initing condition");
       return;
@@ -472,68 +506,107 @@ namespace Arc {
     inited = true;
   }
 
-  int Lister::close_connection() {
-    if (!connected)
-      return 0;
-    logger.msg(VERBOSE, "Closing connection");
-    if (globus_ftp_control_quit(handle, resp_callback, this) !=
-        GLOBUS_SUCCESS)
-      if (globus_ftp_control_force_close(handle, resp_callback, this) !=
-          GLOBUS_SUCCESS) {
-        logger.msg(INFO, "Failed to close connection 1");
-        return -1;
-      }
-    if (wait_for_callback() != CALLBACK_DONE) {
-      if (globus_ftp_control_force_close(handle, resp_callback, this) !=
-          GLOBUS_SUCCESS) {
-        logger.msg(INFO, "Failed to close connection 2");
-        return -1;
-      }
-      if (wait_for_callback() != CALLBACK_DONE) {
-        logger.msg(INFO, "Failed to close connection 3");
-        return -1;
-      }
-    }
+  void Lister::close_connection() {
+    if (!connected) return;
     connected = false;
-    logger.msg(VERBOSE, "Closed successfully");
-    return 0;
+    bool res = true;
+    close_callback_status = CALLBACK_NOTREADY;
+    logger.msg(VERBOSE, "Closing connection");
+    if (globus_ftp_control_data_force_close(handle, simple_callback, this) != GLOBUS_SUCCESS) {
+    } else if (wait_for_callback() != CALLBACK_DONE) {
+      res = false;
+    }
+    //if (globus_ftp_control_abort(handle, resp_callback, this) != GLOBUS_SUCCESS) {
+    //} else if (wait_for_callback() != CALLBACK_DONE) {
+    //    res = false;
+    //}
+    if(send_command("ABOR", NULL, true, NULL) == GLOBUS_FTP_UNKNOWN_REPLY) {
+      res = false;
+    }
+    if (globus_ftp_control_quit(handle, resp_callback, this) != GLOBUS_SUCCESS) {
+    } else if (wait_for_callback() != CALLBACK_DONE) {
+      res = false;
+    }
+    if (globus_ftp_control_force_close(handle, close_callback, this) != GLOBUS_SUCCESS) {
+    } else if (wait_for_close_callback() != CALLBACK_DONE) {
+      res = false;
+    }
+    if(res) {
+      logger.msg(VERBOSE, "Closed successfully");
+    } else {
+      logger.msg(VERBOSE, "Closing may have failed");
+    }
   }
 
   Lister::~Lister() {
-    GlobusResult res;
     close_connection();
     if (inited) {
       inited = false;
-      // Waiting for stalled callbacks
-      while (handle && (!(res=globus_ftp_control_handle_destroy(handle)))) {
-        logger.msg(DEBUG, "Failed destroying handle: %s",res.str());
-        globus_abstime_t timeout;
-        GlobusTimeAbstimeSet(timeout,0,100000);
-        logger.msg(VERBOSE, "Looping for (globus_ftp_control_handle_t) to finish all operations");
-        globus_mutex_lock(&mutex);
-        globus_cond_timedwait(&cond, &mutex, &timeout);
-        globus_mutex_unlock(&mutex);
-      }
-      free(handle);
-      handle = NULL;
+      if(handle) {
+        // Waiting for stalled callbacks
+        // If globus_ftp_control_handle_destroy is called with dc_handle
+        // state not GLOBUS_FTP_DATA_STATE_NONE then handle is messed
+        // and next call causes assertion. So here we are waiting for
+        // proper state.
+        bool first_time = true;
+        time_t start_time = time(NULL);
+        globus_mutex_lock(&(handle->cc_handle.mutex));
+        while ((handle->dc_handle.state != GLOBUS_FTP_DATA_STATE_NONE) ||
+               (handle->cc_handle.cc_state != GLOBUS_FTP_CONTROL_UNCONNECTED)) {
+//          if((handle->cc_handle.cc_state == GLOBUS_FTP_CONTROL_UNCONNECTED) &&
+//             (handle->dc_handle.state == GLOBUS_FTP_DATA_STATE_CLOSING)) {
+//            handle->dc_handle.state = GLOBUS_FTP_DATA_STATE_NONE;
+//            break;
+//          };
+          globus_mutex_unlock(&(handle->cc_handle.mutex));
+          if(first_time) {
+            logger.msg(VERBOSE, "Waiting for globus handle to settle");
+            first_time = false;
+          }
+          globus_abstime_t timeout;
+          GlobusTimeAbstimeSet(timeout,0,100000);
+          logger.msg(DEBUG, "Handle is not in proper state %u/%u",handle->cc_handle.cc_state,handle->dc_handle.state);
+          globus_mutex_lock(&mutex);
+          globus_cond_timedwait(&cond, &mutex, &timeout);
+          globus_mutex_unlock(&mutex);
+          globus_mutex_lock(&(handle->cc_handle.mutex));
+          if(((unsigned int)(time(NULL) - start_time)) > 60) {
+            logger.msg(VERBOSE, "Globus handle is stuck");
+            first_time = false;
+            break;
+          }
+        }
+        globus_mutex_unlock(&(handle->cc_handle.mutex));
+        GlobusResult res;
+        if(!(res=globus_ftp_control_handle_destroy(handle))) {
+          // This situation can't be fixed because call to globus_ftp_control_handle_destroy
+          // makes handle unusable even if it fails.
+          logger.msg(DEBUG, "Failed destroying handle: %s. Can't handle such situation.",res.str());
+        } else {
+          free(handle);
+        };
+        handle = NULL;
+      };
       globus_mutex_destroy(&mutex);
       globus_cond_destroy(&cond);
     }
   }
 
-  int Lister::setup_pasv(globus_ftp_control_host_port_t& pasv_addr) {
-    if(pasv_set) return 0;
+  DataStatus Lister::setup_pasv(globus_ftp_control_host_port_t& pasv_addr) {
+    if(pasv_set) return DataStatus::Success;
     char *sresp;
     GlobusResult res;
+    DataStatus result = DataStatus::ListError;
     if (send_command("PASV", NULL, true, &sresp, NULL, '(') !=
         GLOBUS_FTP_POSITIVE_COMPLETION_REPLY) {
       if (sresp) {
         logger.msg(INFO, "PASV failed: %s", sresp);
+        result.SetDesc(sresp);
         free(sresp);
       }
       else
         logger.msg(INFO, "PASV failed");
-      return -1;
+      return result;
     }
     pasv_addr.port = 0;
     if (sresp) {
@@ -546,9 +619,10 @@ namespace Arc {
     }
     if (pasv_addr.port == 0) {
       logger.msg(INFO, "Can't parse host and port in response to PASV");
+      result.SetDesc("Can't parse host and port in response to PASV");
       if (sresp)
         free(sresp);
-      return -1;
+      return result;
     }
     free(sresp);
     logger.msg(VERBOSE, "Data channel: %d.%d.%d.%d %d", pasv_addr.host[0],
@@ -556,8 +630,10 @@ namespace Arc {
                pasv_addr.port);
     if (!(res = globus_ftp_control_local_port(handle, &pasv_addr))) {
       logger.msg(INFO, "Obtained host and address are not acceptable");
-      logger.msg(INFO, "Failure: %s", res.str());
-      return -1;
+      std::string globus_err(res.str());
+      logger.msg(INFO, "Failure: %s", globus_err);
+      result.SetDesc(globus_err);
+      return result;
     }
     /* it looks like _pasv is not enough for connection - start reading
        immediately */
@@ -565,15 +641,17 @@ namespace Arc {
     if (globus_ftp_control_data_connect_read(handle, &list_conn_callback,
                                              this) != GLOBUS_SUCCESS) {
       logger.msg(INFO, "Failed to open data channel");
+      result.SetDesc("Failed to open data channel");
       pasv_set = false;
-      return -1;
+      return result;
     }
     pasv_set = true;
-    return 0;
+    return DataStatus::Success;
   }
 
-  int Lister::handle_connect(const URL& url) {
+  DataStatus Lister::handle_connect(const URL& url) {
     GlobusResult res;
+    DataStatus result = DataStatus::ListError;
     /* get listing */
     fnames.clear();
     globus_ftp_control_auth_info_t auth;
@@ -581,12 +659,13 @@ namespace Arc {
     if ((url.Protocol() != "ftp") &&
         (url.Protocol() != "gsiftp")) {
       logger.msg(ERROR, "Unsupported protocol in url %s", url.str());
-      return -1;
+      result.SetDesc("Unsupported protocol in url " + url.str());
+      return result;
     }
 
     bool reconnect = true;
 
-    if (connected)
+    if (connected) {
       if ((host == url.Host()) &&
           (port == url.Port()) &&
           (scheme == url.Protocol()) &&
@@ -595,9 +674,11 @@ namespace Arc {
         /* same server - check if connection alive */
         logger.msg(VERBOSE, "Reusing connection");
         if (send_command("NOOP", NULL, true, NULL) ==
-            GLOBUS_FTP_POSITIVE_COMPLETION_REPLY)
+            GLOBUS_FTP_POSITIVE_COMPLETION_REPLY) {
           reconnect = false;
+        }
       }
+    }
 
     path = url.Path();
     if ((path.length() != 0) && (path[path.length() - 1] == '/'))
@@ -619,14 +700,15 @@ namespace Arc {
                                              port, &resp_callback, this))) {
         logger.msg(ERROR, "Failed connecting to server %s:%d",
                    host.c_str(), port);
-        logger.msg(ERROR, "Failure: %s", res.str());
-        return -1;
+        result.SetDesc(res.str());
+        return result;
       }
       if (wait_for_callback() != CALLBACK_DONE) {
         logger.msg(ERROR, "Failed to connect to server %s:%d",
                    host.c_str(), port);
+        result.SetDesc("Failed to connect to server");
         resp_destroy();
-        return -1;
+        return result;
       }
       connected = true;
       resp_destroy();
@@ -638,13 +720,19 @@ namespace Arc {
           username_ = default_gsiftp_user;
         if (userpass.empty())
           userpass_ = default_gsiftp_pass;
-        if (globus_ftp_control_auth_info_init(&auth, credential,
+        if (!credential) {
+          logger.msg(ERROR, "Missing authentication information");
+          result.SetDesc("Missing authentication information");
+          return result;
+        }
+        if (globus_ftp_control_auth_info_init(&auth, *credential,
                                               GLOBUS_TRUE, username_,
                                               userpass_, GLOBUS_NULL,
                                               GLOBUS_NULL) !=
             GLOBUS_SUCCESS) {
           logger.msg(ERROR, "Bad authentication information");
-          return -1;
+          result.SetDesc("Bad authentication information");
+          return result;
         }
         use_auth = GLOBUS_TRUE;
       }
@@ -657,30 +745,39 @@ namespace Arc {
                                               GLOBUS_FALSE, username_,
                                               userpass_, GLOBUS_NULL,
                                               GLOBUS_NULL))) {
-          logger.msg(ERROR, "Bad authentication information");
-          logger.msg(ERROR, "Failure: %s", res.str());
-          return -1;
+          std::string globus_err(res.str());
+          logger.msg(ERROR, "Bad authentication information: %s", globus_err);
+          result.SetDesc(globus_err);
+          return result;
         }
         use_auth = GLOBUS_FALSE;
       }
       if (!(res = globus_ftp_control_authenticate(handle, &auth, use_auth,
                                           resp_callback, this))) {
-        logger.msg(ERROR, "Failed authenticating");
-        logger.msg(ERROR, "Failure: %s", res.str());
-        return -1;
+        std::string globus_err(res.str());
+        logger.msg(ERROR, "Failed authenticating: %s", globus_err);
+        result.SetDesc(globus_err);
+        return result;
       }
       if (wait_for_callback() != CALLBACK_DONE) {
         logger.msg(ERROR, "Failed authenticating");
+        result.SetDesc("Failed authenticating");
         resp_destroy();
-        return -1;
+        return result;
       }
       resp_destroy();
+    } else {
+      // Calling NOOP to test connection
     }
-    return 0;
+    return DataStatus::Success;
   }
 
-  int Lister::retrieve_file_info(const URL& url,bool names_only) {
-    if(handle_connect(url) != 0) return -1;
+  DataStatus Lister::retrieve_file_info(const URL& url,bool names_only) {
+
+    DataStatus result = DataStatus::StatError;
+    DataStatus con_result = handle_connect(url);
+    if(!con_result) return con_result;
+
     globus_ftp_control_response_class_t cmd_resp;
     char *sresp;
     if (url.Protocol() == "gsiftp") {
@@ -689,11 +786,12 @@ namespace Arc {
           (cmd_resp != GLOBUS_FTP_PERMANENT_NEGATIVE_COMPLETION_REPLY)) {
         if (sresp) {
           logger.msg(INFO, "DCAU failed: %s", sresp);
+          result.SetDesc(sresp);
           free(sresp);
         }
         else
           logger.msg(INFO, "DCAU failed");
-        return -1;
+        return result;
       }
       free(sresp);
     }
@@ -708,11 +806,12 @@ namespace Arc {
       int code = 0;
       cmd_resp = send_command("MLST", path.c_str(), true, &sresp, &code);
       if (cmd_resp == GLOBUS_FTP_PERMANENT_NEGATIVE_COMPLETION_REPLY) {
-        if((code == 500) || (code == 550)) { // 550 - workaround for old bug
+        if (code == 500) {
           logger.msg(INFO, "MLST is not supported - trying LIST");
           free(sresp);
           /* run NLST */
-          if (setup_pasv(pasv_addr) != 0) return -1;
+          DataStatus pasv_res = setup_pasv(pasv_addr);
+          if (!pasv_res) return pasv_res;
           facts = false;
           free_format = true;
           cmd_resp = send_command("LIST", path.c_str(), true, &sresp);
@@ -724,8 +823,9 @@ namespace Arc {
         // 250 -
         if (cmd_resp != GLOBUS_FTP_POSITIVE_COMPLETION_REPLY) {
           logger.msg(INFO, "Immediate completion expected: %s", sresp);
+          result.SetDesc(sresp);
           free(sresp);
-          return -1;
+          return result;
         }
         // Try to collect full response
         char* nresp = strchr(sresp,'\n');
@@ -736,8 +836,9 @@ namespace Arc {
           cmd_resp = send_command(NULL, NULL, true, &sresp);
           if(cmd_resp != GLOBUS_FTP_UNKNOWN_REPLY) {
             logger.msg(INFO, "Missing information in reply: %s", sresp);
+            result.SetDesc(sresp);
             free(sresp);
-            return -1;
+            return result;
           }
           nresp=sresp;
         }
@@ -749,7 +850,7 @@ namespace Arc {
           *fresp=0;
           list_shift = 0;
           fnames.clear();
-          int nlength = strlen(nresp);
+          size_t nlength = strlen(nresp);
           if(nlength > sizeof(readbuf)) nlength=sizeof(readbuf);
           memcpy(readbuf,nresp,nlength);
           data_activated = true;
@@ -763,16 +864,18 @@ namespace Arc {
           cmd_resp = send_command(NULL, NULL, true, &sresp);
           if(cmd_resp != GLOBUS_FTP_POSITIVE_COMPLETION_REPLY) {
             logger.msg(INFO, "Missing final reply: %s", sresp);
+            result.SetDesc(sresp);
             free(sresp);
-            return -1;
+            return result;
           }
           fresp=sresp;
         }
         free(sresp);
-        return 0; 
+        return DataStatus::Success;
       }
     } else {
-      if (setup_pasv(pasv_addr) != 0) return -1;
+      DataStatus pasv_res = setup_pasv(pasv_addr);
+      if (!pasv_res) return pasv_res;
       facts = false;
       free_format = true;
       cmd_resp = send_command("LIST", path.c_str(), true, &sresp);
@@ -781,25 +884,31 @@ namespace Arc {
       /* completion is not expected here */
       pasv_set = false;
       logger.msg(INFO, "Unexpected immediate completion: %s", sresp);
+      result.SetDesc(sresp);
       if (sresp) free(sresp);
-      return -1;
+      return result;
     }
     if ((cmd_resp != GLOBUS_FTP_POSITIVE_PRELIMINARY_REPLY) &&
         (cmd_resp != GLOBUS_FTP_POSITIVE_INTERMEDIATE_REPLY)) {
       if (sresp) {
         logger.msg(INFO, "LIST/MLST failed: %s", sresp);
+        result.SetDesc(sresp);
         free(sresp);
       }
       else
         logger.msg(INFO, "LIST/MLST failed");
-      return -1;
+      return result;
     }
     free(sresp);
     return transfer_list();
   }
 
-  int Lister::retrieve_dir_info(const URL& url,bool names_only) {
-    if(handle_connect(url) != 0) return -1;
+  DataStatus Lister::retrieve_dir_info(const URL& url,bool names_only) {
+
+    DataStatus result = DataStatus::StatError;
+    DataStatus con_result = handle_connect(url);
+    if(!con_result) return con_result;
+
     globus_ftp_control_response_class_t cmd_resp;
     char *sresp = NULL;
     if (url.Protocol() == "gsiftp") {
@@ -808,11 +917,12 @@ namespace Arc {
           (cmd_resp != GLOBUS_FTP_PERMANENT_NEGATIVE_COMPLETION_REPLY)) {
         if (sresp) {
           logger.msg(INFO, "DCAU failed: %s", sresp);
+          result.SetDesc(sresp);
           free(sresp);
         }
         else
           logger.msg(INFO, "DCAU failed");
-        return -1;
+        return result;
       }
       free(sresp);
     }
@@ -822,9 +932,9 @@ namespace Arc {
     globus_ftp_control_host_port_t pasv_addr;
     facts = true;
     free_format = false;
-    if (setup_pasv(pasv_addr) != 0)
-      return -1;
-    if(!names_only) {
+    DataStatus pasv_res = setup_pasv(pasv_addr);
+    if (!pasv_res) return pasv_res;
+    if (!names_only) {
       /* try MLSD */
       int code = 0;
       cmd_resp = send_command("MLSD", path.c_str(), true, &sresp, &code);
@@ -845,59 +955,62 @@ namespace Arc {
       /* completion is not expected here */
       pasv_set = false;
       logger.msg(INFO, "Immediate completion: %s", sresp);
+      result.SetDesc(sresp);
       if (sresp)
         free(sresp);
-      return -1;
+      return result;
     }
     if ((cmd_resp != GLOBUS_FTP_POSITIVE_PRELIMINARY_REPLY) &&
         (cmd_resp != GLOBUS_FTP_POSITIVE_INTERMEDIATE_REPLY)) {
       if (sresp) {
         logger.msg(INFO, "NLST/MLSD failed: %s", sresp);
+        result.SetDesc(sresp);
         free(sresp);
       }
       else
         logger.msg(INFO, "NLST/MLSD failed");
-      return -1;
+      return result;
     }
     free(sresp);
     return transfer_list();
   }
 
-  int Lister::transfer_list(void) {
+  DataStatus Lister::transfer_list(void) {
+    DataStatus result = DataStatus::ListError;
     globus_ftp_control_response_class_t cmd_resp;
     char* sresp = NULL;
     /* start transfer */
     for (;;) {
       /* waiting for response received */
       cmd_resp = send_command(NULL, NULL, true, &sresp);
-      if (cmd_resp == GLOBUS_FTP_POSITIVE_COMPLETION_REPLY)
-        break;
+      if (cmd_resp == GLOBUS_FTP_POSITIVE_COMPLETION_REPLY) break;
       if ((cmd_resp != GLOBUS_FTP_POSITIVE_PRELIMINARY_REPLY) &&
           (cmd_resp != GLOBUS_FTP_POSITIVE_INTERMEDIATE_REPLY)) {
         if (sresp) {
           logger.msg(INFO, "Data transfer aborted: %s", sresp);
+          result.SetDesc(sresp);
           free(sresp);
         }
-        else
+        else {
           logger.msg(INFO, "Data transfer aborted");
+        }
         // Destroy data connections here ?????????
         pasv_set = false;
-        return -1;
+        return result;
       }
-      if (sresp)
-        free(sresp);
+      if (sresp) free(sresp);
     }
-    if (sresp)
-      free(sresp);
+    if (sresp) free(sresp);
     /* waiting for data ended */
     if (wait_for_data_callback() != CALLBACK_DONE) {
       logger.msg(INFO, "Failed to transfer data");
+      result.SetDesc("Failed to obtain data");
       pasv_set = false;
-      return -1;
+      return result;
     }
     pasv_set = false;
     /* success */
-    return 0;
+    return DataStatus::Success;
   }
 
 } // namespace Arc
