@@ -18,10 +18,6 @@
 #include <arc/message/PayloadRaw.h>
 #include <arc/client/ClientInterface.h>
 
-#ifdef WIN32
-#include <arc/win32.h>
-#endif
-
 #include "DataPointHTTP.h"
 
 namespace Arc {
@@ -329,6 +325,7 @@ namespace Arc {
           if (!url_end)
             return false; // Broken HTML
           std::string url(url_start, url_end - url_start);
+          url = uri_unencode(url);
           if (url.find("://") != std::string::npos) {
             URL u(url);
             std::string b = base.str();
@@ -400,7 +397,7 @@ namespace Arc {
     usercfg.ApplyToConfig(cfg);
     ClientHTTP client(cfg, url, usercfg.Timeout());
 
-    DataStatus r = do_stat(url.FullPath(), client, file);
+    DataStatus r = do_stat(url.FullPathURIEncoded(), client, file);
     if(!r) return r;
     std::string name = url.FullPath();
     std::string::size_type p = name.rfind('/');
@@ -433,7 +430,7 @@ namespace Arc {
     {
       ClientHTTP client(cfg, curl, usercfg.Timeout());
       FileInfo file;
-      DataStatus r = do_stat(curl.FullPath(), client, file);
+      DataStatus r = do_stat(curl.FullPathURIEncoded(), client, file);
       if(r) {
         if(file.CheckSize()) size = file.GetSize();
         if(file.CheckCreated()) created = file.GetCreated();
@@ -500,7 +497,7 @@ namespace Arc {
           }
           */
           ClientHTTP client(cfg, curl, usercfg.Timeout());
-          do_stat(furl.FullPath(), client, *f);
+          do_stat(furl.FullPathURIEncoded(), client, *f);
         }
       }
     }
@@ -609,7 +606,7 @@ namespace Arc {
     PayloadRaw request;
     PayloadRawInterface *inbuf;
     HTTPClientInfo info;
-    MCC_Status r = client.process("GET", url.FullPath(), 0, 15,
+    MCC_Status r = client.process("GET", url.FullPathURIEncoded(), 0, 15,
                                   &request, &info, &inbuf);
     PayloadRawInterface::Size_t logsize = 0;
     if (inbuf){
@@ -637,6 +634,7 @@ namespace Arc {
     URL client_url = point.url;
     bool transfer_failure = false;
     int retries = 0;
+    std::string path = point.CurrentLocation().FullPathURIEncoded();
     for (;;) {
       unsigned int transfer_size = 0;
       int transfer_handle = -1;
@@ -654,7 +652,6 @@ namespace Arc {
       HTTPClientInfo transfer_info;
       PayloadRaw request;
       PayloadRawInterface *inbuf;
-      std::string path = point.CurrentLocation().FullPath();
       MCC_Status r = client->process("GET", path, transfer_offset,
                                      transfer_end, &request, &transfer_info,
                                      &inbuf);
@@ -708,6 +705,7 @@ namespace Arc {
             (client_url.Protocol() == "https") ||
             (client_url.Protocol() == "httpg"))) {
           client = new ClientHTTP(cfg, client_url, point.usercfg.Timeout());
+          path = client_url.FullPathURIEncoded();
           continue;
         }
         transfer_failure = true;
@@ -805,6 +803,7 @@ namespace Arc {
     ClientHTTP *client = info.client;
     bool transfer_failure = false;
     int retries = 0;
+    std::string path = point.CurrentLocation().FullPathURIEncoded();
     for (;;) {
       unsigned int transfer_size = 0;
       int transfer_handle = -1;
@@ -822,7 +821,6 @@ namespace Arc {
                               transfer_offset, transfer_size,
                               point.CheckSize() ? point.GetSize() : 0);
       PayloadRawInterface *response;
-      std::string path = point.CurrentLocation().FullPath();
       MCC_Status r = client->process("PUT", path, &request, &transfer_info,
                                      &response);
       if (response)
@@ -873,7 +871,6 @@ namespace Arc {
           HTTPClientInfo transfer_info;
           PayloadMemConst request(NULL, 0, 0, 0);
           PayloadRawInterface *response;
-          std::string path = point.CurrentLocation().FullPath();
           MCC_Status r = client->process("PUT", path, &request, &transfer_info,
                                          &response);
           if (response)
@@ -922,6 +919,6 @@ namespace Arc {
 } // namespace Arc
 
 Arc::PluginDescriptor PLUGINS_TABLE_NAME[] = {
-  { "http", "HED:DMC", 0, &Arc::DataPointHTTP::Instance },
-  { NULL, NULL, 0, NULL }
+  { "http", "HED:DMC", "HTTP or HTTP over SSL (https)", 0, &Arc::DataPointHTTP::Instance },
+  { NULL, NULL, NULL, 0, NULL }
 };
