@@ -161,36 +161,26 @@ namespace Arc {
       if(r == DataStatus::ListError) r = DataStatus::CheckError;
       return r;
     }
-    if(files.size() == 0) return DataStatus::CheckError;
+    if(files.empty()) return DataStatus::CheckError;
     return DataStatus::Success;
   }
 
-  /* perform resolve operation, which can take long time */
   DataStatus DataPointLFC::Resolve(bool source) {
     int lfc_r;
 
-    LFCLOCKINT(lfc_r,lfc_startsess(const_cast<char*>(url.Host().c_str()),
-                      const_cast<char*>("ARC")), url);
-    if(lfc_r != 0) {
-      logger.msg(ERROR, "Error starting session: %s", sstrerror(serrno));
-      lfc_endsess();
-      if (serrno == SECOMERR || serrno == ENSNACT || serrno == SETIMEDOUT)
-        return source ? DataStatus::ReadResolveErrorRetryable : DataStatus::WriteResolveErrorRetryable;
-      return source ? DataStatus::ReadResolveError : DataStatus::WriteResolveError;
-    }
+    // NOTE: Sessions are not used in Resolve(), because under heavy load in A-REX
+    // it can end up using all the LFC server threads. See bug 2576 for more info.
 
     std::string path = url.Path();
 
     if (source || path.empty() || path == "/") {
       path = ResolveGUIDToLFN();
       if (path.empty()) {
-        lfc_endsess();
         return source ? DataStatus::ReadResolveError : DataStatus::WriteResolveError;
       }
     }
-    if (!source && url.Locations().size() == 0) {
+    if (!source && url.Locations().size() == 0 && !HaveLocations()) {
       logger.msg(ERROR, "Locations are missing in destination LFC URL");
-      lfc_endsess();
       return DataStatus::WriteResolveError;
     }
 
@@ -202,7 +192,6 @@ namespace Arc {
     if(lfc_r != 0) {
       if (source || ((serrno != ENOENT) && (serrno != ENOTDIR))) {
         logger.msg(ERROR, "Error finding replicas: %s", sstrerror(serrno));
-        lfc_endsess();
         return source ? DataStatus::ReadResolveError : DataStatus::WriteResolveError;
       }
       nbentries = 0;
@@ -265,7 +254,6 @@ namespace Arc {
     LFCLOCKINT(lfc_r,lfc_statg(path.c_str(), NULL, &st), url);
     if(lfc_r == 0) {
       if (st.filemode & S_IFDIR) { // directory
-        lfc_endsess();
         return DataStatus::Success;
       }
       registered = true;
@@ -283,7 +271,6 @@ namespace Arc {
       }
       guid = st.guid;
     }
-    lfc_endsess();
     if (!HaveLocations()) {
       logger.msg(ERROR, "No locations found for %s", url.str());
       return source ? DataStatus::ReadResolveError : DataStatus::WriteResolveError;
@@ -317,7 +304,7 @@ namespace Arc {
                       const_cast<char*>("ARC")), url);
     if(lfc_r != 0) {
       logger.msg(ERROR, "Error starting session: %s", sstrerror(serrno));
-      if (serrno == SECOMERR || serrno == ENSNACT || serrno == SETIMEDOUT)
+      if (IsTempError())
         return DataStatus::PreRegisterErrorRetryable;      
       return DataStatus::PreRegisterError;
     }
@@ -434,7 +421,7 @@ namespace Arc {
     LFCLOCKINT(lfc_r,lfc_startsess(const_cast<char*>(url.Host().c_str()), const_cast<char*>("ARC")), url);
     if(lfc_r != 0) {
       logger.msg(ERROR, "Error starting session: %s", sstrerror(serrno));
-      if (serrno == SECOMERR || serrno == ENSNACT || serrno == SETIMEDOUT)
+      if (IsTempError())
         return DataStatus::PostRegisterErrorRetryable;
       return DataStatus::PostRegisterError;
     }
@@ -492,7 +479,7 @@ namespace Arc {
                       const_cast<char*>("ARC")), url);
     if(lfc_r != 0) {
       logger.msg(ERROR, "Error starting session: %s", sstrerror(serrno));
-      if (serrno == SECOMERR || serrno == ENSNACT || serrno == SETIMEDOUT)
+      if (IsTempError())
         return DataStatus::UnregisterErrorRetryable;
       return DataStatus::UnregisterError;
     }
@@ -523,7 +510,7 @@ namespace Arc {
                       const_cast<char*>("ARC")), url);
     if(lfc_r != 0) {
       logger.msg(ERROR, "Error starting session: %s", sstrerror(serrno));
-      if (serrno == SECOMERR || serrno == ENSNACT || serrno == SETIMEDOUT)
+      if (IsTempError())
         return DataStatus::UnregisterErrorRetryable;
       return DataStatus::UnregisterError;
     }
@@ -616,7 +603,7 @@ namespace Arc {
                       const_cast<char*>("ARC")), url);
     if(lfc_r != 0) {
       logger.msg(ERROR, "Error starting session: %s", sstrerror(serrno));
-      if (serrno == SECOMERR || serrno == ENSNACT || serrno == SETIMEDOUT)
+      if (IsTempError())
         return DataStatus::ListErrorRetryable;
       return DataStatus::ListError;
     }
@@ -794,12 +781,16 @@ namespace Arc {
   std::string DataPointLFC::ResolveGUIDToLFN() {
 
     // check if guid is already defined
-    if (!guid.empty())
+    if (!guid.empty()) {
+      if (path_for_guid.empty()) return "/";
       return path_for_guid;
+    }
 
     // check for guid in the attributes
-    if (url.MetaDataOption("guid").empty())
+    if (url.MetaDataOption("guid").empty()) {
+      if (url.Path().empty()) return "/";
       return url.Path();
+    }
 
     guid = url.MetaDataOption("guid");
 
@@ -823,6 +814,7 @@ namespace Arc {
       lfc_listlinks(NULL, (char*)guid.c_str(), CNS_LIST_END, &listp);
     }
 
+    if (path_for_guid.empty()) return "/";
     return path_for_guid;
   }
 
@@ -838,9 +830,22 @@ namespace Arc {
     return tmp;
   }
 
+  bool DataPointLFC::IsTempError() const {
+    // err codes in includepath/lfc/serrno.h
+    if (serrno == SECOMERR ||     // communication error
+        serrno == ENSNACT ||      // name server not active
+        serrno == SETIMEDOUT ||   // timeout
+        serrno == SEINTERNAL ||   // internal error
+        serrno == SECONNDROP ||   // connection dropped by server
+        serrno == SEWOULDBLOCK || // temporarily unavailable
+        serrno == SESYSERR        // system error
+        ) return true;
+    return false;
+  }
+
 } // namespace Arc
 
 Arc::PluginDescriptor PLUGINS_TABLE_NAME[] = {
-  { "lfc", "HED:DMC", 0, &Arc::DataPointLFC::Instance },
-  { NULL, NULL, 0, NULL }
+  { "lfc", "HED:DMC", "LCG File Catalog", 0, &Arc::DataPointLFC::Instance },
+  { NULL, NULL, NULL, 0, NULL }
 };
