@@ -5,7 +5,6 @@
 #endif
 
 #ifdef WIN32 
-#include <arc/win32.h>
 #include <fcntl.h>
 #endif
 
@@ -229,7 +228,7 @@ namespace Arc {
       if (source)
         registered = true;
     }
-    if (url.Locations().size() == 0) {
+    if (url.Locations().size() == 0 && !HaveLocations()) {
       for (globus_list_t *lp = pfns_list; lp; lp = globus_list_rest(lp)) {
         globus_rls_string2_t *str2 =
           (globus_rls_string2_t*)globus_list_first(lp);
@@ -385,7 +384,7 @@ namespace Arc {
       std::list<URL> lrcs;
       rlis.push_back(url.ConnectionURL());
       lrcs.push_back(url.ConnectionURL());
-      if (url.Locations().size() == 0)
+      if (url.Locations().size() == 0 && !HaveLocations())
         logger.msg(INFO, "Locations are missing in destination RLS url - "
                    "will use those registered with special name");
       meta_resolve_rls_t arg(*this, source);
@@ -720,7 +719,6 @@ namespace Arc {
     std::string& guid(((meta_unregister_rls_t*)arg)->guid);
 
     int lrc_offset = 0;
-    int lrc_limit = 0;
     globus_result_t err;
     int errcode;
     char errmsg[MAXERRMSG + 32];
@@ -754,6 +752,7 @@ namespace Arc {
       globus_rls_client_free_list(guids);
     }
     if (all) {
+      int lrc_limit = 0;
       if (!guid.empty())
         err = globus_rls_client_lrc_get_pfn
                 (h, const_cast<char*>(guid.c_str()),
@@ -1216,7 +1215,7 @@ namespace Arc {
     list_files_rls_t arg(*this, files, (verb | INFO_TYPE_NAME) != INFO_TYPE_NAME);
     rls_find_lrcs(rlis, lrcs, true, false, usercfg,
                   &list_files_callback, (void*)&arg);
-    if ((files.size() > 0) && (arg.success.Passed())) {
+    if (!files.empty() && arg.success.Passed()) {
       file = files.front();
       // set some metadata
       if (file.CheckSize()) SetSize(file.GetSize());
@@ -1235,6 +1234,6 @@ namespace Arc {
 } // namespace Arc
 
 Arc::PluginDescriptor PLUGINS_TABLE_NAME[] = {
-  { "rls", "HED:DMC", 0, &Arc::DataPointRLS::Instance },
-  { NULL, NULL, 0, NULL }
+  { "rls", "HED:DMC", "Globus Replica Location Service", 0, &Arc::DataPointRLS::Instance },
+  { NULL, NULL, NULL, 0, NULL }
 };
