@@ -4,10 +4,6 @@
 #include <config.h>
 #endif
 
-#ifdef WIN32
-#include <arc/win32.h>
-#endif
-
 #include <cstdlib>
 
 #include <glibmm/fileutils.h>
@@ -19,7 +15,7 @@
 #include <arc/UserConfig.h>
 #include <arc/data/DataBuffer.h>
 #include <arc/data/DataCallback.h>
-#include <arc/data/CheckSum.h>
+#include <arc/CheckSum.h>
 
 #include "DataPointSRM.h"
 
@@ -35,6 +31,7 @@ namespace Arc {
       writing(false) {
     valid_url_options.push_back("protocol");
     valid_url_options.push_back("spacetoken");
+    valid_url_options.push_back("transferprotocol");
   }
 
   DataPointSRM::~DataPointSRM() {
@@ -61,10 +58,13 @@ namespace Arc {
       return DataStatus::CheckError;
     }
 
-    SRMClientRequest srm_request_tmp(
-      (url.HTTPOption("SFN", "") == "")?
-      url.str():(url.Protocol() + "://" + url.Host() + "/" + url.HTTPOption("SFN"))
-    );
+    std::string canonic_url;
+    if (!url.HTTPOption("SFN").empty())
+      canonic_url = url.Protocol() + "://" + url.Host() + "/" + Arc::uri_encode(url.HTTPOption("SFN"), false);
+    else
+      canonic_url = url.Protocol() + "://" + url.Host() + url.FullPathURIEncoded();
+
+    SRMClientRequest srm_request_tmp(canonic_url);
     
     // first check permissions
     SRMReturnCode res = client->checkPermissions(srm_request_tmp);
@@ -124,12 +124,12 @@ namespace Arc {
       return DataStatus::DeleteError;
     }
 
-    // take out options in srm url
+    // take out options in srm url and encode path
     std::string canonic_url;
     if (!url.HTTPOption("SFN").empty())
-      canonic_url = url.Protocol() + "://" + url.Host() + "/" + url.HTTPOption("SFN");
+      canonic_url = url.Protocol() + "://" + url.Host() + "/" + Arc::uri_encode(url.HTTPOption("SFN"), false);
     else
-      canonic_url = url.Protocol() + "://" + url.Host() + url.Path();
+      canonic_url = url.Protocol() + "://" + url.Host() + url.FullPathURIEncoded();
 
     SRMClientRequest srm_request_tmp(canonic_url);
 
@@ -158,6 +158,10 @@ namespace Arc {
     SRMReturnCode res;
     bool timedout;
 
+    // choose transfer procotols
+    std::list<std::string> transport_protocols;
+    ChooseTransferProtocols(transport_protocols);
+
     // If the file is NEARLINE (on tape) bringOnline is called
     // Whether or not to do this should eventually be specified by the user
     if (access_latency == ACCESS_LATENCY_LARGE) {
@@ -186,12 +190,12 @@ namespace Arc {
           return DataStatus::ReadPrepareError;
         }
 
-        // take out options in srm url
+        // take out options in srm url and encode path
         std::string canonic_url;
         if (!url.HTTPOption("SFN").empty())
-          canonic_url = url.Protocol() + "://" + url.Host() + "/" + url.HTTPOption("SFN");
+          canonic_url = url.Protocol() + "://" + url.Host() + "/" + Arc::uri_encode(url.HTTPOption("SFN"), false);
         else
-          canonic_url = url.Protocol() + "://" + url.Host() + url.Path();
+          canonic_url = url.Protocol() + "://" + url.Host() + url.FullPathURIEncoded();
 
         delete srm_request;
         srm_request = new SRMClientRequest(canonic_url);
@@ -248,16 +252,24 @@ namespace Arc {
         return DataStatus::ReadPrepareError;
       }
 
-      // take out options in srm url
+      // take out options in srm url and encode path
       std::string canonic_url;
       if (!url.HTTPOption("SFN").empty())
-        canonic_url = url.Protocol() + "://" + url.Host() + "/" + url.HTTPOption("SFN");
+        canonic_url = url.Protocol() + "://" + url.Host() + "/" + Arc::uri_encode(url.HTTPOption("SFN"), false);
       else
-        canonic_url = url.Protocol() + "://" + url.Host() + url.Path();
+        canonic_url = url.Protocol() + "://" + url.Host() + url.FullPathURIEncoded();
 
       delete srm_request;
+
+      CheckProtocols(transport_protocols);
+      if (transport_protocols.empty()) {
+        logger.msg(ERROR, "None of the requested transport protocols are supported");
+        delete client;
+        return DataStatus::ReadPrepareError;
+      }
       srm_request = new SRMClientRequest(canonic_url);
       srm_request->request_timeout(stage_timeout);
+      srm_request->transport_protocols(transport_protocols);
       res = client->getTURLs(*srm_request, transport_urls);
       delete client;
     }
@@ -295,6 +307,7 @@ namespace Arc {
 
       if (turls.empty()) {
         logger.msg(ERROR, "SRM returned no useful Transfer URLs: %s", url.str());
+        srm_request->finished_abort();
         return DataStatus::ReadPrepareError;
       }
     }
@@ -354,8 +367,8 @@ namespace Arc {
     StopReading();
     reading = false;
 
-    bool timedout;
     if (srm_request) {
+      bool timedout;
       SRMClient *client = SRMClient::getInstance(usercfg, url.fullstr(), timedout);
       // if the request finished with an error there is no need to abort or release request
       if (client && (srm_request->status() != SRM_REQUEST_FINISHED_ERROR)) {
@@ -385,6 +398,10 @@ namespace Arc {
     SRMReturnCode res;
     bool timedout;
 
+    // choose transfer procotols
+    std::list<std::string> transport_protocols;
+    ChooseTransferProtocols(transport_protocols);
+
     // If a request already exists, query status
     if (srm_request) {
       if (srm_request->status() != SRM_REQUEST_ONGOING) {
@@ -408,14 +425,22 @@ namespace Arc {
         return DataStatus::WritePrepareError;
       }
 
-      // take out options in srm url
+      // take out options in srm url and encode path
       std::string canonic_url;
       if (!url.HTTPOption("SFN").empty())
-        canonic_url = url.Protocol() + "://" + url.Host() + "/" + url.HTTPOption("SFN");
+        canonic_url = url.Protocol() + "://" + url.Host() + "/" + Arc::uri_encode(url.HTTPOption("SFN"), false);
       else
-        canonic_url = url.Protocol() + "://" + url.Host() + url.Path();
+        canonic_url = url.Protocol() + "://" + url.Host() + url.FullPathURIEncoded();
 
       delete srm_request;
+
+      CheckProtocols(transport_protocols);
+      if (transport_protocols.empty()) {
+        logger.msg(ERROR, "None of the requested transport protocols are supported");
+        delete client;
+        return DataStatus::WritePrepareError;
+      }
+
       srm_request = new SRMClientRequest(canonic_url);
       // set space token
       std::string space_token = url.Option("spacetoken");
@@ -451,6 +476,7 @@ namespace Arc {
       }
       srm_request->request_timeout(stage_timeout);
       if (CheckSize()) srm_request->total_size(GetSize());
+      srm_request->transport_protocols(transport_protocols);
       res = client->putTURLs(*srm_request, transport_urls);
       delete client;
     }
@@ -489,6 +515,7 @@ namespace Arc {
 
       if (turls.empty()) {
         logger.msg(ERROR, "SRM returned no useful Transfer URLs: %s", url.str());
+        srm_request->finished_abort();
         return DataStatus::WritePrepareError;
       }
     }
@@ -537,6 +564,8 @@ namespace Arc {
     DataStatus r = DataStatus::Success;
     if (r_handle) {
       r = (*r_handle)->StopWriting();
+      // check if checksum was verified at lower level
+      if ((*r_handle)->CheckCheckSum()) SetCheckSum((*r_handle)->GetCheckSum());
       delete r_handle;
       r_handle = NULL;
     }
@@ -549,10 +578,10 @@ namespace Arc {
     writing = false;
 
     DataStatus r = DataStatus::Success;
-    bool timedout;
 
     // if the request finished with an error there is no need to abort or release request
     if (srm_request) {
+      bool timedout;
       SRMClient * client = SRMClient::getInstance(usercfg, url.fullstr(), timedout);
       if (client && (srm_request->status() != SRM_REQUEST_FINISHED_ERROR)) {
         // call abort if failure, or releasePut on success
@@ -561,42 +590,46 @@ namespace Arc {
            client->remove(*srm_request);
         }
         else {
-          // checksum verification - calculated or supplied checksum will be
-          // stored in DataPoint checksum object
-          if (srm_request->status() == SRM_REQUEST_FINISHED_SUCCESS && additional_checks) {
-            std::string csum = GetCheckSum();
-            if (!csum.empty() && csum.find(':') != std::string::npos) {
-              // get checksum info for checksum verification
-              logger.msg(VERBOSE, "FinishWriting: looking for metadata: %s", url.str());
-              // create a new request
-              SRMClientRequest list_request(srm_request->surls());
-              list_request.long_list(true);
-              std::list<struct SRMFileMetaData> metadata;
-              SRMReturnCode res = client->info(list_request,metadata);
-              if (res != SRM_OK) {
-                client->abort(*srm_request); // if we can't list then we can't remove either
-                delete client;
-                delete srm_request;
-                srm_request = NULL;
-                if (res == SRM_ERROR_TEMPORARY) return DataStatus::WriteFinishErrorRetryable;
-                return DataStatus::WriteFinishError;
-              }
-              if (!metadata.empty()) {
-                if (metadata.front().checkSumValue.length() > 0 &&
-                   metadata.front().checkSumType.length() > 0) {
-                  std::string servercsum(metadata.front().checkSumType+":"+metadata.front().checkSumValue);
-                  logger.msg(INFO, "FinishWriting: obtained checksum: %s", servercsum);
-                  if (csum.substr(0, csum.find(':')) == metadata.front().checkSumType) {
-                    if (csum.substr(csum.find(':')+1) == metadata.front().checkSumValue) {
-                      logger.msg(INFO, "Calculated/supplied transfer checksum %s matches checksum reported by SRM destination %s", csum, servercsum);
-                    }
-                    else {
-                      logger.msg(ERROR, "Checksum mismatch between calculated/supplied checksum (%s) and checksum reported by SRM destination (%s)", csum, servercsum);
-                      r = DataStatus::WriteFinishErrorRetryable;
-                    }
-                  } else logger.msg(WARNING, "Checksum type of SRM (%s) and calculated/supplied checksum (%s) differ, cannot compare", servercsum, csum);
+          // checksum verification - if requested and not already done at lower level
+          if (srm_request->status() == SRM_REQUEST_FINISHED_SUCCESS && additional_checks && !CheckCheckSum()) {
+            const CheckSum * calc_sum = buffer->checksum_object();
+            if (calc_sum && *calc_sum && buffer->checksum_valid()) {
+              char buf[100];
+              calc_sum->print(buf,100);
+              std::string csum(buf);
+              if (!csum.empty() && csum.find(':') != std::string::npos) {
+                // get checksum info for checksum verification
+                logger.msg(VERBOSE, "FinishWriting: looking for metadata: %s", url.str());
+                // create a new request
+                SRMClientRequest list_request(srm_request->surls());
+                list_request.long_list(true);
+                std::list<struct SRMFileMetaData> metadata;
+                SRMReturnCode res = client->info(list_request,metadata);
+                if (res != SRM_OK) {
+                  client->abort(*srm_request); // if we can't list then we can't remove either
+                  delete client;
+                  delete srm_request;
+                  srm_request = NULL;
+                  if (res == SRM_ERROR_TEMPORARY) return DataStatus::WriteFinishErrorRetryable;
+                  return DataStatus::WriteFinishError;
+                }
+                if (!metadata.empty()) {
+                  if (metadata.front().checkSumValue.length() > 0 &&
+                     metadata.front().checkSumType.length() > 0) {
+                    std::string servercsum(metadata.front().checkSumType+":"+metadata.front().checkSumValue);
+                    logger.msg(INFO, "FinishWriting: obtained checksum: %s", servercsum);
+                    if (csum.substr(0, csum.find(':')) == metadata.front().checkSumType) {
+                      if (csum.substr(csum.find(':')+1) == metadata.front().checkSumValue) {
+                        logger.msg(INFO, "Calculated/supplied transfer checksum %s matches checksum reported by SRM destination %s", csum, servercsum);
+                      }
+                      else {
+                        logger.msg(ERROR, "Checksum mismatch between calculated/supplied checksum (%s) and checksum reported by SRM destination (%s)", csum, servercsum);
+                        r = DataStatus::WriteFinishErrorRetryable;
+                      }
+                    } else logger.msg(WARNING, "Checksum type of SRM (%s) and calculated/supplied checksum (%s) differ, cannot compare", servercsum, csum);
+                  } else logger.msg(WARNING, "No checksum information from server");
                 } else logger.msg(WARNING, "No checksum information from server");
-              } else logger.msg(WARNING, "No checksum information from server");
+              } else logger.msg(INFO, "No checksum verification possible");
             } else logger.msg(INFO, "No checksum verification possible");
           }
           if (r.Passed()) {
@@ -648,9 +681,9 @@ namespace Arc {
     std::string sfn_path = url.HTTPOption("SFN");
     if (!sfn_path.empty()) {
       while (sfn_path[0] == '/') sfn_path.erase(0,1);
-      canonic_url = url.Protocol() + "://" + url.Host() + "/" + sfn_path;
+      canonic_url = url.Protocol() + "://" + url.Host() + "/" + Arc::uri_encode(sfn_path, false);
     } else {
-      canonic_url = url.Protocol() + "://" + url.Host() + url.Path();
+      canonic_url = url.Protocol() + "://" + url.Host() + url.FullPathURIEncoded();
     }
 
     SRMClientRequest srm_request_tmp(canonic_url);
@@ -756,7 +789,7 @@ namespace Arc {
     return std::string("adler32");
   }
 
-  bool DataPointSRM::ProvidesMeta() {
+  bool DataPointSRM::ProvidesMeta() const {
     return true;
   }
 
@@ -768,9 +801,38 @@ namespace Arc {
     return turls;
   }
 
+  void DataPointSRM::CheckProtocols(std::list<std::string>& transport_protocols) {
+    for (std::list<std::string>::iterator protocol = transport_protocols.begin();
+         protocol != transport_protocols.end();) {
+      // try to load plugins
+      URL url(*protocol+"://host/path");
+      DataHandle handle(url, usercfg);
+      if (handle) {
+        ++protocol;
+      } else {
+        logger.msg(WARNING, "plugin for transport protocol %s is not installed", *protocol);
+        protocol = transport_protocols.erase(protocol);
+      }
+    }
+  }
+
+  void DataPointSRM::ChooseTransferProtocols(std::list<std::string>& transport_protocols) {
+
+    std::string option_protocols(url.Option("transferprotocol"));
+    if (option_protocols.empty()) {
+      transport_protocols.push_back("gsiftp");
+      transport_protocols.push_back("http");
+      transport_protocols.push_back("https");
+      transport_protocols.push_back("httpg");
+      transport_protocols.push_back("ftp");
+    } else {
+      Arc::tokenize(option_protocols, transport_protocols, ",");
+    }
+  }
+
 } // namespace Arc
 
 Arc::PluginDescriptor PLUGINS_TABLE_NAME[] = {
-  { "srm", "HED:DMC", 0, &Arc::DataPointSRM::Instance },
-  { NULL, NULL, 0, NULL }
+  { "srm", "HED:DMC", "Storage Resource Manager", 0, &Arc::DataPointSRM::Instance },
+  { NULL, NULL, NULL, 0, NULL }
 };
