@@ -13,6 +13,15 @@
 
 namespace Arc {
 
+  /**
+   * The Storage Resource Manager (SRM) protocol allows access to data
+   * distributed across physical storage through a unified namespace
+   * and management interface. PrepareReading() or PrepareWriting() must
+   * be used before reading or writing a physical file.
+   *
+   * This class is a loadable module and cannot be used directly. The DataHandle
+   * class loads modules at runtime and should be used instead of this.
+   */
   class DataPointSRM
     : public DataPointDirect {
   public:
@@ -35,7 +44,7 @@ namespace Arc {
     DataStatus Stat(FileInfo& file, DataPointInfoType verb = INFO_TYPE_ALL);
     DataStatus List(std::list<FileInfo>& files, DataPointInfoType verb = INFO_TYPE_ALL);
     virtual const std::string DefaultCheckSum() const;
-    virtual bool ProvidesMeta();
+    virtual bool ProvidesMeta() const;
     virtual bool IsStageable() const;
     virtual std::vector<URL> TransferLocations() const;
   private:
@@ -47,6 +56,10 @@ namespace Arc {
     bool reading;
     bool writing;
     DataStatus ListFiles(std::list<FileInfo>& files, DataPointInfoType verb, int recursion);
+    /** Check protocols given in list can be used, and if not remove them */
+    void CheckProtocols(std::list<std::string>& transport_protocols);
+    /** Select transfer protocols from URL option or hard-coded list */
+    void ChooseTransferProtocols(std::list<std::string>& transport_protocols);
   };
 
 } // namespace Arc
