@@ -268,11 +268,9 @@ namespace Arc {
       .NewChild("sourceSURL") = creq.surls().front();
     XMLNode protocols = req.NewChild("transferParameters")
                         .NewChild("arrayOfTransferProtocols");
-    protocols.NewChild("stringArray") = "gsiftp";
-    protocols.NewChild("stringArray") = "https";
-    protocols.NewChild("stringArray") = "httpg";
-    protocols.NewChild("stringArray") = "http";
-    protocols.NewChild("stringArray") = "ftp";
+    std::list<std::string> transport_protocols(creq.transport_protocols());
+    for (std::list<std::string>::iterator prot = transport_protocols.begin(); prot != transport_protocols.end(); ++prot)
+      protocols.NewChild("stringArray") = *prot;
 
     PayloadSOAP *response = NULL;
     SRMReturnCode status = process(&request, &response);
@@ -670,11 +668,10 @@ namespace Arc {
     reqarray.NewChild("expectedFileSize") = tostring(creq.total_size());
     XMLNode protocols = req.NewChild("transferParameters")
                         .NewChild("arrayOfTransferProtocols");
-    protocols.NewChild("stringArray") = "gsiftp";
-    protocols.NewChild("stringArray") = "https";
-    protocols.NewChild("stringArray") = "httpg";
-    protocols.NewChild("stringArray") = "http";
-    protocols.NewChild("stringArray") = "ftp";
+
+    std::list<std::string> transport_protocols(creq.transport_protocols());
+    for (std::list<std::string>::iterator prot = transport_protocols.begin(); prot != transport_protocols.end(); ++prot)
+      protocols.NewChild("stringArray") = *prot;
 
     // set space token if supplied
     if (!creq.space_token().empty())
@@ -772,35 +769,6 @@ namespace Arc {
       return SRM_ERROR_PERMANENT;
 
     }
-    else if (statuscode != SRM_SUCCESS) {
-      // check individual file statuses
-      std::string statusexplanation;
-      SRMStatusCode status = GetStatus(res["arrayOfFileStatuses"]
-                                       ["statusArray"]["status"],
-                                       statusexplanation);
-      if (status == SRM_INVALID_PATH) {
-        // make directories
-        logger.msg(VERBOSE,
-                   "Path %s is invalid, creating required directories",
-                   creq.surls().front());
-        SRMReturnCode mkdirres = mkDir(creq);
-        delete response;
-        if (mkdirres == SRM_OK)
-          return putTURLs(creq, urls);
-        logger.msg(ERROR, "Error creating required directories for %s",
-                   creq.surls().front());
-        creq.finished_error();
-        return mkdirres;
-      }
-      if (res["arrayOfFileStatuses"]["statusArray"]["status"])
-        logger.msg(ERROR, statusexplanation);
-      logger.msg(ERROR, explanation);
-      creq.finished_error();
-      delete response;
-      if (statuscode == SRM_INTERNAL_ERROR)
-        return SRM_ERROR_TEMPORARY;
-      return SRM_ERROR_PERMANENT;
-    }
 
     // the file is ready and pinned - we can get the TURL
     std::string turl = (std::string)res["arrayOfFileStatuses"]["statusArray"]
@@ -1053,6 +1021,7 @@ namespace Arc {
         do {
           list_metadata.clear();
           SRMClientRequest list_req(creq.surls().front());
+          list_req.long_list(creq.long_list());
           int list_offset = max_files_list * list_no;
           int list_count = max_files_list;
           SRMReturnCode res = info(list_req, list_metadata, 0, true,
@@ -1106,12 +1075,12 @@ namespace Arc {
       metadata.size = -1;
 
     if (details["checkSumType"])
-      metadata.checkSumType = (std::string)details["checkSumType"];
+      metadata.checkSumType = lower((std::string)details["checkSumType"]);
     else
       metadata.checkSumType = "";
 
     if (details["checkSumValue"])
-      metadata.checkSumValue = (std::string)details["checkSumValue"];
+      metadata.checkSumValue = lower((std::string)details["checkSumValue"]);
     else
       metadata.checkSumValue = "";
 
