@@ -153,7 +153,7 @@ namespace Arc {
       if (!cfg.cadir.empty())
         comp.NewChild("CACertificatesDir") = cfg.cadir;
       comp.NewAttribute("entry") = "tls";
-      if (sec == SSL3Sec) 
+      if (sec == SSL3Sec)
         comp.NewChild("Handshake") = "SSLv3";
     }
     else if ((sec == GSISec) || (sec == GSIIOSec)) {
@@ -281,6 +281,7 @@ namespace Arc {
 
   static SecurityLayer http_url_to_sec(const URL& url) {
     if(url.Protocol() == "https") {
+      if(url.Option("protocol") == "ssl3") return SSL3Sec;
       return TLSSec;
     } else if(url.Protocol() == "httpg") {
       if(url.Option("protocol") == "gsi") return GSIIOSec;
@@ -386,7 +387,7 @@ namespace Arc {
     reqmsg.Attributes()->set("HTTP:METHOD", method);
     if (!path.empty()) {
       URL url(default_url);
-      url.ChangePath(path);
+      url.ChangeFullPath(path);
       if(relative_uri) {
         // Workaround for servers which can't handle full URLs in request
         reqmsg.Attributes()->set("HTTP:HOST", url.Host() + ":" + tostring(url.Port()));
