@@ -135,6 +135,7 @@ namespace Arc {
     // Attributes from 6.3 Computing Share
 
     ComputingShareName = target.ComputingShareName;
+    MappingQueue = target.MappingQueue;
     MaxWallTime = target.MaxWallTime;
     MaxTotalWallTime = target.MaxTotalWallTime;
     MinWallTime = target.MinWallTime;
@@ -232,7 +233,7 @@ namespace Arc {
 
     //WorkingAreaFree
     if (jobdesc.Resources.DiskSpaceRequirement.DiskSpace) {
-      WorkingAreaFree -= (int)(jobdesc.Resources.DiskSpaceRequirement.DiskSpace / 10E9);
+      WorkingAreaFree -= (int)(jobdesc.Resources.DiskSpaceRequirement.DiskSpace / 1024);
       if (WorkingAreaFree < 0)
         WorkingAreaFree = 0;
     }
@@ -299,6 +300,9 @@ namespace Arc {
     if (!ComputingShareName.empty()) {
        out << IString(" Queue: %s", ComputingShareName) << std::endl;
     }
+    if (!MappingQueue.empty()) {
+       out << IString(" MappingQueue: %s", MappingQueue) << std::endl;
+    }
     if (!HealthState.empty()){
       out << IString(" Health State: %s", HealthState) << std::endl;
     }
@@ -427,7 +431,7 @@ namespace Arc {
       out << std::endl << IString("Queue information:") << std::endl;
 
       if (!ComputingShareName.empty())
-        out << IString(" Mapping Queue: %s", ComputingShareName) << std::endl;
+        out << IString(" Name: %s", ComputingShareName) << std::endl;
       if (MaxWallTime != -1)
         out << IString(" Max Wall Time: %s", MaxWallTime.istr())
                   << std::endl;
