@@ -217,6 +217,10 @@ namespace Arc {
     return *this;
   }
 
+  int Job::operator==(const Job& other) {
+    return IDFromEndpoint == other.IDFromEndpoint;
+  }
+
   void Job::Print(bool longlist) const {
     logger.msg(WARNING, "The Job::Print method is DEPRECATED, use the Job::SaveToStream method instead.");
     SaveToStream(std::cout, longlist);
@@ -422,7 +426,7 @@ namespace Arc {
   }
 
   void Job::SaveToStream(std::ostream& out, bool longlist) const {
-    out << IString("Job: %s", IDFromEndpoint.str()) << std::endl;
+    out << IString("Job: %s", IDFromEndpoint.fullstr()) << std::endl;
     if (!Name.empty())
       out << IString(" Name: %s", Name) << std::endl;
     if (!State().empty())
@@ -551,7 +555,7 @@ namespace Arc {
         logger.msg(VERBOSE, "Waiting for lock on job list file %s", filename);
       }
 
-      usleep(tryInterval);
+      Glib::usleep(tryInterval);
     }
 
     if (!acquired) {
@@ -568,16 +572,19 @@ namespace Arc {
 
   bool Job::WriteJobsToTruncatedFile(const std::string& filename, const std::list<Job>& jobs, unsigned nTries, unsigned tryInterval) {
     Config jobfile;
+    std::map<std::string, XMLNode> jobIDXMLMap;
     for (std::list<Job>::const_iterator it = jobs.begin();
          it != jobs.end(); it++) {
-      std::list<Job>::const_iterator it2 = it;
-      for (++it2; it2 != jobs.end(); ++it2) {
-        if (it->IDFromEndpoint == it2->IDFromEndpoint) {
-          return false; // Identical jobs found in list.
-        }
+      std::map<std::string, XMLNode>::iterator itJobXML = jobIDXMLMap.find(it->IDFromEndpoint.fullstr());
+      if (itJobXML == jobIDXMLMap.end()) {
+        XMLNode xJob = jobfile.NewChild("Job");
+        it->ToXML(xJob);
+        jobIDXMLMap[it->IDFromEndpoint.fullstr()] = xJob;
+      }
+      else {
+        itJobXML->second.Replace(XMLNode(NS(), "Job"));
+        it->ToXML(itJobXML->second);
       }
-      XMLNode xJob = jobfile.NewChild("Job");
-      it->ToXML(xJob);
     }
 
     FileLock lock(filename);
@@ -595,7 +602,7 @@ namespace Arc {
         logger.msg(WARNING, "Waiting for lock on job list file %s", filename);
       }
 
-      usleep(tryInterval);
+      Glib::usleep(tryInterval);
     }
 
     return false;
@@ -607,38 +614,51 @@ namespace Arc {
   }
 
   bool Job::WriteJobsToFile(const std::string& filename, const std::list<Job>& jobs, std::list<const Job*>& newJobs, unsigned nTries, unsigned tryInterval) {
-    newJobs.clear();
-
     FileLock lock(filename);
     for (int tries = (int)nTries; tries > 0; --tries) {
       if (lock.acquire()) {
         Config jobfile;
         jobfile.ReadFromFile(filename);
 
-        for (std::list<Job>::const_iterator it = jobs.begin(); it != jobs.end(); ++it) {
-          std::list<Job>::const_iterator it2 = it;
-          for (++it2; it2 != jobs.end(); ++it2) {
-            if (it->IDFromEndpoint == it2->IDFromEndpoint) {
-              newJobs.clear();
-              lock.release();
-              return false; // Identical jobs found in list.
-            }
+        // Use std::map to store job IDs to be searched for duplicates.
+        std::map<std::string, XMLNode> jobIDXMLMap;
+        for (Arc::XMLNode j = jobfile["Job"]; j; ++j) {
+          if (!((std::string)j["IDFromEndpoint"]).empty()) {
+            jobIDXMLMap[(std::string)j["IDFromEndpoint"]] = j;
           }
-          bool jobExists = false;
-          for (Arc::XMLNode j = jobfile["Job"]; j; ++j) {
-            if (((std::string)j["JobID"])          == it->IDFromEndpoint.fullstr() ||
-                ((std::string)j["IDFromEndpoint"]) == it->IDFromEndpoint.fullstr()) {
-              j.Destroy(); // Overwrite existing job with the current.
-              jobExists = true;
-              break;
-            }
-          }
-
-          it->ToXML(jobfile.NewChild("Job"));
-          if (!jobExists) {
-            newJobs.push_back(&*it);
+          else if (!((std::string)j["JobID"]).empty()) {
+            jobIDXMLMap[(std::string)j["JobID"]] = j;
           }
         }
+
+        std::map<std::string, const Job*> newJobsMap;
+        for (std::list<Job>::const_iterator it = jobs.begin(); it != jobs.end(); ++it) {
+          std::map<std::string, XMLNode>::iterator itJobXML = jobIDXMLMap.find(it->IDFromEndpoint.fullstr());
+          if (itJobXML == jobIDXMLMap.end()) {
+            XMLNode xJob = jobfile.NewChild("Job");
+            it->ToXML(xJob);
+            jobIDXMLMap[it->IDFromEndpoint.fullstr()] = xJob;
+            newJobsMap[it->IDFromEndpoint.fullstr()] = &(*it);
+          }
+          else {
+            // Duplicate found, replace it.
+            itJobXML->second.Replace(XMLNode(NS(), "Job"));
+            it->ToXML(itJobXML->second);
+            
+            // Only add to newJobsMap if this is a new job, i.e. not previous present in jobfile.
+            std::map<std::string, const Job*>::iterator itNewJobsMap = newJobsMap.find(it->IDFromEndpoint.fullstr());
+            if (itNewJobsMap != newJobsMap.end()) {
+              itNewJobsMap->second = &(*it);
+            }
+          }
+        }
+
+        // Add pointers to new Job objects to the newJobs list.
+        for (std::map<std::string, const Job*>::const_iterator it = newJobsMap.begin();
+             it != newJobsMap.end(); ++it) {
+          newJobs.push_back(it->second);
+        }
+
         if (!jobfile.SaveToFile(filename)) {
           lock.release();
           return false;
@@ -651,7 +671,7 @@ namespace Arc {
         logger.msg(WARNING, "Waiting for lock on job list file %s", filename);
       }
 
-      usleep(tryInterval);
+      Glib::usleep(tryInterval);
     }
 
     return false;
@@ -674,8 +694,8 @@ namespace Arc {
         XMLNodeList xmlJobs = jobstorage.Path("Job");
         for (std::list<URL>::const_iterator it = jobids.begin(); it != jobids.end(); ++it) {
           for (XMLNodeList::iterator xJIt = xmlJobs.begin(); xJIt != xmlJobs.end(); ++xJIt) {
-            if ((*xJIt)["IDFromEndpoint"] == it->str() ||
-                (*xJIt)["JobID"] == it->str() // Included for backwards compatibility.
+            if ((*xJIt)["IDFromEndpoint"] == it->fullstr() ||
+                (*xJIt)["JobID"] == it->fullstr() // Included for backwards compatibility.
                 ) {
               xJIt->Destroy(); // Do not break, since for some reason there might be multiple identical jobs in the file.
             }
@@ -693,7 +713,7 @@ namespace Arc {
       if (tries == 6) {
         logger.msg(VERBOSE, "Waiting for lock on job list file %s", filename);
       }
-      usleep(tryInterval);
+      Glib::usleep(tryInterval);
     }
 
     return false;
@@ -727,7 +747,7 @@ namespace Arc {
         logger.msg(WARNING, "Waiting for lock on file %s", filename);
       }
 
-      usleep(tryInterval);
+      Glib::usleep(tryInterval);
     }
 
     return false;
@@ -756,7 +776,7 @@ namespace Arc {
         logger.msg(WARNING, "Waiting for lock on file %s", filename);
       }
 
-      usleep(tryInterval);
+      Glib::usleep(tryInterval);
     }
 
     return false;
@@ -788,7 +808,40 @@ namespace Arc {
         logger.msg(WARNING, "Waiting for lock on file %s", filename);
       }
 
-      usleep(tryInterval);
+      Glib::usleep(tryInterval);
+    }
+
+    return false;
+  }
+
+  bool Job::WriteJobIDsToFile(const std::list<Job>& jobs, const std::string& filename, unsigned nTries, unsigned tryInterval) {
+    if (Glib::file_test(filename, Glib::FILE_TEST_IS_DIR)) return false;
+
+    FileLock lock(filename);
+    for (int tries = (int)nTries; tries > 0; --tries) {
+      if (lock.acquire()) {
+        std::ofstream os(filename.c_str(), std::ios::app);
+        if (!os.good()) {
+          os.close();
+          lock.release();
+          return false;
+        }
+        for (std::list<Job>::const_iterator it = jobs.begin();
+             it != jobs.end(); ++it) {
+          os << it->IDFromEndpoint.fullstr() << std::endl;
+        }
+
+        bool good = os.good();
+        os.close();
+        lock.release();
+        return good;
+      }
+
+      if (tries == 6) {
+        logger.msg(WARNING, "Waiting for lock on file %s", filename);
+      }
+
+      Glib::usleep(tryInterval);
     }
 
     return false;
