@@ -45,31 +45,32 @@ namespace Arc {
     if(data_destination) delete data_destination;
   }
 
-  void JobController::FillJobStore(const Job& job) {
+  bool JobController::FillJobStore(const Job& job) {
 
     if (job.Flavour != flavour) {
       logger.msg(WARNING, "The middleware flavour of the job (%s) does not match that of the job controller (%s)", job.Flavour, flavour);
-      return;
+      return false;
     }
 
     if (!job.JobID) {
       logger.msg(WARNING, "The job ID (%s) is not a valid URL", job.JobID.str());
-      return;
+      return false;
     }
 
     if (!job.Cluster) {
       logger.msg(WARNING, "The resource URL is not a valid URL", job.Cluster.str());
-      return;
+      return false;
     }
 
     jobstore.push_back(job);
+    return true;
   }
 
   bool JobController::Get(const std::list<std::string>& status,
                           const std::string& downloaddir,
-                          const bool keep,
-                          const bool usejobname,
-                          const bool force) {
+                          bool keep,
+                          bool usejobname,
+                          bool force) {
     GetJobInformation();
 
     std::list< std::list<Job>::iterator > downloadable;
@@ -129,7 +130,7 @@ namespace Arc {
   }
 
   bool JobController::Kill(const std::list<std::string>& status,
-                           const bool keep) {
+                           bool keep) {
     GetJobInformation();
 
     std::list< std::list<Job>::iterator > killable;
@@ -178,7 +179,7 @@ namespace Arc {
           continue;
         }
 
-        toberemoved.push_back((*it)->IDFromEndpoint.str());
+        toberemoved.push_back((*it)->IDFromEndpoint.fullstr());
         jobstore.erase(*it);
       }
     }
@@ -188,7 +189,7 @@ namespace Arc {
   }
 
   bool JobController::Clean(const std::list<std::string>& status,
-                            const bool force) {
+                            bool force) {
     GetJobInformation();
 
     std::list<URL> toberemoved;
@@ -318,6 +319,8 @@ namespace Arc {
     bool ok = true;
     for (std::list<Job*>::iterator it = catable.begin();
          it != catable.end(); it++) {
+      // saving to a temp file is necessary because chunks from server
+      // may arrive out of order
       std::string filename = Glib::build_filename(Glib::get_tmp_dir(), "arccat.XXXXXX");
       int tmp_h = Glib::mkstemp(filename);
       if (tmp_h == -1) {
@@ -326,19 +329,22 @@ namespace Arc {
         ok = false;
         continue;
       }
-      close(tmp_h);
 
       logger.msg(VERBOSE, "Catting %s for job %s", whichfile, (*it)->JobID.str());
 
       URL src = GetFileUrlForJob((**it), whichfile);
       if (!src) {
         logger.msg(ERROR, "Cannot create output of %s for job (%s): Invalid source %s", whichfile, (*it)->JobID.str(), src.str());
+        close(tmp_h);
+        unlink(filename.c_str());
         continue;
       }
 
-      URL dst(filename);
+      URL dst("stdio:///"+tostring(tmp_h));
       if (!dst) {
         logger.msg(ERROR, "Cannot create output of %s for job (%s): Invalid destination %s", whichfile, (*it)->JobID.str(), dst.str());
+        close(tmp_h);
+        unlink(filename.c_str());
         continue;
       }
 
@@ -353,17 +359,19 @@ namespace Arc {
           out.put(c);
         }
         is.close();
-        unlink(filename.c_str());
       }
-      else
+      else {
         ok = false;
+      }
+      close(tmp_h);
+      unlink(filename.c_str());
     }
 
     return ok;
   }
 
   bool JobController::PrintJobStatus(const std::list<std::string>& status,
-                                     const bool longlist) {
+                                     bool longlist) {
     logger.msg(WARNING, "The JobController::PrintJobStatus method is DEPRECATED, use the Job::SaveJobStatusToStream method instead.");
     return SaveJobStatusToStream(std::cout, status, longlist);
   }
@@ -422,7 +430,7 @@ namespace Arc {
   bool JobController::Migrate(TargetGenerator& targetGen,
                               Broker *broker,
                               const UserConfig& usercfg,
-                              const bool forcemigration,
+                              bool forcemigration,
                               std::list<URL>& migratedJobIDs) {
     GetJobInformation();
 
@@ -500,14 +508,16 @@ namespace Arc {
           std::find(status.begin(), status.end(), it->State.GetGeneralState()) == status.end())
         continue;
 
-      if (!it->State.IsFinished()) {
-        logger.msg(WARNING, "Job has not finished yet: %s", it->JobID.str());
+      if (it->State == JobState::FINISHED || it->State == JobState::KILLED || it->State == JobState::DELETED) {
+        logger.msg(WARNING, "Job has already finished: %s", it->JobID.str());
         continue;
       }
 
       renewable.push_back(&(*it));
     }
 
+    if (renewable.empty()) return false;
+
     bool ok = true;
     for (std::list<Job*>::iterator it = renewable.begin();
          it != renewable.end(); it++) {
@@ -643,6 +653,11 @@ namespace Arc {
       return false;
     }
 
+    // Set desired number of retries. Also resets any lost
+    // tries from previous files.
+    source->SetTries((src.Protocol() == "file")?1:3);
+    destination->SetTries((dst.Protocol() == "file")?1:3);
+
     // Turn off all features we do not need
     source->SetAdditionalChecks(false);
     destination->SetAdditionalChecks(false);
@@ -656,6 +671,13 @@ namespace Arc {
         logger.msg(ERROR, "File download failed: %s - %s", std::string(res), res.GetDesc());
       else
         logger.msg(ERROR, "File download failed: %s", std::string(res));
+      // Reset connection because one can't be sure how failure
+      // affects server and/or connection state.
+      // TODO: Investigate/define DMC behavior in such case.
+      delete data_source;
+      data_source = NULL;
+      delete data_destination;
+      data_destination = NULL;
       return false;
     }
 
@@ -664,7 +686,7 @@ namespace Arc {
   }
 
   std::list<Job> JobController::GetJobDescriptions(const std::list<std::string>& status,
-                                                   const bool getlocal) {
+                                                   bool getlocal) {
     GetJobInformation();
 
     // Only selected jobs with specified status
@@ -691,11 +713,11 @@ namespace Arc {
         continue;
       }
       if (GetJobDescription(*it, it->JobDescriptionDocument)) {
-        logger.msg(VERBOSE, "Job description retrieved from execution service for job (%s)", it->IDFromEndpoint.str());
+        logger.msg(VERBOSE, "Job description retrieved from execution service for job (%s)", it->IDFromEndpoint.fullstr());
         it++;
       }
       else {
-        logger.msg(INFO, "Failed retrieving job description for job (%s)", it->IDFromEndpoint.str());
+        logger.msg(INFO, "Failed retrieving job description for job (%s)", it->IDFromEndpoint.fullstr());
         it = gettable.erase(it);
       }
     }
