@@ -10,6 +10,7 @@
 #include <unistd.h>
 
 #include <arc/ArcConfig.h>
+#include <arc/CheckSum.h>
 #include <arc/FileLock.h>
 #include <arc/Logger.h>
 #include <arc/StringConv.h>
@@ -33,6 +34,7 @@ namespace Arc {
     }
 
     std::list<Job> jobs;
+    std::list<Job> alreadyFoundJobs;
     Job::ReadAllJobsFromFile(usercfg.JobListFile(), jobs);
 
     // Add jobs explicitly specified.
@@ -40,21 +42,28 @@ namespace Arc {
          it != jobIDsAndNames.end(); ++it) {
       std::list<Job>::iterator itJ = jobs.begin();
       for (; itJ != jobs.end(); ++itJ) {
-        if (*it == itJ->IDFromEndpoint.str() || *it == itJ->Name) {
-          break;
+        if (*it == itJ->IDFromEndpoint.fullstr() || *it == itJ->Name) {
+          // If the job wasn't already selected in a previous cycle...
+          bool alreadySelectedJob = false;
+          for (std::list<Job>::iterator itAJ = alreadyFoundJobs.begin(); itAJ != alreadyFoundJobs.end(); itAJ++) {
+            if (*itAJ == *itJ) {
+              alreadySelectedJob = true;
+            }
+          }
+          // Add it!
+          if (!alreadySelectedJob && AddJob(*itJ)) {
+            // Job was added to JobController, remove it from list.
+            // jobs.erase(itJ);
+            alreadyFoundJobs.push_back(*itJ);
+          } else {
+            logger.msg(WARNING, "Unable to handle job (%s), no suitable JobController plugin found.", *it);
+          }
         }
       }
 
-      if (itJ == jobs.end()) {
+      if (alreadyFoundJobs.empty()) {
         logger.msg(WARNING, "Job not found in job list: %s", *it);
       }
-      else if (AddJob(*itJ)) {
-        // Job was added to JobController, remove it from list.
-        jobs.erase(itJ);
-      }
-      else {
-        logger.msg(WARNING, "Unable to handle job (%s), no suitable JobController plugin found.", *it);
-      }
     }
 
     // Exclude jobs on rejected services.
@@ -92,7 +101,7 @@ namespace Arc {
               continue;
             }
             else {
-              logger.msg(WARNING, "Unable to handle job (%s), no suitable JobController plugin found.", itJ->IDFromEndpoint.str());
+              logger.msg(WARNING, "Unable to handle job (%s), no suitable JobController plugin found.", itJ->IDFromEndpoint.fullstr());
             }
           }
           ++itJ;
@@ -105,7 +114,7 @@ namespace Arc {
       for (std::list<Job>::const_iterator itJ = jobs.begin();
            itJ != jobs.end(); ++itJ) {
         if (!AddJob(*itJ)) {
-          logger.msg(WARNING, "Unable to handle job (%s), no suitable JobController plugin found.", itJ->IDFromEndpoint.str());
+          logger.msg(WARNING, "Unable to handle job (%s), no suitable job management plugin found.", itJ->IDFromEndpoint.fullstr());
         }
       }
     }
@@ -123,7 +132,7 @@ namespace Arc {
 
   bool JobSupervisor::AddJob(const Job& job) {
     if (job.Flavour.empty()) {
-      logger.msg(VERBOSE, "Ignoring job (%s), the Job::Flavour attribute must be specified", job.IDFromEndpoint.str());
+      logger.msg(VERBOSE, "Ignoring job (%s), the Job::Flavour attribute must be specified", job.IDFromEndpoint.fullstr());
       return false;
     }
 
@@ -145,6 +154,225 @@ namespace Arc {
     return true;
   }
 
+  std::list<Job> JobSupervisor::GetJobs() const {
+    std::list<Job> jobs;
+    for (std::list<JobController*>::const_iterator itJC = loader.GetJobControllers().begin();
+         itJC != loader.GetJobControllers().end(); ++itJC) {
+      for (std::list<Job>::const_iterator itJ = (*itJC)->jobstore.begin();
+           itJ != (*itJC)->jobstore.end(); ++itJ) {
+        jobs.push_back(*itJ);
+      }
+    }
+
+    return jobs;
+  }
+
+  void JobSupervisor::Update() {
+    for (std::list<JobController*>::const_iterator itJC = loader.GetJobControllers().begin();
+         itJC != loader.GetJobControllers().end(); ++itJC) {
+      (*itJC)->GetJobInformation();
+    }
+  }
+
+  bool JobSupervisor::Get(const std::list<std::string>& status,
+                          const std::string& downloaddir,
+                          bool usejobname,
+                          bool force,
+                          std::list<URL>& retrievedJobs) {
+    bool ok = true;
+
+    std::list<JobController*> jobConts = loader.GetJobControllers();
+    for (std::list<JobController*>::iterator itJobC = jobConts.begin();
+         itJobC != jobConts.end(); itJobC++) {
+      (*itJobC)->GetJobInformation();
+
+      std::list<Job*> downloadable;
+      for (std::list<Job>::iterator it = (*itJobC)->jobstore.begin();
+           it != (*itJobC)->jobstore.end(); it++) {
+
+        if (!it->State) {
+          logger.msg(WARNING, "Unable to get job (%s), job information not found at execution service", it->JobID.str());
+          continue;
+        }
+
+        if (!status.empty() &&
+            std::find(status.begin(), status.end(), it->State()) == status.end() &&
+            std::find(status.begin(), status.end(), it->State.GetGeneralState()) == status.end()) {
+          continue;
+        }
+
+        if (it->State == JobState::DELETED) {
+          logger.msg(WARNING, "Unable to get job (%s), job is deleted", it->JobID.str());
+          continue;
+        }
+        else if (!it->State.IsFinished()) {
+          logger.msg(WARNING, "Unable to get job (%s), it has not finished yet", it->JobID.str());
+          continue;
+        }
+
+        downloadable.push_back(&(*it));
+      }
+
+      for (std::list<Job*>::iterator it = downloadable.begin();
+           it != downloadable.end(); it++) {
+        if (!(*itJobC)->GetJob(**it, downloaddir, usejobname, force)) {
+          logger.msg(ERROR, "Failed getting job (%s)", (*it)->JobID.str());
+          ok = false;
+        }
+        else {
+          retrievedJobs.push_back((*it)->JobID);
+        }
+      }
+    }
+
+    return ok;
+  }
+
+  bool JobSupervisor::Kill(const std::list<std::string>& status,
+                           std::list<URL>& killedJobs) {
+    bool ok = true;
+
+    std::list<JobController*> jobConts = loader.GetJobControllers();
+    for (std::list<JobController*>::iterator itJobC = jobConts.begin();
+         itJobC != jobConts.end(); itJobC++) {
+      (*itJobC)->GetJobInformation();
+
+      std::list<Job*> killable;
+      for (std::list<Job>::iterator it = (*itJobC)->jobstore.begin();
+           it != (*itJobC)->jobstore.end(); it++) {
+
+        if (!it->State) {
+          logger.msg(WARNING, "Unable to kill job (%s), job information not found at execution service", it->JobID.str());
+          continue;
+        }
+
+        if (!status.empty() &&
+            std::find(status.begin(), status.end(), it->State()) == status.end() &&
+            std::find(status.begin(), status.end(), it->State.GetGeneralState()) == status.end()) {
+          continue;
+        }
+
+        if (it->State == JobState::DELETED) {
+          logger.msg(WARNING, "Unable to kill job (%s), job is deleted", it->JobID.str());
+          continue;
+        }
+        else if (it->State.IsFinished()) {
+          logger.msg(WARNING, "Unable to kill job (%s), job has already finished", it->JobID.str());
+          continue;
+        }
+
+        killable.push_back(&(*it));
+      }
+
+      for (std::list<Job*>::iterator it = killable.begin();
+           it != killable.end(); it++) {
+        if (!(*itJobC)->CancelJob(**it)) {
+          logger.msg(ERROR, "Failed killing job (%s)", (*it)->JobID.str());
+          ok = false;
+        }
+        else {
+          killedJobs.push_back((*it)->JobID);
+        }
+      }
+    }
+
+    return ok;
+  }
+
+  bool JobSupervisor::Renew(const std::list<std::string>& status,
+                            std::list<URL>& renewedJobs) {
+    bool ok = true;
+
+    std::list<JobController*> jobConts = loader.GetJobControllers();
+    for (std::list<JobController*>::iterator itJobC = jobConts.begin();
+         itJobC != jobConts.end(); itJobC++) {
+      (*itJobC)->GetJobInformation();
+
+      std::list<Job*> renewable;
+      for (std::list<Job>::iterator it = (*itJobC)->jobstore.begin();
+           it != (*itJobC)->jobstore.end(); it++) {
+
+        if (!it->State) {
+          logger.msg(WARNING, "Unable to renew job (%s), job information not found at execution service", it->JobID.str());
+          continue;
+        }
+
+        if (!status.empty() &&
+            std::find(status.begin(), status.end(), it->State()) == status.end() &&
+            std::find(status.begin(), status.end(), it->State.GetGeneralState()) == status.end()) {
+          continue;
+        }
+
+        if (it->State == JobState::FINISHED || it->State == JobState::KILLED || it->State == JobState::DELETED) {
+          logger.msg(WARNING, "Unable to renew job (%s), job already finished", it->JobID.str());
+          continue;
+        }
+
+        renewable.push_back(&(*it));
+      }
+
+      for (std::list<Job*>::iterator it = renewable.begin();
+           it != renewable.end(); it++) {
+        if (!(*itJobC)->RenewJob(**it)) {
+          logger.msg(ERROR, "Failed renewing job (%s)", (*it)->JobID.str());
+          ok = false;
+        }
+        else {
+          renewedJobs.push_back((*it)->JobID);
+        }
+      }
+    }
+
+    return ok;
+  }
+
+  bool JobSupervisor::Resume(const std::list<std::string>& status,
+                             std::list<URL>& resumedJobs) {
+    bool ok = true;
+
+    std::list<JobController*> jobConts = loader.GetJobControllers();
+    for (std::list<JobController*>::iterator itJobC = jobConts.begin();
+         itJobC != jobConts.end(); itJobC++) {
+      (*itJobC)->GetJobInformation();
+
+      std::list<Job*> resumable;
+      for (std::list<Job>::iterator it = (*itJobC)->jobstore.begin();
+           it != (*itJobC)->jobstore.end(); it++) {
+
+        if (!it->State) {
+          logger.msg(WARNING, "Unable to resume job (%s), job information not found", it->JobID.str());
+          continue;
+        }
+
+        if (!status.empty() &&
+            std::find(status.begin(), status.end(), it->State()) == status.end() &&
+            std::find(status.begin(), status.end(), it->State.GetGeneralState()) == status.end()) {
+          continue;
+        }
+
+        if (it->State == JobState::FINISHED || it->State == JobState::KILLED || it->State == JobState::DELETED) {
+          logger.msg(WARNING, "Unable to resume job (%s), job is %s and cannot be resumed", it->JobID.str(), it->State.GetGeneralState());
+          continue;
+        }
+
+        resumable.push_back(&(*it));
+      }
+
+      for (std::list<Job*>::iterator it = resumable.begin();
+           it != resumable.end(); it++) {
+        if (!(*itJobC)->ResumeJob(**it)) {
+          logger.msg(ERROR, "Failed resuming job %s", (*it)->JobID.str());
+          ok = false;
+        }
+        else {
+          resumedJobs.push_back((*it)->JobID);
+        }
+      }
+    }
+
+    return ok;
+  }
+
   bool JobSupervisor::Resubmit(const std::list<std::string>& status, int destination,
                                std::list<Job>& resubmittedJobs, std::list<URL>& notresubmitted) {
     bool ok = true;
@@ -169,7 +397,7 @@ namespace Arc {
 
         // If job description is not set, then try to fetch it from execution service.
         if (it->JobDescriptionDocument.empty() && !(*itJobC)->GetJobDescription(*it, it->JobDescriptionDocument)) {
-          logger.msg(ERROR, "Unable to resubmit job (%s), job description could not be retrieved remotely", it->IDFromEndpoint.str());
+          logger.msg(ERROR, "Unable to resubmit job (%s), job description could not be retrieved remotely", it->IDFromEndpoint.fullstr());
           notresubmitted.push_back(it->IDFromEndpoint);
           ok = false;
           continue;
@@ -179,13 +407,13 @@ namespace Arc {
         if (!it->LocalInputFiles.empty()) {
           std::map<std::string, std::string>::iterator itF = it->LocalInputFiles.begin();
           for (; itF != it->LocalInputFiles.end(); ++itF) {
-            if (itF->second != Submitter::GetCksum(itF->first, usercfg)) {
+            if (itF->second != CheckSumAny::FileChecksum(itF->first, CheckSumAny::cksum, true)) {
               break;
             }
           }
 
           if (itF != it->LocalInputFiles.end()) {
-            logger.msg(ERROR, "Unable to resubmit job (%s), local input file (%s) has changed", it->IDFromEndpoint.str(), itF->first);
+            logger.msg(ERROR, "Unable to resubmit job (%s), local input file (%s) has changed", it->IDFromEndpoint.fullstr(), itF->first);
             notresubmitted.push_back(it->IDFromEndpoint);
             ok = false;
             continue;
@@ -234,7 +462,7 @@ namespace Arc {
 
       std::list<JobDescription> jobdescs;
       if (!JobDescription::Parse((*it)->JobDescriptionDocument, jobdescs) || jobdescs.empty()) {
-        logger.msg(ERROR, "Unable to resubmit job (%s), unable to parse obtained job description", (*it)->IDFromEndpoint.str());
+        logger.msg(ERROR, "Unable to resubmit job (%s), unable to parse obtained job description", (*it)->IDFromEndpoint.fullstr());
         resubmittedJobs.pop_back();
         notresubmitted.push_back((*it)->IDFromEndpoint);
         ok = false;
@@ -257,7 +485,7 @@ namespace Arc {
         tg = new TargetGenerator(resubmitUsercfg);
         tg->RetrieveExecutionTargets();
         if (tg->GetExecutionTargets().empty()) {
-          logger.msg(ERROR, "Unable to resubmit job (%s), target information retrieval failed for target: %s", (*it)->IDFromEndpoint.str(), (*it)->Cluster.str());
+          logger.msg(ERROR, "Unable to resubmit job (%s), target information retrieval failed for target: %s", (*it)->IDFromEndpoint.fullstr(), (*it)->Cluster.str());
           delete tg;
           resubmittedJobs.pop_back();
           notresubmitted.push_back((*it)->IDFromEndpoint);
@@ -272,7 +500,7 @@ namespace Arc {
         resubmittedJobs.pop_back();
         notresubmitted.push_back((*it)->IDFromEndpoint);
         ok = false;
-        logger.msg(ERROR, "Unable to resubmit job (%s), no targets applicable for submission", (*it)->IDFromEndpoint.str());
+        logger.msg(ERROR, "Unable to resubmit job (%s), no targets applicable for submission", (*it)->IDFromEndpoint.fullstr());
       }
 
       if (destination == 1) {
@@ -304,7 +532,7 @@ namespace Arc {
 
         // If job description is not set, then try to fetch it from execution service.
         if (it->JobDescriptionDocument.empty() && !(*itJobC)->GetJobDescription(*it, it->JobDescriptionDocument)) {
-          logger.msg(ERROR, "Unable to migrate job (%s), job description could not be retrieved remotely", it->IDFromEndpoint.str());
+          logger.msg(ERROR, "Unable to migrate job (%s), job description could not be retrieved remotely", it->IDFromEndpoint.fullstr());
           notmigrated.push_back(it->IDFromEndpoint);
           ok = false;
           continue;
@@ -344,7 +572,7 @@ namespace Arc {
          itJ != migratableJobs.end(); ++itJ) {
       std::list<JobDescription> jobdescs;
       if (!JobDescription::Parse((*itJ)->JobDescriptionDocument, jobdescs) || jobdescs.empty()) {
-        logger.msg(ERROR, "Unable to migrate job (%s), unable to parse obtained job description", (*itJ)->IDFromEndpoint.str());
+        logger.msg(ERROR, "Unable to migrate job (%s), unable to parse obtained job description", (*itJ)->IDFromEndpoint.fullstr());
         continue;
       }
       jobdescs.front().Identification.ActivityOldId = (*itJ)->ActivityOldID;
@@ -366,7 +594,7 @@ namespace Arc {
       }
 
       if (chosenBroker->EndOfList()) {
-        logger.msg(ERROR, "Job migration failed for job (%s), no applicable targets", (*itJ)->IDFromEndpoint.str());
+        logger.msg(ERROR, "Job migration failed for job (%s), no applicable targets", (*itJ)->IDFromEndpoint.fullstr());
         ok = false;
         migratedJobs.pop_back();
         notmigrated.push_back((*itJ)->IDFromEndpoint);
