@@ -62,12 +62,186 @@ namespace Arc {
      * @param jobs List of Job objects which will be managed by the created
      *  object.
      **/
-    JobSupervisor(const UserConfig& usercfg, const std::list<Job>& jobs);
+    JobSupervisor(const UserConfig& usercfg, const std::list<Job>& jobs = std::list<Job>());
 
     ~JobSupervisor();
 
+    /// Add job
+    /**
+     * Add Job object to this JobSupervisor for job management. The Job
+     * object will be passed to the corresponding specialized JobController.
+     *
+     * @param job Job object to add for job management
+     * @return true is returned if the passed Job object was added to the
+     *  underlying JobController, otherwise false is returned and a log
+     *  message emitted with the reason.
+     **/
     bool AddJob(const Job& job);
 
+    /// Get list of managed jobs
+    /**
+     * The list of jobs managed by this JobSupervisor is returned when
+     * calling this method.
+     *
+     * @return list of Job objects managed by this JobSupervisor
+     **/
+    std::list<Job> GetJobs() const;
+
+    /// Update job information
+    /**
+     * When invoking this method the job information for the jobs managed by
+     * this JobSupervisor will be updated. Internally, for each loaded
+     * JobController the JobController::GetJobInformation method will be
+     * called, which will be responsible for updating job information.
+     **/
+    void Update();
+
+    /// Retrieve job output files
+    /**
+     * This method retrieves output files of jobs managed by this JobSupervisor.
+     *
+     * Before identifying jobs for which to retrieve output files, the
+     * JobController::GetJobInformation method is called for each loaded
+     * JobController in order to retrieve the most up to date job information.
+     * If an empty status-filter is specified, all jobs managed by this
+     * JobSupervisor will be considered for retrieval, except jobs in the
+     * undefined state (see JobState). If the status-filter is not empty, then
+     * only jobs with a general or specific state (see JobState) identical to
+     * any of the entries in the status-filter will be considered, except jobs
+     * in the undefined state. Jobs in the state JobState::DELETED and
+     * unfinished jobs (see JobState::IsFinished) will also not be considered.
+     *
+     * For each of the jobs considered for retrieval, the files will be
+     * downloaded to a directory named either as the last part of the job ID
+     * or the job name, which is determined by the 'usejobname' argument. The
+     * download directories will be located in the directory specified by the
+     * 'downloaddir' argument, as either a relative or absolute path. If the
+     * 'force' argument is set to 'true', and a download directory for a
+     * given job already exist it will be overwritten, otherwise files for
+     * that job will not be downloaded. This method calls the
+     * JobController::GetJob method in order to download jobs, and if a job
+     * is successfully retrieved the job ID will be appended to the
+     * 'retrievedJobs' list. If all jobs are successfully retrieved this
+     * method returns true, otherwise false.
+     *
+     * @param statusfilter list of job status used for filtering jobs.
+     * @param downloaddir specifies the path to in which job download
+     *   directories will be located.
+     * @param usejobname specifies whether to use the job name or job ID as
+     *   directory name to store job output files in.
+     * @param force indicates whether existing job directories should be
+     *   overwritten or not.
+     * @param retrievedJobs job IDs of successfully retrieved jobs will be
+     *   appended to this list.
+     * @see JobController::GetJob.
+     * @return true if all jobs are successfully retrieved, otherwise false.
+     **/
+    bool Get(const std::list<std::string>& statusfilter, const std::string& downloaddir,
+             bool usejobname, bool force, std::list<URL>& retrievedJobs);
+
+    /// Kill jobs
+    /**
+     * This method kills jobs managed by this JobSupervisor.
+     *
+     * Before identifying jobs to kill, the JobController::GetJobInformation
+     * method is called for each loaded JobController in order to retrieve
+     * the most up to date job information.
+     *
+     * Since jobs in the JobState::DELETED, JobState::FINISHED,
+     * JobState::KILLED or JobState::FAILED states is already in a terminal
+     * state, a terminate action will not be send for those. Also no
+     * terminate action will be send for jobs in the JobState::UNDEFINED
+     * state, since job information is not available.
+     * If the status-filter is non-empty, a terminate action will only be
+     * send to jobs with a general or specific state (see JobState)
+     * identical to any of the entries in the status-filter, excluding the
+     * states mentioned above.
+     *
+     * For each job to be killed, the specialized JobController::CancelJob
+     * method is called and is responsible for terminating the given job. If
+     * the method fails to terminate a job, this method will return false,
+     * otherwise true is returned. The job ID of successfully terminated
+     * jobs will be appended to the passed killedJobs list.
+     *
+     * @param statusfilter list of job status used for filtering jobs.
+     * @param killedJobs list of URLs which to append job IDs of
+     *  successfully terminated jobs to.
+     * @see JobController::CancelJob.
+     * @return true if all jobs terminated successfully, otherwise false.
+     **/
+    bool Kill(const std::list<std::string>& statusfilter, std::list<URL>& killedJobs);
+
+    /// Renew job credentials
+    /**
+     * This method renew credentials of the jobs managed by this
+     * JobSupervisor.
+     *
+     * Before identifying jobs for which to renew credentials, the
+     * JobController::GetJobInformation method is called for each loaded
+     * JobController in order to retrieve the most up to date job
+     * information.
+     *
+     * Since jobs in the JobState::DELETED, JobState::FINISHED or
+     * JobState::KILLED states is in a terminal state credentials for those
+     * jobs will not be renewed. Also jobs in the JobState::UNDEFINED state
+     * will not get their credentials renewed, since job information is not
+     * available. The JobState::FAILED state is also a terminal state, but
+     * since jobs in this state can be restarted, credentials for such jobs
+     * can be renewed. If the status-filter is non-empty, a renewal of
+     * credentials will be done for jobs with a general or specific state
+     * (see JobState) identical to any of the entries in the status-filter,
+     * excluding the already filtered states as mentioned above.
+     *
+     * For each job for which to renew credentials, the specialized
+     * JobController::RenewJob method is called and is responsible for
+     * renewing the credentials for the given job. If the method fails to
+     * renew job credentials, this method will return false,
+     * otherwise true is returned. The job ID of successfully renewed jobs
+     * will be appended to the passed renewedJobs list.
+     *
+     * @param statusfilter list of job status used for filtering jobs.
+     * @param renewedJobs list of URLs which to append job IDs to, of
+     *  jobs for which credentials was successfully renewed.
+     * @see JobController::RenewJob.
+     * @return true if credentials for all jobs were successfully renewed,
+     *  otherwise false.
+     **/
+    bool Renew(const std::list<std::string>& statusfilter, std::list<URL>& renewedJobs);
+
+    /// Resume job
+    /**
+     * This method resumes jobs managed by this
+     * JobSupervisor.
+     *
+     * Before identifying jobs to resume, the
+     * JobController::GetJobInformation method is called for each loaded
+     * JobController in order to retrieve the most up to date job
+     * information.
+     *
+     * Since jobs in the JobState::DELETED, JobState::FINISHED or
+     * JobState::KILLED states is in a terminal state credentials for those
+     * jobs will not be renewed. Also jobs in the JobState::UNDEFINED state
+     * will not be resumed, since job information is not available. The
+     * JobState::FAILED state is also a terminal state, but jobs in this
+     * state are allowed to be restarted. If the status-filter is non-empty,
+     * only jobs with a general or specific state (see JobState) identical
+     * to any of the entries in the status-filter will be resumed, excluding
+     * the already filtered states as mentioned above.
+     *
+     * For each job to resume, the specialized JobController::ResumeJob
+     * method is called and is responsible for resuming the particular job.
+     * If the method fails to resume a job, this method will return false,
+     * otherwise true is returned. The job ID of successfully resumed jobs
+     * will be appended to the passed resumedJobs list.
+     *
+     * @param statusfilter list of job status used for filtering jobs.
+     * @param resumeJobs list of URLs which to append job IDs to, of jobs
+     *  which was successfully resumed.
+     * @see JobController::ResumeJob.
+     * @return true if all jobs were successfully resumed, otherwise false.
+     **/
+    bool Resume(const std::list<std::string>& statusfilter, std::list<URL>& resumedJobs);
+
     /// Resubmit jobs
     /**
      * Jobs managed by this JobSupervisor will be resubmitted when invoking this
