@@ -73,6 +73,10 @@ namespace Arc {
     bool open(const std::string& path, int flags, mode_t mode);
     /// Close open file.
     bool close(void);
+    /// Open new temporary file for writing.
+    /// On input path contains template of file name ending with XXXXXX.
+    /// On output path is path to created file.
+    bool mkstemp(std::string& path, mode_t mode);
     /// Change current position in open file.
     off_t lseek(off_t offset, int whence);
     /// Read from open file.
