@@ -18,8 +18,6 @@
 #ifndef WIN32
 #include <poll.h>
 #include <sys/mman.h>
-#else
-#include <arc/win32.h>
 #endif
 
 #include <arc/StringConv.h>
@@ -47,9 +45,18 @@ static bool write_all(int h,const void* buf,size_t l) {
   return true;
 }
 
-//int FileOpen(const std::string& path,int flags,mode_t mode) {
-//  return FileOpen(path,flags,0,0,mode);
-//}
+static bool write_all(FileAccess& fa,const void* buf,size_t l) {
+  for(;l>0;) {
+    ssize_t ll = fa.write(buf,l);
+    if(ll == -1) {
+      if(fa.geterrno() == EINTR) continue;
+      return false;
+    };
+    buf = (const void*)(((const char*)buf)+ll);
+    l-=ll;
+  }
+  return true;
+}
 
 bool FileCopy(const std::string& source_path,const std::string& destination_path,uid_t uid,gid_t gid) {
   if((uid && (uid != getuid())) || (gid && (gid != getgid()))) {
@@ -175,6 +182,8 @@ bool FileCreate(const std::string& filename, const std::string& data, uid_t uid,
     if(!fa.setuid(uid,gid)) { errno = fa.geterrno(); return false; }
     if((!fa.remove(filename)) && (fa.geterrno() != ENOENT)) { errno = fa.geterrno(); return false; }
     if(!fa.open(filename,O_WRONLY|O_CREAT,S_IRUSR|S_IWUSR)) { errno = fa.geterrno(); return false; }
+    if(!write_all(fa,data.c_str(),data.length())) { errno = fa.geterrno(); fa.close(); return false; }
+    fa.close();
     return true;
   }
   if (remove(filename.c_str()) != 0 && errno != ENOENT) return false;
@@ -241,7 +250,7 @@ bool FileLink(const std::string& oldpath,const std::string& newpath,uid_t uid,gi
 std::string FileReadLink(const std::string& path,uid_t uid,gid_t gid) {
   if((uid && (uid != getuid())) || (gid && (gid != getgid()))) {
     FileAccess fa;
-    if(!fa.setuid(uid,gid)) return false;
+    if(!fa.setuid(uid,gid)) return "";
     std::string linkpath;
     fa.readlink(path,linkpath);
     errno = fa.geterrno();
@@ -296,14 +305,48 @@ bool FileDelete(const std::string& path,uid_t uid,gid_t gid) {
 // TODO: find non-blocking way to create directory
 bool DirCreate(const std::string& path,uid_t uid,gid_t gid,mode_t mode,bool with_parents) {
   if((uid && (uid != getuid())) || (gid && (gid != getgid()))) {
+    bool created = false;
+    bool exists = false;
     FileAccess fa;
     if(!fa.setuid(uid,gid)) return false;
     if(with_parents) {
-      if(fa.mkdirp(path,mode)) return true;
+      created = fa.mkdirp(path,mode);
     } else {
-      if(fa.mkdir(path,mode)) return true;
+      created = fa.mkdir(path,mode);
     }
-    if(fa.geterrno() == EEXIST) return true;
+    int err = fa.geterrno();
+    exists = created;
+    if(!created) {
+      // Normally following should be done only if errno is EEXIST.
+      // But some file systems may return different error even if 
+      // directory exists. Lustre for example returns EACCESS
+      // if user can't create directory even if directory already exists.
+      // So doing stat in order to check if directory exists and that it 
+      // is directory. That still does not solve problem with parent
+      // directory without x access right.
+      struct stat st;
+      // Doing lstat because this function is supposed to create directory,
+      // not link to directory.
+      if(fa.lstat(path,st)) {
+        if(S_ISDIR(st.st_mode)) {
+          exists = true;
+        } else {
+          err = EEXIST; // exists, but not a directory
+        }
+      }
+    }
+    if(!exists) {
+      // Nothing we can do - escape
+      errno = err;
+      return false;
+    }
+    if(!created) {
+      // Directory was created by another actor. 
+      // There is no sense to apply permissions in that case.
+      errno = EEXIST;
+      return true;
+    }
+    if(fa.chmod(path,mode)) return true;
     errno = fa.geterrno();
     return false;
   }
@@ -313,27 +356,11 @@ bool DirCreate(const std::string& path,uid_t uid,gid_t gid,mode_t mode,bool with
 bool DirCreate(const std::string& path,mode_t mode,bool with_parents) {
 
   if(::mkdir(path.c_str(),mode) == 0) {
-    if (chmod(path.c_str(), mode) == 0) return true;
+    if (::chmod(path.c_str(), mode) == 0) return true;
     else return false;
   }
 
-  if(errno == EEXIST) {
-    /*
-    Should it be just dumb mkdir or something clever?
-    struct stat st; 
-    int r = ::stat(path.c_str(),&st);
-    if((r == 0) && (S_ISDIR(st.st_mode))) {
-      if((uid == 0) || (st.st_uid == uid)) {
-        if((gid == 0) || (st.st_gid == gid) ||
-           (::chown(path.c_str(),(uid_t)(-1),gid) == 0)) {
-          // mode ?
-          return true;
-        }
-      }
-    }
-    */
-    return true;
-  } else if(errno == ENOENT) {
+  if(errno == ENOENT) {
     if(with_parents) {
       std::string ppath(path);
       if(!Glib::path_is_absolute(ppath)) {
@@ -347,7 +374,12 @@ bool DirCreate(const std::string& path,mode_t mode,bool with_parents) {
       }
     }
   }
-  return false;
+  // Look above in previous DirCreate() for description of following
+  struct stat st;
+  if(::lstat(path.c_str(),&st) != 0) return false;
+  if(!S_ISDIR(st.st_mode)) return false;
+  errno = EEXIST;
+  return true;
 }
 
 bool DirDelete(const std::string& path,uid_t uid,gid_t gid) {
@@ -395,8 +427,8 @@ bool TmpDirCreate(std::string& path) {
   std::string tmpdir(Glib::get_tmp_dir());
   bool result = false;
 #ifdef HAVE_MKDTEMP
-  char tmpdirtemplate[] = "ARC-XXXXXX";
-  tmpdir = Glib::build_filename(tmpdir, tmpdirtemplate);
+  char tmptemplate[] = "ARC-XXXXXX";
+  tmpdir = Glib::build_filename(tmpdir, tmptemplate);
   char* tmpdirchar = mkdtemp(const_cast<char*>(tmpdir.c_str()));
   if (tmpdirchar) {
     path = tmpdirchar;
@@ -411,6 +443,26 @@ bool TmpDirCreate(std::string& path) {
   return result;
 }
 
+bool TmpFileCreate(std::string& filename, const std::string& data, uid_t uid, gid_t gid) {
+  std::string tmpdir(Glib::get_tmp_dir());
+  char tmptemplate[] = "ARC-XXXXXX";
+  filename = Glib::build_filename(tmpdir, tmptemplate);
+  if((uid && (uid != getuid())) || (gid && (gid != getgid()))) {
+    FileAccess fa;
+    if(!fa.setuid(uid,gid)) { errno = fa.geterrno(); return false; }
+    if(!fa.mkstemp(filename,S_IRUSR|S_IWUSR)) { errno = fa.geterrno(); return false; }
+    if(!write_all(fa,data.c_str(),data.length())) { errno = fa.geterrno(); fa.close(); return false; }
+    return true;
+  }
+  int h = Glib::mkstemp(filename);
+  if(h == -1) return false;
+  if(!write_all(h,data.c_str(),data.length())) {
+    ::close(h);
+    return false;
+  };
+  ::close(h);
+  return true;
+}
+
 } // namespace Arc
 
-
