@@ -91,5 +91,11 @@ namespace Arc {
    * latter method is not as safe as mkdtemp. */
   bool TmpDirCreate(std::string& path);
 
+  /// Simple method to create a temporary file containing given data.
+  /// Specified uid and gid are used for accessing filesystem.
+  /** Permissions of the created file are determined using the current umask.
+   * If uid/gid are zero then no real switch of uid/gid is done. */
+  bool TmpFileCreate(std::string& filename, const std::string& data, uid_t uid=0, gid_t gid=0);
+
 } // namespace Arc
 
