@@ -24,6 +24,8 @@
 
 namespace Arc {
 
+  #define DefaultLogLevel (DEBUG)
+
   static std::string list_to_domain(const std::list<std::string>& subdomains) {
     std::string domain;
     for(std::list<std::string>::const_iterator subdomain = subdomains.begin();
@@ -64,8 +66,8 @@ namespace Arc {
     else if (str == "FATAL")
       return FATAL;
     else { // should not happen...
-      Logger::getRootLogger().msg(WARNING, "Invalid log level. Using FATAL.");
-      return FATAL;
+      Logger::getRootLogger().msg(WARNING, "Invalid log level. Using default "+level_to_string(DefaultLogLevel)+".");
+      return DefaultLogLevel;
     }
   }
 
@@ -141,8 +143,8 @@ namespace Arc {
     else if (old_level == 0)
       return FATAL;
     else { // cannot happen...
-      Logger::getRootLogger().msg(WARNING, "Invalid old log level. Using FATAL.");
-      return FATAL;
+      Logger::getRootLogger().msg(WARNING, "Invalid old log level. Using default "+level_to_string(DefaultLogLevel)+".");
+      return DefaultLogLevel;
     }
   }
 
@@ -211,7 +213,7 @@ namespace Arc {
     case DebugFormat:
       Time ct;
       static Time lt(0);
-      os << "[" << ct.GetTime() << "." << ct.GetTimeNanosec()/1000;
+      os << "[" << ct.GetTime() << "." << std::setfill('0') << std::setw(6) << ct.GetTimeNanosec()/1000 << std::setfill(' ') << std::setw(0);
       if(lt.GetTime()) {
         Period d = ct - lt;
         os << "(" << d.GetPeriod()*1000000+d.GetPeriodNanoseconds()/1000<<")";
@@ -504,6 +506,16 @@ namespace Arc {
     getContext().destinations.clear();
   }
 
+  void Logger::deleteDestinations(void) {
+    Glib::Mutex::Lock lock(mutex);
+    std::list<LogDestination*> destinations = getContext().destinations;
+    for(std::list<LogDestination*>::iterator dest = destinations.begin();
+                            dest != destinations.end();) {
+      delete *dest;
+      dest = destinations.erase(dest);
+    }
+  }
+
   void Logger::setThreshold(LogLevel threshold) {
     Glib::Mutex::Lock lock(mutex);
     this->getContext().threshold = threshold;
@@ -562,7 +574,7 @@ namespace Arc {
   Logger::Logger()
     : parent(0),
       domain("Arc"),
-      context(DEBUG) {
+      context(DefaultLogLevel) {
     // addDestination(cerr);
   }
 
