@@ -10,10 +10,6 @@
 
 #include <glibmm/miscutils.h>
 
-#ifdef WIN32
-#include <arc/win32.h>
-#endif
-
 #include <arc/Logger.h>
 #include <arc/StringConv.h>
 #include "URL.h"
@@ -82,7 +78,7 @@ namespace Arc {
   }
 
 
-  static std::string AttributeString(const std::list<std::string> attributes,
+  static std::string AttributeString(const std::list<std::string>& attributes,
                                      char separator) {
 
     std::string attrstring;
@@ -157,11 +153,8 @@ namespace Arc {
         protocol = "file";
         path = url;
       }
-      if (!Glib::path_is_absolute(path)) {
-        char cwd[PATH_MAX];
-        if (getcwd(cwd, PATH_MAX))
-          path = Glib::build_filename(cwd, path);
-      }
+      if (!Glib::path_is_absolute(path))
+        path = Glib::build_filename(Glib::get_current_dir(), path);
       // Simple paths are not expected to contain any options or metadata
       return;
     }
@@ -181,11 +174,8 @@ namespace Arc {
       // This must be only path - we can accept path only for
       // limited set of protocols
       if ((protocol == "file" || protocol == "urllist" || protocol == "link")) {
-        if (!Glib::path_is_absolute(path)) {
-          char cwd[PATH_MAX];
-          if (getcwd(cwd, PATH_MAX))
-            path = Glib::build_filename(cwd, path);
-        }
+        if (!Glib::path_is_absolute(path))
+          path = Glib::build_filename(Glib::get_current_dir(), path);
         return;
       } else if (protocol == "arc") {
         // TODO: It is not defined how arc protocol discovers
@@ -355,6 +345,8 @@ namespace Arc {
         port = GSIFTP_DEFAULT_PORT;
       if (protocol == "lfc")
         port = LFC_DEFAULT_PORT;
+      if (protocol == "root")
+        port = XROOTD_DEFAULT_PORT;
     }
 
     if (protocol != "ldap" && protocol != "arc") {
@@ -377,6 +369,16 @@ namespace Arc {
       }
     }
 
+    ParsePath();
+
+    // Normally host/authority names are case-insensitive
+    host = lower(host);
+  }
+
+
+  void URL::ParsePath(void) {
+    std::string::size_type pos, pos2, pos3;
+
     // if protocol = http, get the options after the ?
     if (protocol == "http" ||
         protocol == "https" ||
@@ -428,8 +430,6 @@ namespace Arc {
         path.erase(0,1);
     }
 
-    // Normally host/authority names are case-insensitive
-    host = lower(host);
   }
 
   URL::~URL() {}
@@ -516,6 +516,61 @@ namespace Arc {
     return fullpath;
   }
 
+  std::string URL::FullPathURIEncoded() const {
+    std::string fullpath;
+
+    if (!path.empty())
+      fullpath += uri_encode(path, false);
+
+    for (std::map<std::string, std::string>::const_iterator
+         it = httpoptions.begin(); it != httpoptions.end(); it++) {
+      if (it == httpoptions.begin())
+        fullpath += '?';
+      else
+        fullpath += '&';
+      fullpath += uri_encode(it->first, true) + '=' + uri_encode(it->second, true);
+    }
+
+    if (!ldapattributes.empty() || (ldapscope != base) || !ldapfilter.empty()) {
+      for (std::list<std::string>::const_iterator
+          it = ldapattributes.begin(); it != ldapattributes.end(); it++) {
+        if (it == ldapattributes.begin())
+          fullpath += '?';
+        else
+          fullpath += ',';
+        fullpath += uri_encode(*it, true);
+      }
+    }
+
+    if ((ldapscope != base) || !ldapfilter.empty()) {
+      switch (ldapscope) {
+      case base:
+        fullpath += "?base";
+        break;
+
+      case onelevel:
+        fullpath += "?one";
+        break;
+
+      case subtree:
+        fullpath += "?sub";
+        break;
+      }
+    }
+
+    if (!ldapfilter.empty())
+      fullpath += '?' + uri_encode(ldapfilter, true);
+
+    return fullpath;
+  }
+
+  void URL::ChangeFullPath(const std::string& newpath) {
+    path = newpath;
+    ParsePath();
+    std::string basepath = path;
+    if (protocol != "ldap") ChangePath(basepath);
+  }
+
   void URL::ChangePath(const std::string& newpath) {
     path = newpath;
 
@@ -523,18 +578,13 @@ namespace Arc {
     if (protocol == "ldap") {
       if (path.find("/") != std::string::npos)
         path = Path2BaseDN(path);
-
     // add absolute path for relative file URLs
     } else if (protocol == "file" || protocol == "urllist") {
       if(!Glib::path_is_absolute(path)) {
-        char cwd[PATH_MAX];
-        if (getcwd(cwd, PATH_MAX))
-          path = Glib::build_filename(cwd, path);
+        path = Glib::build_filename(Glib::get_current_dir(), path);
       }
-    }
-
     // for generic URL just make sure path has leading /
-    else if ((path[0] != '/') && (!path.empty())) {
+    } else if ((path[0] != '/') && (!path.empty())) {
       URLLogger.msg(WARNING, "Attempt to assign relative path to URL - making it absolute");
       path = "/" + path;
     }
@@ -555,6 +605,19 @@ namespace Arc {
       return undefined;
   }
 
+  bool URL::AddHTTPOption(const std::string& option, const std::string& value,
+                      bool overwrite) {
+    if (option.empty() || value.empty() ||
+        (!overwrite && httpoptions.find(option) != httpoptions.end()))
+      return false;
+    httpoptions[option] = value;
+    return true;
+  }
+
+  void URL::RemoveHTTPOption(const std::string& option) {
+    httpoptions.erase(option);
+  }
+
   const std::map<std::string, std::string>& URL::MetaDataOptions() const {
     return metadataoptions;
   }
@@ -637,6 +700,14 @@ namespace Arc {
     locations.push_back(location);
   }
 
+  void URL::RemoveOption(const std::string& option) {
+    urloptions.erase(option);
+  }
+
+  void URL::RemoveMetaDataOption(const std::string& option) {
+    metadataoptions.erase(option);
+  }
+
   const std::list<URLLocation>& URL::Locations() const {
     return locations;
   }
@@ -682,10 +753,11 @@ namespace Arc {
       urlstr += '@';
 
     if (!host.empty()) {
-      if(ip6addr)
+      if(ip6addr) {
         urlstr += "[" + host + "]";
-      else
+      } else {
         urlstr += host;
+      }
     }
 
     if (port != -1)
@@ -757,10 +829,11 @@ namespace Arc {
       urlstr += '@';
 
     if (!host.empty()) {
-      if(ip6addr)
+      if(ip6addr) {
         urlstr += "[" + host + "]";
-      else
+      } else {
         urlstr += host;
+      }
     }
 
     if (port != -1)
@@ -829,10 +902,11 @@ namespace Arc {
       urlstr = protocol + "://";
 
     if (!host.empty()) {
-      if(ip6addr)
+      if(ip6addr) {
         urlstr += "[" + host + "]";
-      else
+      } else {
         urlstr += host;
+      }
     }
 
     if (port != -1)
@@ -973,7 +1047,8 @@ namespace Arc {
   PathIterator::PathIterator(const std::string& path, bool end)
     : path(path),
       pos(std::string::npos),
-      end(end) {
+      end(end),
+      done(false) {
     if (end)
       operator--();
     else
@@ -1056,14 +1131,14 @@ namespace Arc {
       str.erase(str.length()-1);
     }
 
-    if (protocol + "://" == str.substr(0, protocol.length() + 3)) {
+    if (lower(protocol) + "://" == lower(str.substr(0, protocol.length() + 3))) {
       str.erase(0, protocol.length()+3);
       if (str.empty()) {
         return false;
       }
     }
 
-    if (host != str.substr(0, host.length())) {
+    if (lower(host) != lower(str.substr(0, host.length()))) {
       return false;
     }
 
