@@ -62,7 +62,8 @@ namespace Arc {
 
   const std::string UserConfig::DEFAULTCONFIG = Glib::build_filename(ARCUSERDIRECTORY, "client.conf");
 
-  UserConfig::UserConfig(initializeCredentialsType initializeCredentials) {
+  UserConfig::UserConfig(initializeCredentialsType initializeCredentials)
+    : ok(false), initializeCredentials(initializeCredentials) {
     if (initializeCredentials != initializeCredentialsType::SkipCredentials) {
       InitializeCredentials();
       if ((!CredentialsFound()) && (initializeCredentials == initializeCredentialsType::RequireCredentials))
@@ -82,7 +83,7 @@ namespace Arc {
   UserConfig::UserConfig(const std::string& conffile,
                          initializeCredentialsType initializeCredentials,
                          bool loadSysConfig)
-    : ok(false) {
+    : ok(false), initializeCredentials(initializeCredentials)  {
     if (loadSysConfig) {
 #ifndef WIN32
       if (Glib::file_test(SYSCONFIG, Glib::FILE_TEST_IS_REGULAR)) {
@@ -144,7 +145,7 @@ namespace Arc {
 
   UserConfig::UserConfig(const std::string& conffile, const std::string& jfile,
                          initializeCredentialsType initializeCredentials, bool loadSysConfig)
-    : ok(false) {
+    : ok(false), initializeCredentials(initializeCredentials)  {
     // If job list file have been specified, try to initialize it, and
     // if it fails then this object is non-valid (ok = false).
     if (!jfile.empty() && !JobListFile(jfile))
@@ -297,22 +298,28 @@ namespace Arc {
 
   void UserConfig::InitializeCredentials() {
     const User user;
+    bool has_proxy = false;
     // Look for credentials.
     if (!GetEnv("X509_USER_PROXY").empty()) {
       if (!Glib::file_test(proxyPath = GetEnv("X509_USER_PROXY"), Glib::FILE_TEST_IS_REGULAR)) {
         logger.msg(ERROR, "Can not access proxy file: %s", proxyPath);
         proxyPath.clear();
       }
+      else has_proxy = true;
     }
     else if (!proxyPath.empty()) {
       if (!Glib::file_test(proxyPath, Glib::FILE_TEST_IS_REGULAR)) {
         logger.msg(ERROR, "Can not access proxy file: %s", proxyPath);
         proxyPath.clear();
       }
+      else has_proxy = true;
     }
     else if (!Glib::file_test(proxyPath = Glib::build_filename(Glib::get_tmp_dir(), std::string("x509up_u") + tostring(user.get_uid())), Glib::FILE_TEST_IS_REGULAR)) {
-      logger.msg(WARNING, "Proxy file does not exist: %s ", proxyPath);
-      proxyPath.clear();
+      if (initializeCredentials == initializeCredentialsType::RequireCredentials) {
+        logger.msg(WARNING, "Proxy file does not exist: %s ", proxyPath);
+        proxyPath.clear();
+      }
+      else has_proxy = true;
     }
     if (!GetEnv("X509_USER_CERT").empty() &&
              !GetEnv("X509_USER_KEY").empty()) {
@@ -335,55 +342,74 @@ namespace Arc {
         keyPath.clear();
       }
     }
-    else if (user.get_uid() == 0) {
-      certificatePath = Glib::build_filename(G_DIR_SEPARATOR_S + std::string("etc"), std::string("grid-security") + G_DIR_SEPARATOR_S + std::string("hostcert.pem"));
-      keyPath = Glib::build_filename(G_DIR_SEPARATOR_S + std::string("etc"), std::string("grid-security") + G_DIR_SEPARATOR_S + std::string("hostkey.pem"));
-    }
     else {
-      certificatePath = Glib::build_filename(user.Home(), std::string(".globus") + G_DIR_SEPARATOR_S +  std::string("usercert.pem"));
-      keyPath = Glib::build_filename(user.Home(), std::string(".globus") + G_DIR_SEPARATOR_S + std::string("userkey.pem"));
-#ifdef WIN32
-      certificatePath = Glib::build_filename(std::string(g_get_home_dir()), std::string(".globus") + G_DIR_SEPARATOR_S +  std::string("usercert.pem"));
-      keyPath = Glib::build_filename(std::string(g_get_home_dir()), std::string(".globus") + G_DIR_SEPARATOR_S + std::string("userkey.pem"));
+      if (user.get_uid() == 0) {
+        certificatePath = Glib::build_filename(G_DIR_SEPARATOR_S + std::string("etc"), std::string("grid-security") + G_DIR_SEPARATOR_S + std::string("hostcert.pem"));
+        keyPath = Glib::build_filename(G_DIR_SEPARATOR_S + std::string("etc"), std::string("grid-security") + G_DIR_SEPARATOR_S + std::string("hostkey.pem"));
+        if (!Glib::file_test(certificatePath, Glib::FILE_TEST_IS_REGULAR)) {
+          logger.msg(ERROR, "Can not access certificate file: %s", certificatePath);
+          certificatePath.clear();
+        }
+        if (!Glib::file_test(keyPath, Glib::FILE_TEST_IS_REGULAR)) {
+          logger.msg(ERROR, "Can not access key file: %s", keyPath);
+          keyPath.clear();
+        }
+      }
+      else if(
+#ifndef WIN32
+        !(Glib::file_test(certificatePath = user.Home() + G_DIR_SEPARATOR_S + ".arc" + G_DIR_SEPARATOR_S + "usercert.pem", Glib::FILE_TEST_EXISTS) &&
+        Glib::file_test(keyPath = user.Home() + G_DIR_SEPARATOR_S + ".arc" + G_DIR_SEPARATOR_S + "userkey.pem", Glib::FILE_TEST_EXISTS)) &&
+        !(Glib::file_test(certificatePath = user.Home() + G_DIR_SEPARATOR_S + ".globus" + G_DIR_SEPARATOR_S + "usercert.pem", Glib::FILE_TEST_EXISTS) &&
+        Glib::file_test(keyPath = user.Home() + G_DIR_SEPARATOR_S + ".globus" + G_DIR_SEPARATOR_S + "userkey.pem", Glib::FILE_TEST_EXISTS)) &&
+#else
+        !(Glib::file_test(certificatePath = Glib::get_home_dir() + G_DIR_SEPARATOR_S + ".arc" + G_DIR_SEPARATOR_S + "usercert.pem", Glib::FILE_TEST_EXISTS) &&
+        Glib::file_test(keyPath = Glib::get_home_dir() + G_DIR_SEPARATOR_S + ".arc" + G_DIR_SEPARATOR_S + "userkey.pem", Glib::FILE_TEST_EXISTS)) &&
 #endif
-    }
-
-    if (!Glib::file_test(certificatePath, Glib::FILE_TEST_IS_REGULAR)) {
-      logger.msg(ERROR, "Can not access certificate file: %s", certificatePath);
-      certificatePath.clear();
-    }
-    if (!Glib::file_test(keyPath, Glib::FILE_TEST_IS_REGULAR)) {
-      logger.msg(ERROR, "Can not access key file: %s", keyPath);
-      keyPath.clear();
-    }
-
-    if (!GetEnv("X509_CERT_DIR").empty()) {
-      if (!Glib::file_test(caCertificatesDirectory = GetEnv("X509_CERT_DIR"), Glib::FILE_TEST_IS_DIR)) {
-        logger.msg(WARNING, "Can not access CA certificate directory: %s. The certificates will not be verified.", caCertificatesDirectory);
-        caCertificatesDirectory.clear();
+        !(Glib::file_test(certificatePath = ArcLocation::Get() + G_DIR_SEPARATOR_S + "etc" + G_DIR_SEPARATOR_S + "arc" + G_DIR_SEPARATOR_S + "usercert.pem", Glib::FILE_TEST_EXISTS) &&
+        Glib::file_test(keyPath = ArcLocation::Get() + G_DIR_SEPARATOR_S + "etc" + G_DIR_SEPARATOR_S + "arc" + G_DIR_SEPARATOR_S + "userkey.pem", Glib::FILE_TEST_EXISTS)) &&
+        !(Glib::file_test(certificatePath = std::string(Glib::get_current_dir()) + G_DIR_SEPARATOR_S + "usercert.pem", Glib::FILE_TEST_EXISTS) &&
+        Glib::file_test(keyPath = std::string(Glib::get_current_dir()) + G_DIR_SEPARATOR_S + "userkey.pem", Glib::FILE_TEST_EXISTS))) {
+        if((has_proxy == false) && (initializeCredentials == initializeCredentialsType::RequireCredentials))
+          logger.msg(WARNING, "Can not find certificate/key (usercert.pem, userkey.pem) in default location: ~/.arc/, ~/.globus/, $ARC_LOCATION/etc/arc, and $PWD/; And proxy certificate path has not been set and does not existing in /tmp; Please manually specify the certificate/key location, or use arcproxy to create a proxy certificte");
+        certificatePath.clear(); keyPath.clear();
       }
     }
-    else if (!caCertificatesDirectory.empty()) {
-      if (!Glib::file_test(caCertificatesDirectory, Glib::FILE_TEST_IS_DIR)) {
-        logger.msg(WARNING, "Can not access CA certificate directory: %s. The certificates will not be verified", caCertificatesDirectory);
-        caCertificatesDirectory.clear();
+
+    if(initializeCredentials != initializeCredentialsType::SkipCACredentials) {
+      if (!GetEnv("X509_CERT_DIR").empty()) {
+        if (!Glib::file_test(caCertificatesDirectory = GetEnv("X509_CERT_DIR"), Glib::FILE_TEST_IS_DIR)) {
+          logger.msg(WARNING, "Can not access CA certificate directory: %s. The certificates will not be verified.", caCertificatesDirectory);
+          caCertificatesDirectory.clear();
+        }
       }
-    }
-    else if ((user.get_uid() == 0 || !Glib::file_test(caCertificatesDirectory = user.Home() + G_DIR_SEPARATOR_S + ".globus" + G_DIR_SEPARATOR_S + "certificates", Glib::FILE_TEST_IS_DIR)) &&
-             !Glib::file_test(caCertificatesDirectory = std::string(Glib::get_home_dir()) + G_DIR_SEPARATOR_S + ".globus" + G_DIR_SEPARATOR_S + "certificates", Glib::FILE_TEST_IS_DIR) &&
-             !Glib::file_test(caCertificatesDirectory = ArcLocation::Get() + G_DIR_SEPARATOR_S + "etc" + G_DIR_SEPARATOR_S + "certificates", Glib::FILE_TEST_IS_DIR) &&
-             !Glib::file_test(caCertificatesDirectory = ArcLocation::Get() + G_DIR_SEPARATOR_S + "etc" + G_DIR_SEPARATOR_S + "grid-security" + G_DIR_SEPARATOR_S + "certificates", Glib::FILE_TEST_IS_DIR) &&
-             !Glib::file_test(caCertificatesDirectory = ArcLocation::Get() + G_DIR_SEPARATOR_S + "share" + G_DIR_SEPARATOR_S + "certificates", Glib::FILE_TEST_IS_DIR)) {
-      caCertificatesDirectory = Glib::build_filename(G_DIR_SEPARATOR_S + std::string("etc"), std::string("grid-security") + G_DIR_SEPARATOR_S + std::string("certificates"));
-      if (!Glib::file_test(caCertificatesDirectory.c_str(), Glib::FILE_TEST_IS_DIR)) {
-        logger.msg(WARNING, "Can not locate CA certificate directory on the default directories: ~/.globus/certificates, $ARC_LOCATION/etc/certificates, $ARC_LOCATION/etc/grid-security/certificates, $ARC_LOCATION/share/certificates, /etc/grid-security/certificates. The certificate will not be verified.");
-        caCertificatesDirectory.clear();
+      else if (!caCertificatesDirectory.empty()) {
+        if (!Glib::file_test(caCertificatesDirectory, Glib::FILE_TEST_IS_DIR)) {
+          logger.msg(WARNING, "Can not access CA certificate directory: %s. The certificates will not be verified.", caCertificatesDirectory);
+          caCertificatesDirectory.clear();
+        }
+      }
+      else if (user.get_uid() == 0 ||
+#ifndef WIN32
+               !Glib::file_test(caCertificatesDirectory = user.Home() + G_DIR_SEPARATOR_S + ".arc" + G_DIR_SEPARATOR_S + "certificates", Glib::FILE_TEST_IS_DIR) &&
+               !Glib::file_test(caCertificatesDirectory = user.Home() + G_DIR_SEPARATOR_S + ".globus" + G_DIR_SEPARATOR_S + "certificates", Glib::FILE_TEST_IS_DIR) &&
+#else
+               !Glib::file_test(caCertificatesDirectory = Glib::get_home_dir() + G_DIR_SEPARATOR_S + ".arc" + G_DIR_SEPARATOR_S + "certificates", Glib::FILE_TEST_IS_DIR) &&
+               !Glib::file_test(caCertificatesDirectory = Glib::get_home_dir() + G_DIR_SEPARATOR_S + ".globus" + G_DIR_SEPARATOR_S + "certificates", Glib::FILE_TEST_IS_DIR) &&
+#endif
+               !Glib::file_test(caCertificatesDirectory = ArcLocation::Get() + G_DIR_SEPARATOR_S + "etc" + G_DIR_SEPARATOR_S + "certificates", Glib::FILE_TEST_IS_DIR) &&
+               !Glib::file_test(caCertificatesDirectory = ArcLocation::Get() + G_DIR_SEPARATOR_S + "etc" + G_DIR_SEPARATOR_S + "grid-security" + G_DIR_SEPARATOR_S + "certificates", Glib::FILE_TEST_IS_DIR) &&
+               !Glib::file_test(caCertificatesDirectory = ArcLocation::Get() + G_DIR_SEPARATOR_S + "share" + G_DIR_SEPARATOR_S + "certificates", Glib::FILE_TEST_IS_DIR)) {
+        caCertificatesDirectory = Glib::build_filename(G_DIR_SEPARATOR_S + std::string("etc"), std::string("grid-security") + G_DIR_SEPARATOR_S + std::string("certificates"));
+        if (!Glib::file_test(caCertificatesDirectory.c_str(), Glib::FILE_TEST_IS_DIR)) {
+          logger.msg(WARNING, "Can not find CA certificate directory in default locations: ~/.arc/certificates, ~/.globus/certificates, $ARC_LOCATION/etc/certificates, $ARC_LOCATION/etc/grid-security/certificates, $ARC_LOCATION/share/certificates, /etc/grid-security/certificates. The certificate will not be verified.");
+          caCertificatesDirectory.clear();
+        }
       }
     }
 
     if (!proxyPath.empty())
       logger.msg(INFO, "Using proxy file: %s", proxyPath);
-    if (!certificatePath.empty() && !keyPath.empty()) {
+    if ((!certificatePath.empty()) && (!keyPath.empty())) {
       logger.msg(INFO, "Using certificate file: %s", certificatePath);
       logger.msg(INFO, "Using key file: %s", keyPath);
     }
@@ -392,6 +418,49 @@ namespace Arc {
       logger.msg(INFO, "Using CA certificate directory: %s", caCertificatesDirectory);
   }
 
+  const std::string& UserConfig::VOMSESPath() {
+    if(!vomsesPath.empty()) return vomsesPath;
+
+    //vomsesPath could be regular file or directory, therefore only existence is checked here.
+    //multiple voms paths under one vomsesPath is processed under arcproxy implementation.
+    if (!GetEnv("X509_VOMS_FILE").empty()) {
+      if (!Glib::file_test(vomsesPath = GetEnv("X509_VOMS_FILE"), Glib::FILE_TEST_EXISTS)) {
+        logger.msg(WARNING, "Can not access VOMSES file/directory: %s.", vomsesPath);
+        vomsesPath.clear();
+      }
+    }
+    else if (!GetEnv("X509_VOMSES").empty()) {
+      if (!Glib::file_test(vomsesPath = GetEnv("X509_VOMSES"), Glib::FILE_TEST_EXISTS)) {
+        logger.msg(WARNING, "Can not access VOMSES file/directory: %s.", vomsesPath);
+        vomsesPath.clear();
+      }
+    }
+    else if (!vomsesPath.empty()) {
+      if (!Glib::file_test(vomsesPath, Glib::FILE_TEST_EXISTS)) {
+        logger.msg(WARNING, "Can not access VOMS file/directory: %s.", vomsesPath);
+        vomsesPath.clear();
+      }
+    }
+    else if ((user.get_uid() == 0 || !Glib::file_test(vomsesPath = user.Home() + G_DIR_SEPARATOR_S + ".arc" + G_DIR_SEPARATOR_S + "vomses", Glib::FILE_TEST_EXISTS) &&
+             !Glib::file_test(vomsesPath = user.Home() + G_DIR_SEPARATOR_S + ".voms" + G_DIR_SEPARATOR_S + "vomses", Glib::FILE_TEST_EXISTS)) &&
+             !Glib::file_test(vomsesPath = std::string(Glib::get_home_dir()) + G_DIR_SEPARATOR_S + ".arc" + G_DIR_SEPARATOR_S + "vomses", Glib::FILE_TEST_EXISTS) &&
+             !Glib::file_test(vomsesPath = std::string(Glib::get_home_dir()) + G_DIR_SEPARATOR_S + ".voms" + G_DIR_SEPARATOR_S + "vomses", Glib::FILE_TEST_EXISTS) &&
+             !Glib::file_test(vomsesPath = ArcLocation::Get() + G_DIR_SEPARATOR_S + "etc" + G_DIR_SEPARATOR_S + "vomses", Glib::FILE_TEST_EXISTS) &&
+             !Glib::file_test(vomsesPath = ArcLocation::Get() + G_DIR_SEPARATOR_S + "etc" + G_DIR_SEPARATOR_S + "grid-security" + G_DIR_SEPARATOR_S + "vomses", Glib::FILE_TEST_EXISTS) &&
+             !Glib::file_test(vomsesPath = std::string(Glib::get_current_dir()) + G_DIR_SEPARATOR_S + "vomses", Glib::FILE_TEST_EXISTS)) {
+      vomsesPath = Glib::build_filename(G_DIR_SEPARATOR_S + std::string("etc"), std::string("vomses"));
+      if (!Glib::file_test(vomsesPath.c_str(), Glib::FILE_TEST_EXISTS)) {
+        vomsesPath = Glib::build_filename(G_DIR_SEPARATOR_S + std::string("etc") + G_DIR_SEPARATOR_S + "grid-security", std::string("vomses"));
+        if (!Glib::file_test(vomsesPath.c_str(), Glib::FILE_TEST_EXISTS)) {
+          logger.msg(WARNING, "Can not find voms service configuration file (vomses) in default locations: ~/.arc/vomses, ~/.voms/vomses, $ARC_LOCATION/etc/vomses, $ARC_LOCATION/etc/grid-security/vomses, $PWD/vomses, /etc/vomses, /etc/grid-security/vomses");
+          vomsesPath.clear();
+        }
+      }
+    }
+
+    return vomsesPath;
+  }
+
   bool UserConfig::LoadConfigurationFile(const std::string& conffile, bool ignoreJobListFile) {
     if (!conffile.empty()) {
       IniConfig ini(conffile);
@@ -470,7 +539,7 @@ namespace Arc {
               while (common["bartender"]) common["bartender"].Destroy();
             }
           }
-          HANDLESTRATT("vomsserverpath", VOMSServerPath)
+          HANDLESTRATT("vomsespath", VOMSESPath)
           HANDLESTRATT("username", UserName)
           HANDLESTRATT("password", Password)
           HANDLESTRATT("proxypath", ProxyPath)
@@ -674,8 +743,8 @@ namespace Arc {
       }
       file << std::endl;
     }
-    if (!vomsServerPath.empty())
-      file << "vomsserverpath = " << vomsServerPath << std::endl;
+    if (!vomsesPath.empty())
+      file << "vomsespath = " << vomsesPath << std::endl;
     if (!username.empty())
       file << "username = " << username << std::endl;
     if (!password.empty())
@@ -717,7 +786,7 @@ namespace Arc {
     }
     if (!rejectedServices[COMPUTING].empty() ||
         !rejectedServices[INDEX].empty()) {
-      file << "rejectedservices =";
+      file << "rejectservices =";
       for (std::list<std::string>::const_iterator it =
              rejectedServices[COMPUTING].begin();
            it != rejectedServices[COMPUTING].end(); it++)
@@ -881,7 +950,7 @@ namespace Arc {
           if (port)
             service = "*:" + service;
         }
-        logger.msg(VERBOSE, "Adding %s service %s ", isSelectedNotRejected ?
+        logger.msg(VERBOSE, "Adding %s service %s", isSelectedNotRejected ?
                    istring("selected") : istring("rejected"), service);
         servicesRef.push_back(service);
       }
@@ -1066,7 +1135,7 @@ namespace Arc {
       }
       else {
         logger.msg(WARNING,
-                   "Alias (%s) contains a wrongly formated element (%s)",
+                   "Alias (%s) contains a wrongly formatted element (%s)",
                    resolvedAliases.front(), *it);
       }
     }
@@ -1195,17 +1264,17 @@ static std::string cert_file_fix(const std::string& old_file,std::string& new_fi
     GET_OLD_VAR("X509_USER_KEY",x509_user_key_old,x509_user_key_set);
     GET_OLD_VAR("X509_USER_CERT",x509_user_cert_old,x509_user_cert_set);
     GET_OLD_VAR("X509_USER_PROXY",x509_user_proxy_old,x509_user_proxy_set);
-    GET_OLD_VAR("CA_CERT_DIR",ca_cert_dir_old,ca_cert_dir_set);
+    GET_OLD_VAR("X509_CERT_DIR",ca_cert_dir_old,ca_cert_dir_set);
     SET_NEW_VAR_FILE("X509_USER_KEY",cfg.KeyPath(),x509_user_key_new);
     SET_NEW_VAR_FILE("X509_USER_CERT",cfg.CertificatePath(),x509_user_cert_new);
     SET_NEW_VAR_FILE("X509_USER_PROXY",cfg.ProxyPath(),x509_user_proxy_new);
-    SET_NEW_VAR("CA_CERT_DIR",cfg.CACertificatesDirectory());
+    SET_NEW_VAR("X509_CERT_DIR",cfg.CACertificatesDirectory());
     EnvLockWrap(false);
   }
 
   CertEnvLocker::~CertEnvLocker(void) {
     EnvLockUnwrap(false);
-    SET_OLD_VAR("CA_CERT_DIR",ca_cert_dir_old,ca_cert_dir_set);
+    SET_OLD_VAR("X509_CERT_DIR",ca_cert_dir_old,ca_cert_dir_set);
     SET_OLD_VAR("X509_USER_PROXY",x509_user_proxy_old,x509_user_proxy_set);
     SET_OLD_VAR("X509_USER_CERT",x509_user_cert_old,x509_user_cert_set);
     SET_OLD_VAR("X509_USER_KEY",x509_user_key_old,x509_user_key_set);
