@@ -141,14 +141,19 @@ static int verify_callback(int ok, X509_STORE_CTX* store_ctx) {
       break;
 #endif
 
-#if (OPENSSL_VERSION_NUMBER > 0x0090706fL)
+#ifdef X509_V_ERR_UNHANDLED_CRITICAL_EXTENSION
       /*
       * In the later version (097g+) OpenSSL does know about
       * proxies, but not non-rfc compliant proxies, it will
       * count them as unhandled critical extensions.
+      * Of cause before version 097g, any proxy (not only non-rfc proxy) 
+      * extension will be count as unhandled critical extensions. 
       * So we will ignore the errors and do our
       * own checks later on, when we check the last
       * certificate in the chain we will check the chain.
+      * Since the X509_V_ERR_UNHANDLED_CRITICAL_EXTENSION (with value 34) 
+      * starts to be defined in version 097(first version of 097 series), 
+      * the detection of this value is used to switch to the following case.
       * As OpenSSL does not recognize legacy proxies (pre-RFC, and older fasion proxies)
       */
     case X509_V_ERR_UNHANDLED_CRITICAL_EXTENSION:
@@ -172,6 +177,9 @@ static int verify_callback(int ok, X509_STORE_CTX* store_ctx) {
   #endif
       ok = 1;
       break;
+#endif
+
+#ifdef X509_V_ERR_INVALID_PURPOSE
     case X509_V_ERR_INVALID_PURPOSE:
       /*
       * Invalid purpose if we init sec context with a server that does
@@ -253,6 +261,7 @@ static int verify_callback(int ok, X509_STORE_CTX* store_ctx) {
     logger.msg(Arc::ERROR,"Can not get the certificate type");
     return 0;
   }
+
   if(CERT_IS_PROXY(type)){
    /* it is a proxy */
         /* a legacy globus proxy may only be followed by another legacy globus
@@ -286,7 +295,7 @@ static int verify_callback(int ok, X509_STORE_CTX* store_ctx) {
     }
     vctx->cert_type=type;
   }
-
+	
   /** We need to check whether the certificate is revoked if it is not a proxy;
    *for proxy, it does not ever get revoked
   */
@@ -390,20 +399,20 @@ static int verify_callback(int ok, X509_STORE_CTX* store_ctx) {
 
 
     /** Only need to check signing policy file for no-proxy certificate*/
-    char* cadir = NULL;
+    std::string cadir;
     char* ca_policy_file_path = NULL;
     if (X509_NAME_cmp(X509_get_subject_name(store_ctx->current_cert), X509_get_issuer_name(store_ctx->current_cert))) {
-      cadir = (char*)(vctx->ca_dir.c_str());
-      if(!(*cadir)) {
-        logger.msg(Arc::ERROR,"Directory of trusted CAs is not specified/found");
-        return (0);
+      cadir = vctx->ca_dir;
+      if(cadir.empty()) {
+        logger.msg(Arc::INFO,"Directory of trusted CAs is not specified/found; Using current path as the CA direcroty");
+        cadir = ".";
       }
 
       unsigned int buffer_len;
       unsigned long hash;
       hash = X509_NAME_hash(X509_get_issuer_name(store_ctx->current_cert));
 
-      buffer_len = strlen(cadir) + strlen(FILE_SEPERATOR) + 8 /* hash */
+      buffer_len = cadir.length() + strlen(FILE_SEPERATOR) + 8 /* hash */
         + strlen(SIGNING_POLICY_FILE_EXTENSION) + 1 /* NULL */;
       ca_policy_file_path = (char*) malloc(buffer_len);
       if(ca_policy_file_path == NULL) {
@@ -411,13 +420,14 @@ static int verify_callback(int ok, X509_STORE_CTX* store_ctx) {
         store_ctx->error = X509_V_ERR_APPLICATION_VERIFICATION;
         return (0);
       }
-      snprintf(ca_policy_file_path,buffer_len,"%s%s%08lx%s", cadir, FILE_SEPERATOR, hash, SIGNING_POLICY_FILE_EXTENSION);
+      snprintf(ca_policy_file_path,buffer_len,"%s%s%08lx%s", cadir.c_str(), FILE_SEPERATOR, hash, SIGNING_POLICY_FILE_EXTENSION);
       ca_policy_file_path[buffer_len-1]=0;
 
       //TODO check the certificate against policy
 
       free(ca_policy_file_path);
     }
+    return (1); //Every thing that need to check for non-proxy has been passed
   }
 
   /**Add the current certificate into cert chain*/
@@ -455,6 +465,7 @@ static int verify_callback(int ok, X509_STORE_CTX* store_ctx) {
         return (0);
       }
 
+
 // TODO: do not use openssl version - instead use result of check if
 // proxy extension is supported
 #if (OPENSSL_VERSION_NUMBER > 0x0090706fL) && (nid == NID_proxyCertInfo)
@@ -532,7 +543,7 @@ static int verify_callback(int ok, X509_STORE_CTX* store_ctx) {
       /**Parse the policy*/
       if(proxycertinfo != NULL) {
         int policynid = OBJ_obj2nid(PROXYPOLICY_get_policy_language(proxycertinfo->proxypolicy));
-        if(policynid == OBJ_sn2nid(INDEPENDENT_PROXY_SN)) {
+        if(policynid == OBJ_txt2nid(INDEPENDENT_PROXY_OID)) {
           /* Put whatever explicit policy here to this particular proxy certificate, usually by
            * pulling them from some database. If there is none policy which need to be explicitly
            * inserted here, clear all the policy storage (make this and any subsequent proxy certificate
@@ -540,7 +551,7 @@ static int verify_callback(int ok, X509_STORE_CTX* store_ctx) {
            */
           vctx->proxy_policy.clear();
         }
-        else if(policynid == OBJ_sn2nid(IMPERSONATION_PROXY_SN)) {
+        else if(policynid == OBJ_txt2nid(IMPERSONATION_PROXY_OID)) {
           /* This is basically a NOP */
         }
         else {
@@ -645,10 +656,10 @@ bool check_cert_type(X509* cert, certType& type) {
           goto err;
         }
         policynid = OBJ_obj2nid(policylang);
-        if(policynid == OBJ_sn2nid(IMPERSONATION_PROXY_SN)) { type = CERT_TYPE_RFC_IMPERSONATION_PROXY; }
-        else if(policynid == OBJ_sn2nid(INDEPENDENT_PROXY_SN)) { type = CERT_TYPE_RFC_INDEPENDENT_PROXY; }
-        else if(policynid == OBJ_sn2nid(ANYLANGUAGE_PROXY_SN)) { type = CERT_TYPE_RFC_ANYLANGUAGE_PROXY; }
-        else if(policynid == OBJ_sn2nid(LIMITED_PROXY_SN)) { type = CERT_TYPE_RFC_LIMITED_PROXY; }
+        if(policynid == OBJ_txt2nid(IMPERSONATION_PROXY_OID)) { type = CERT_TYPE_RFC_IMPERSONATION_PROXY; }
+        else if(policynid == OBJ_txt2nid(INDEPENDENT_PROXY_OID)) { type = CERT_TYPE_RFC_INDEPENDENT_PROXY; }
+        else if(policynid == OBJ_txt2nid(ANYLANGUAGE_PROXY_OID)) { type = CERT_TYPE_RFC_ANYLANGUAGE_PROXY; }
+        else if(policynid == OBJ_txt2nid(LIMITED_PROXY_OID)) { type = CERT_TYPE_RFC_LIMITED_PROXY; }
         else { type = CERT_TYPE_RFC_RESTRICTED_PROXY; }
 
         if((index = X509_get_ext_by_NID(cert, OBJ_txt2nid(PROXYCERTINFO_V3), -1)) != -1) {
@@ -673,9 +684,9 @@ bool check_cert_type(X509* cert, certType& type) {
           goto err;
         }
         policynid = OBJ_obj2nid(policylang);
-        if(policynid == OBJ_sn2nid(IMPERSONATION_PROXY_SN)) { type = CERT_TYPE_GSI_3_IMPERSONATION_PROXY; }
-        else if(policynid == OBJ_sn2nid(INDEPENDENT_PROXY_SN)){ type = CERT_TYPE_GSI_3_INDEPENDENT_PROXY; }
-        else if(policynid == OBJ_sn2nid(LIMITED_PROXY_SN)) { type = CERT_TYPE_GSI_3_LIMITED_PROXY; }
+        if(policynid == OBJ_txt2nid(IMPERSONATION_PROXY_OID)) { type = CERT_TYPE_GSI_3_IMPERSONATION_PROXY; }
+        else if(policynid == OBJ_txt2nid(INDEPENDENT_PROXY_OID)){ type = CERT_TYPE_GSI_3_INDEPENDENT_PROXY; }
+        else if(policynid == OBJ_txt2nid(LIMITED_PROXY_OID)) { type = CERT_TYPE_GSI_3_LIMITED_PROXY; }
         else {type = CERT_TYPE_GSI_3_RESTRICTED_PROXY; }
 
         if((index = X509_get_ext_by_NID(cert, OBJ_txt2nid(PROXYCERTINFO_V4), -1)) != -1) {
@@ -691,7 +702,7 @@ bool check_cert_type(X509* cert, certType& type) {
      * as well.
      */
     X509_NAME_ENTRY* new_name_entry = NULL;
-    if(ret != CERT_TYPE_EEC && ret != CERT_TYPE_CA) {
+    if(type != CERT_TYPE_EEC && type != CERT_TYPE_CA) {
       issuer = X509_NAME_dup(X509_get_issuer_name(cert));
       new_name_entry = X509_NAME_ENTRY_create_by_NID(NULL, NID_commonName, V_ASN1_APP_CHOOSE, data->data, -1);
       if(!new_name_entry) goto err;
