@@ -27,18 +27,19 @@ void VOMSUtilTest::VOMSTrustListTest() {
 
   //  CPPUNIT_ASSERT_EQUAL(!parseVOMSAC(c, emptystring, emptystring, emptylist, output, false),false);
 
-  //
   // Create the AC on the VOMS side
-  //
 
   std::string CAcert("ca_cert.pem");
-  std::string voms_key_file("host_key.pem");
-  std::string voms_cert_file("host_cert.pem");
+  std::string user_key_file("user_key.pem");
+  std::string user_cert_file("user_cert.pem");
+  Arc::Credential issuer_cred(user_cert_file, user_key_file, "", CAcert, "userpassword");
 
-  Arc::Credential voms_cred(voms_cert_file, voms_key_file, ".", CAcert);
+  std::string vomsserver_key_file("host_key.pem");
+  std::string vomsserver_cert_file("host_cert.pem");
+  Arc::Credential ac_issuer_cred(vomsserver_cert_file, vomsserver_key_file, "", CAcert, "");
 
-  std::string user_proxy_file("user_proxy.pem");
-  Arc::Credential proxy_cred(user_proxy_file, user_proxy_file, ".", CAcert);
+  std::string holder_proxy_file("user_proxy.pem");
+  Arc::Credential holder_cred(holder_proxy_file, "", "", CAcert);
 
   std::vector<std::string> fqan;
   fqan.push_back("/nordugrid.org");
@@ -54,7 +55,7 @@ void VOMSUtilTest::VOMSTrustListTest() {
   std::string uri = "voms.nordugrid.org:50000";
 
   std::string ac_str;
-  Arc::createVOMSAC(ac_str, voms_cred, proxy_cred, fqan, targets, attrs, voname, uri, 3600*12);
+  Arc::createVOMSAC(ac_str, ac_issuer_cred, holder_cred, fqan, targets, attrs, voname, uri, 3600*12);
   
   //
   // Create the full AC which is an ordered list of AC 
@@ -66,32 +67,24 @@ void VOMSUtilTest::VOMSTrustListTest() {
 
   std::string voms_proxy_file("voms_proxy.pem");
 
+  // Request
   int keybits = 1024;
   int proxydepth = 10;
   Arc::Time t;
-
-  Arc::Credential proxy_req(t, Arc::Period(12*3600), keybits);
-  std::string proxy_req_file("./request_withac.pem");
+  Arc::Credential proxy_req(t, Arc::Period(12*3600), keybits, "gsi2", "limited", "", proxydepth);
+  std::string proxy_req_file("voms_req.pem");
   proxy_req.GenerateRequest(proxy_req_file.c_str());
 
-  //Signing side
-  Arc::Credential proxy;
-  proxy.InquireRequest(proxy_req_file.c_str());
-  proxy.SetProxyPolicy("gsi2", "limited", "", proxydepth);
   //Add AC extension to proxy certificat before signing it
-  proxy.AddExtension("acseq", (char**) aclist);
+  proxy_req.AddExtension("acseq", (char**) aclist);
 
-  // The User credentials should sign the voms proxy
-  std::string user_cert_file("user_cert.pem");
-  std::string user_key_file("user_key.pem");
-  Arc::Credential user_cred(user_cert_file, user_key_file, ".", CAcert,"userpass");
-
-  user_cred.SignRequest(&proxy, voms_proxy_file.c_str());
+  //Sign the voms proxy
+  issuer_cred.SignRequest(&proxy_req, voms_proxy_file.c_str());
 
   std::string private_key, signing_cert, signing_cert_chain;
   proxy_req.OutputPrivatekey(private_key);
-  user_cred.OutputCertificate(signing_cert);
-  user_cred.OutputCertificateChain(signing_cert_chain);
+  issuer_cred.OutputCertificate(signing_cert);
+  issuer_cred.OutputCertificateChain(signing_cert_chain);
 
   std::ofstream out_f(voms_proxy_file.c_str(), std::ofstream::app);
   out_f.write(private_key.c_str(), private_key.size());
@@ -102,20 +95,17 @@ void VOMSUtilTest::VOMSTrustListTest() {
 
   std::vector<std::string> vomscert_trust_dn;
   vomscert_trust_dn.push_back("/O=Grid/OU=ARC/CN=localhost");
-  vomscert_trust_dn.push_back("^/O=Grid/O=NorduGrid");
-  vomscert_trust_dn.push_back("NEXT CHAIN");
   vomscert_trust_dn.push_back("/O=Grid/OU=ARC/CN=CA");
+  vomscert_trust_dn.push_back("NEXT CHAIN");
+  vomscert_trust_dn.push_back("^/O=Grid/OU=ARC");
 
-  //
   // Read and pars VOMS proxy
-  //
-
-//  static Arc::Logger& logger = Arc::Logger::rootLogger;
 
   Arc::Credential voms_proxy(voms_proxy_file, "", ".", CAcert);
 
   std::vector<Arc::VOMSACInfo> attributes;
-  Arc::parseVOMSAC(voms_proxy, ".", CAcert, vomscert_trust_dn, attributes, false); 
+  Arc::VOMSTrustList trust_dn(vomscert_trust_dn);
+  Arc::parseVOMSAC(voms_proxy, ".", CAcert, trust_dn, attributes, true); 
   
   for(size_t n=0; n<attributes.size(); n++) {
     for(size_t i=0; i<attributes[n].attributes.size(); i++) {
