@@ -11,10 +11,12 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include <utime.h>
 
 #include <glibmm.h>
 
 #include <arc/DateTime.h>
+#include <arc/FileLock.h>
 #include <arc/FileUtils.h>
 #include <arc/Logger.h>
 #include <arc/Thread.h>
@@ -24,17 +26,13 @@
 #include <arc/Utils.h>
 #include <arc/credential/Credential.h>
 #include <arc/data/DataBuffer.h>
-#include <arc/data/CheckSum.h>
+#include <arc/CheckSum.h>
 #include <arc/data/DataMover.h>
 #include <arc/data/DataPoint.h>
 #include <arc/data/DataHandle.h>
 #include <arc/data/FileCache.h>
 #include <arc/data/URLMap.h>
 
-#ifdef WIN32
-#include <arc/win32.h>
-#endif
-
 namespace Arc {
 
   Logger DataMover::logger(Logger::getRootLogger(), "DataMover");
@@ -258,7 +256,7 @@ namespace Arc {
         Credential ci(source.GetUserConfig().ProxyPath(), source.GetUserConfig().ProxyPath(), source.GetUserConfig().CACertificatesDirectory(), "");
         dn = ci.GetIdentityName();
         exp_time = ci.GetEndTime();
-      } catch (CredentialError e) {
+      } catch (CredentialError& e) {
         logger.msg(WARNING, "Couldn't handle certificate: %s", e.what());
       }
     }
@@ -280,7 +278,7 @@ namespace Arc {
                                                 (!source.ReadOnly() || executable || cache_copy),
                                                 executable);
             cache.Stop(canonic_url);
-            source.NextLocation(); /* to decrease retry counter */
+            source.NextTry(); /* to decrease retry counter */
             if (!cache_link_result)
               return DataStatus::CacheError;           
             return DataStatus::SuccessCached;
@@ -297,6 +295,7 @@ namespace Arc {
         if (source.HaveLocations())
           break;
         logger.msg(ERROR, "No locations for source found: %s", source.str());
+        dres = DataStatus::ReadResolveError;
       }
       else
         logger.msg(ERROR, "Failed to resolve source: %s", source.str());
@@ -313,6 +312,7 @@ namespace Arc {
           break;
         logger.msg(ERROR, "No locations for destination found: %s",
                    destination.str());
+        dres = DataStatus::WriteResolveError;
       }
       else
         logger.msg(ERROR, "Failed to resolve destination: %s", destination.str());
@@ -521,7 +521,7 @@ namespace Arc {
         if (!mapped)
           mapped_url = URL();
         else {
-          logger.msg(VERBOSE, "Url is mapped to: %s", mapped_url.str());
+          logger.msg(VERBOSE, "URL is mapped to: %s", mapped_url.str());
           if (mapped_url.Protocol() == "link")
             /* can't cache links */
             cacheable = false;
@@ -544,17 +544,19 @@ namespace Arc {
 #ifndef WIN32
       if (cacheable) {
         res = DataStatus::Success;
+        bool use_remote = true;
         for (;;) { /* cycle for outdated cache files */
           bool is_in_cache = false;
           bool is_locked = false;
-          if (!cache.Start(canonic_url, is_in_cache, is_locked)) {
+          if (!cache.Start(canonic_url, is_in_cache, is_locked, use_remote)) {
             if (is_locked) {
               logger.msg(VERBOSE, "Cached file is locked - should retry");
-              source.NextLocation(); /* to decrease retry counter */
+              source.NextTry(); /* to decrease retry counter */
               return DataStatus::CacheErrorRetryable;
             }
             cacheable = false;
             logger.msg(INFO, "Failed to initiate cache");
+            source.NextLocation(); /* try another source */
             break;
           }
           if (is_in_cache) {
@@ -563,6 +565,7 @@ namespace Arc {
             if (cache_option == "renew") {
               logger.msg(VERBOSE, "Forcing re-download of file %s", canonic_url);
               cache.StopAndDelete(canonic_url);
+              use_remote = false;
               continue;
             }
             /* just need to check permissions */
@@ -608,6 +611,7 @@ namespace Arc {
             if (outdated) {
               cache.StopAndDelete(canonic_url);
               logger.msg(INFO, "Cached file is outdated, will re-download");
+              use_remote = false;
               continue;
             }
             logger.msg(VERBOSE, "Cached copy is still valid");
@@ -618,7 +622,7 @@ namespace Arc {
                             executable)) {
               /* failed cache link is unhandleable */
               cache.Stop(canonic_url);
-              source.NextLocation(); /* to decrease retry counter */
+              source.NextTry(); /* to decrease retry counter */
               return DataStatus::CacheError;
             }
             cache.Stop(canonic_url);
@@ -765,7 +769,7 @@ namespace Arc {
       if (!datares.Passed()) {
         logger.msg(ERROR, "Failed to prepare source: %s",
                    source_url.str());
-        source_url.StopReading();
+        source_url.FinishReading(true);
         res = datares;
         /* try another source */
         if (source.NextLocation())
@@ -842,6 +846,7 @@ namespace Arc {
       if (!datares.Passed()) {
         logger.msg(ERROR, "Failed to prepare destination: %s",
                    destination.str());
+        destination.FinishWriting(true);
         source_url.StopReading();
         source_url.FinishReading(true);
         if (!destination.PreUnregister(replication ||
@@ -857,6 +862,7 @@ namespace Arc {
 
       DataStatus read_failure = DataStatus::Success;
       DataStatus write_failure = DataStatus::Success;
+      std::string cache_lock;
       if (!cacheable) {
         destination.AddCheckSumObject(&crc_dest);
         datares = destination.StartWriting(buffer);
@@ -898,19 +904,29 @@ namespace Arc {
           destination.NextLocation(); /* to decrease retry counter */
           return DataStatus::CacheError; // repeating won't help here
         }
+        cache_lock = chdest.GetURL().Path()+FileLock::getLockSuffix();
 #endif
       }
       logger.msg(VERBOSE, "Waiting for buffer");
-      for (; (!buffer.eof_read() || !buffer.eof_write()) && !buffer.error();)
+      for (; (!buffer.eof_read() || !buffer.eof_write()) && !buffer.error();) {
         buffer.wait_any();
-      logger.msg(INFO, "buffer: read eof : %i", (int)buffer.eof_read());
-      logger.msg(INFO, "buffer: write eof: %i", (int)buffer.eof_write());
-      logger.msg(INFO, "buffer: error    : %i", (int)buffer.error());
+        if (cacheable && !cache_lock.empty()) {
+          // touch cache lock file regularly so it is still valid
+          if (utime(cache_lock.c_str(), NULL) == -1) {
+            logger.msg(WARNING, "Failed updating timestamp on cache lock file %s for file %s: %s",
+                       cache_lock, source_url.str(), StrError(errno));
+          }
+        }
+      }
+      logger.msg(INFO, "buffer: read EOF : %s", buffer.eof_read()?"yes":"no");
+      logger.msg(INFO, "buffer: write EOF: %s", buffer.eof_write()?"yes":"no");
+      logger.msg(INFO, "buffer: error    : %s, read: %s, write: %s", buffer.error()?"yes":"no", buffer.error_read()?"yes":"no", buffer.error_write()?"yes":"no");
       logger.msg(VERBOSE, "Closing read channel");
       read_failure = source_url.StopReading();
       source_url.FinishReading((!read_failure.Passed() || buffer.error()));
-      if (cacheable && mapped)
+      if (cacheable && mapped) {
         source.SetMeta(mapped_p); // pass more metadata (checksum)
+      }
       logger.msg(VERBOSE, "Closing write channel");
       // turn off checks during stop_writing() if force is turned on
       destination_url.SetAdditionalChecks(!force_registration);
@@ -918,7 +934,7 @@ namespace Arc {
         destination_url.FinishWriting(true);
         buffer.error_write(true);
       }
-      else if (!destination_url.FinishWriting(false)) {
+      else if (!destination_url.FinishWriting(buffer.error())) {
         logger.msg(ERROR, "Failed to complete writing to destination");
         buffer.error_write(true);
       }
@@ -1087,7 +1103,7 @@ namespace Arc {
                                          destination_meta_initially_stored).Passed())
             logger.msg(ERROR, "Failed to unregister preregistered lfn. "
                        "You may need to unregister it manually");
-          destination.NextLocation(); /* to decrease retry count */
+          source.NextTry(); /* to decrease retry count */
           return DataStatus::CacheError; /* retry won't help */
         }
       }
