@@ -41,8 +41,8 @@ namespace Arc {
 
   Logger FileCache::logger(Logger::getRootLogger(), "FileCache");
 
-  FileCache::FileCache(std::string cache_path,
-                       std::string id,
+  FileCache::FileCache(const std::string& cache_path,
+                       const std::string& id,
                        uid_t job_uid,
                        gid_t job_gid) {
 
@@ -58,8 +58,8 @@ namespace Arc {
       _caches.clear();
   }
 
-  FileCache::FileCache(std::vector<std::string> caches,
-                       std::string id,
+  FileCache::FileCache(const std::vector<std::string>& caches,
+                       const std::string& id,
                        uid_t job_uid,
                        gid_t job_gid) {
 
@@ -71,10 +71,10 @@ namespace Arc {
       _caches.clear();
   }
 
-  FileCache::FileCache(std::vector<std::string> caches,
-                       std::vector<std::string> remote_caches,
-                       std::vector<std::string> draining_caches,
-                       std::string id,
+  FileCache::FileCache(const std::vector<std::string>& caches,
+                       const std::vector<std::string>& remote_caches,
+                       const std::vector<std::string>& draining_caches,
+                       const std::string& id,
                        uid_t job_uid,
                        gid_t job_gid,
                        int cache_max,
@@ -85,10 +85,10 @@ namespace Arc {
       _caches.clear();
   }
 
-  bool FileCache::_init(std::vector<std::string> caches,
-                        std::vector<std::string> remote_caches,
-                        std::vector<std::string> draining_caches,
-                        std::string id,
+  bool FileCache::_init(const std::vector<std::string>& caches,
+                        const std::vector<std::string>& remote_caches,
+                        const std::vector<std::string>& draining_caches,
+                        const std::string& id,
                         uid_t job_uid,
                         gid_t job_gid,
                         int cache_max,
@@ -184,7 +184,7 @@ namespace Arc {
     return true;
   }
 
-  bool FileCache::Start(std::string url, bool& available, bool& is_locked, bool use_remote) {
+  bool FileCache::Start(const std::string& url, bool& available, bool& is_locked, bool use_remote) {
 
     if (!(*this))
       return false;
@@ -197,7 +197,10 @@ namespace Arc {
     if (!DirCreate(filename.substr(0, filename.rfind("/")), S_IRWXU, true))
       return false;
 
-    int lock_timeout = 86400; // one day timeout on lock TODO: make configurable?
+    // 15 min lock timeout. The lock file is continually updated during the
+    // transfer so 15 mins with no transfer update should mean stale lock.
+    // TODO: make configurable?
+    int lock_timeout = 900;
 
     // acquire lock
     FileLock lock(filename, lock_timeout);
@@ -225,30 +228,43 @@ namespace Arc {
       // check URL inside file for possible hash collisions
       std::list<std::string> lines;
       if (!FileRead(meta_file, lines)) {
-        logger.msg(ERROR, "Error reading meta file %s", _getMetaFileName(url));
+        logger.msg(ERROR, "Error reading meta file %s", meta_file);
         lock.release();
         return false;
       }
       if (lines.empty()) {
-        logger.msg(ERROR, "Meta file %s is empty", _getMetaFileName(url));
-        lock.release();
-        return false;
+        logger.msg(WARNING, "Meta file %s is empty, will recreate", meta_file);
+        if (!FileCreate(meta_file, std::string(url + '\n'))) {
+          logger.msg(WARNING, "Failed to create meta file %s", meta_file);
+        }
       }
-      std::string meta_str = lines.front();
-      std::string::size_type space_pos = meta_str.find(' ', 0);
-      if (meta_str.substr(0, space_pos) != url) {
-        logger.msg(ERROR, "Error: File %s is already cached at %s under a different URL: %s - this file will not be cached",
-                   url, filename, meta_str.substr(0, space_pos));
-        lock.release();
-        return false;
+      else {
+        std::string meta_str = lines.front();
+        // check new and old format for validity time - if old change to new
+        if (meta_str != url) {
+          if (meta_str.substr(0, meta_str.rfind(' ')) != url) {
+            logger.msg(ERROR, "File %s is already cached at %s under a different URL: %s - this file will not be cached",
+                       url, filename, meta_str);
+            lock.release();
+            return false;
+          } else {
+            logger.msg(VERBOSE, "Changing old validity time format to new in %s", meta_file);
+            std::string new_meta(url + '\n' + meta_str.substr(meta_str.rfind(' ')+1) + '\n');
+            lines.pop_front();
+            for (std::list<std::string>::const_iterator i = lines.begin(); i != lines.end(); ++i) {
+              new_meta += *i + '\n';
+            }
+            if (!FileCreate(meta_file, new_meta)) {
+              logger.msg(WARNING, "Could not write meta file %s", meta_file);
+            }
+          }
+        }
       }
     }
     else if (errno == ENOENT) {
       // create new file
       if (!FileCreate(meta_file, std::string(url + '\n'))) {
-        logger.msg(ERROR, "Failed to create meta file %s", meta_file);
-        lock.release();
-        return false;
+        logger.msg(WARNING, "Failed to create meta file %s", meta_file);
       }
     }
     else {
@@ -322,7 +338,7 @@ namespace Arc {
       else {
         logger.msg(VERBOSE, "Creating temporary link from %s to remote cache file %s", filename, remote_cache_file);
         if (!FileLink(remote_cache_file, filename, true)) {
-          logger.msg(ERROR, "Failed to create soft link to remote cache: %s Will download %s from source", StrError(errno), url);
+          logger.msg(ERROR, "Failed to create soft link to remote cache: %s. Will download %s from source", StrError(errno), url);
           if (!remote_lock.release())
             logger.msg(ERROR, "Failed to remove remote lock file on %s. Some manual intervention may be required", remote_cache_file);
           return true;
@@ -337,7 +353,7 @@ namespace Arc {
     return true;
   }
 
-  bool FileCache::Stop(std::string url) {
+  bool FileCache::Stop(const std::string& url) {
 
     if (!(*this))
       return false;
@@ -386,7 +402,7 @@ namespace Arc {
     return true;
   }
 
-  bool FileCache::StopAndDelete(std::string url) {
+  bool FileCache::StopAndDelete(const std::string& url) {
 
     if (!(*this))
       return false;
@@ -442,7 +458,7 @@ namespace Arc {
     return true;
   }
 
-  std::string FileCache::File(std::string url) {
+  std::string FileCache::File(const std::string& url) {
 
     if (!(*this))
       return "";
@@ -471,7 +487,7 @@ namespace Arc {
     return path;
   }
 
-  bool FileCache::Link(std::string dest_path, std::string url, bool copy, bool executable) {
+  bool FileCache::Link(const std::string& dest_path, const std::string& url, bool copy, bool executable) {
 
     if (!(*this))
       return false;
@@ -605,8 +621,10 @@ namespace Arc {
     // the session dir should already exist but in the case of arccp with cache it may not
     // here we use the mapped user to access session dir
 
-    if (!DirCreate(session_dir, _uid, _gid, S_IRWXU, true))
+    if (!DirCreate(session_dir, _uid, _gid, S_IRWXU, true)) {
+      logger.msg(ERROR, "Failed to create directory %s: %s", session_dir, StrError(errno));
       return false;
+    }
 
     // if _cache_link_path is '.' or copy or executable is true then copy instead
     if (copy || executable || cache_params.cache_link_path == ".") {
@@ -653,12 +671,12 @@ namespace Arc {
     return true;
   }
 
-  bool FileCache::Copy(std::string dest_path, std::string url, bool executable) {
+  bool FileCache::Copy(const std::string& dest_path, const std::string& url, bool executable) {
 
     return Link(dest_path, url, true, executable);
   }
 
-  bool FileCache::Release() {
+  bool FileCache::Release() const {
 
     // go through all caches (including remote caches and draining caches)
     // and remove per-job dirs for our job id
@@ -686,10 +704,11 @@ namespace Arc {
     return true;
   }
 
-  bool FileCache::AddDN(std::string url, std::string DN, Time expiry_time) {
+  bool FileCache::AddDN(const std::string& url, const std::string& DN, const Time& exp_time) {
 
     if (DN.empty())
       return false;
+    Time expiry_time(exp_time);
     if (expiry_time == Time(0))
       expiry_time = Time(time(NULL) + CACHE_DEFAULT_AUTH_VALIDITY);
 
@@ -714,32 +733,41 @@ namespace Arc {
     std::list<std::string>::iterator line = lines.begin();
     std::string first_line(*line);
 
-    // check for correct formatting and possible hash collisions between URLs
-    std::string::size_type space_pos = first_line.rfind(' ');
-
-    if (first_line.substr(0, space_pos) != url) {
+    // check for possible hash collisions between URLs
+    // taking into account old meta file format
+    if (first_line != url && first_line.substr(0, first_line.rfind(' ')) != url) {
       logger.msg(ERROR, "Error: File %s is already cached at %s under a different URL: %s - will not add DN to cached list",
-                 url, File(url), first_line.substr(0, space_pos));
+                 url, File(url), first_line);
       return false;
     }
 
     std::string newdnlist(first_line + '\n');
-    newdnlist += std::string(DN + ' ' + expiry_time.str(MDSTime) + '\n');
 
-    // check list of DNs for expired and this DN
-    for (++line; line != lines.end(); ++line) {
-      space_pos = line->rfind(' ');
+    // second line may contain validity time
+    ++line;
+    if (line != lines.end()) {
+      std::string::size_type space_pos = line->rfind(' ');
       if (space_pos == std::string::npos) {
-        logger.msg(WARNING, "Bad format detected in file %s, in line %s", meta_file, *line);
-        continue;
+        newdnlist += std::string(*line + '\n');
+        ++line;
       }
-      // remove expired DNs (after some grace period)
-      if (line->substr(0, space_pos) != DN) {
-        Time exp_time(line->substr(space_pos + 1));
-        if (exp_time > Time(time(NULL) - CACHE_DEFAULT_AUTH_VALIDITY))
-          newdnlist += std::string(*line + '\n');
+
+      // check list of DNs for expired and this DN
+      for (; line != lines.end(); ++line) {
+        space_pos = line->rfind(' ');
+        if (space_pos == std::string::npos) {
+          logger.msg(WARNING, "Bad format detected in file %s, in line %s", meta_file, *line);
+          continue;
+        }
+        // remove expired DNs (after some grace period)
+        if (line->substr(0, space_pos) != DN) {
+          Time exp_time(line->substr(space_pos + 1));
+          if (exp_time > Time(time(NULL) - CACHE_DEFAULT_AUTH_VALIDITY))
+            newdnlist += std::string(*line + '\n');
+        }
       }
     }
+    newdnlist += std::string(DN + ' ' + expiry_time.str(MDSTime) + '\n');
 
     // write everything back to the file
     if (!FileCreate(meta_file, newdnlist)) {
@@ -749,7 +777,7 @@ namespace Arc {
     return true;
   }
 
-  bool FileCache::CheckDN(std::string url, std::string DN) {
+  bool FileCache::CheckDN(const std::string& url, const std::string& DN) {
 
     if (DN.empty())
       return false;
@@ -789,7 +817,7 @@ namespace Arc {
     return false;
   }
 
-  bool FileCache::CheckCreated(std::string url) {
+  bool FileCache::CheckCreated(const std::string& url) {
 
     // check the cache file exists - if so we can get the creation date
     // follow symlinks
@@ -798,7 +826,7 @@ namespace Arc {
     return FileStat(cache_file, &fileStat, true);
   }
 
-  Time FileCache::GetCreated(std::string url) {
+  Time FileCache::GetCreated(const std::string& url) {
 
     // check the cache file exists
     std::string cache_file = File(url);
@@ -812,17 +840,17 @@ namespace Arc {
       return 0;
     }
 
-    time_t ctime = fileStat.st_ctime;
-    if (ctime <= 0)
+    time_t mtime = fileStat.st_mtime;
+    if (mtime <= 0)
       return Time(0);
-    return Time(ctime);
+    return Time(mtime);
   }
 
-  bool FileCache::CheckValid(std::string url) {
+  bool FileCache::CheckValid(const std::string& url) {
     return (GetValid(url) != Time(0));
   }
 
-  Time FileCache::GetValid(std::string url) {
+  Time FileCache::GetValid(const std::string& url) {
 
     // open meta file and pick out expiry time if it exists
     std::string meta_file(_getMetaFileName(url));
@@ -842,39 +870,25 @@ namespace Arc {
       return false;
     }
 
-    // get the first line
-    std::string meta_str(lines.front());
-
-    // if the file contains only the url, we don't have an expiry time
-    if (meta_str == url)
+    // if the file contains only one line, we don't have an expiry time
+    if (lines.size() == 1)
       return Time(0);
 
-    // check sensible formatting - should be like "rls://rls1.ndgf.org/file1 20080101123456Z"
-    if (meta_str.substr(0, url.length() + 1) != url + " ") {
-      logger.msg(ERROR, "Mismatching url in file %s: %s Expected %s", _getMetaFileName(url), meta_str, url);
+    // second line may contain expiry time in form 20080101123456Z,
+    // but it could be a cached DN
+    std::string meta_str(*(lines.erase(lines.begin())));
+
+    if (meta_str.find(' ') != std::string::npos || meta_str.length() != 15)
       return Time(0);
-    }
-    if (meta_str.length() != url.length() + 16) {
-      logger.msg(ERROR, "Bad format in file %s: %s", _getMetaFileName(url), meta_str);
-      return Time(0);
-    }
-    if (meta_str.substr(url.length(), 1) != " ") {
-      logger.msg(ERROR, "Bad separator in file %s: %s", _getMetaFileName(url), meta_str);
-      return Time(0);
-    }
-    if (meta_str.substr(url.length() + 1).length() != 15) {
-      logger.msg(ERROR, "Bad value of expiry time in %s: %s", _getMetaFileName(url), meta_str);
-      return Time(0);
-    }
 
     // convert to Time object
-    return Time(meta_str.substr(url.length() + 1));
+    return Time(meta_str);
   }
 
-  bool FileCache::SetValid(std::string url, Time val) {
+  bool FileCache::SetValid(const std::string& url, const Time& val) {
 
     std::string meta_file(_getMetaFileName(url));
-    std::string file_data(url + " " + val.str(MDSTime));
+    std::string file_data(url + '\n' + val.str(MDSTime));
     if (!FileCreate(meta_file, file_data)) {
       logger.msg(ERROR, "Error opening meta file for writing %s", meta_file);
       return false;
@@ -898,11 +912,11 @@ namespace Arc {
              );
   }
 
-  std::string FileCache::_getMetaFileName(std::string url) {
+  std::string FileCache::_getMetaFileName(const std::string& url) {
     return File(url) + CACHE_META_SUFFIX;
   }
 
-  int FileCache::_chooseCache(std::string url) {
+  int FileCache::_chooseCache(const std::string& url) const {
     
     // get the hash of the url
     std::string hash = FileCacheHash::getHash(url);
@@ -956,14 +970,17 @@ namespace Arc {
       // check if the usage percent is passed
       if ((100 - (100 * cache_it->second.second)/ cache_it->second.first) < _max_used) {                       
         // caches which are under the defined percentage 
-        under_limit.insert(std::make_pair(cache_it->first, (int)roundf((float) cache_it->second.first/total_size*10)));
+        // chance of this cache being used is ratio of cache size to total of all cache sizes
+        // can't use roundf() since it is not supported on all platforms
+        int chance = (int)((float) cache_it->second.first/total_size*10 + 0.5);
+        under_limit.insert(std::make_pair(cache_it->first, chance));
       } else {
         // caches which are passed the defined percentage
         over_limit.insert(std::make_pair((unsigned long long)(cache_it->second.second), cache_it->first));
       }
     }
     int cache_no = 0;
-    if (under_limit.size() > 0) {
+    if (!under_limit.empty()) {
       std::vector<int> utility_cache;
       for ( std::map<int,int> ::iterator cache_it = under_limit.begin(); cache_it != under_limit.end(); cache_it++) {
         // fill the vector with the frequency of cache number according to the cache size. 
@@ -985,7 +1002,7 @@ namespace Arc {
     return cache_no;
   }
   
-  std::pair <unsigned long long, unsigned long long> FileCache::_getCacheInfo(std::string path) {
+  std::pair <unsigned long long, unsigned long long> FileCache::_getCacheInfo(const std::string& path) const {
   
     struct statvfs info;
     if (statvfs(path.c_str(), &info) != 0) {
