@@ -198,29 +198,25 @@ void FileCacheTest::testStart() {
   CPPUNIT_ASSERT(is_locked);
 
   // delete lock file and try again with a non-existent pid
-  // Start only succeeds if processes can be checked through /proc
-  std::string procdir = "/proc";
   CPPUNIT_ASSERT_EQUAL(0, remove(std::string(_fc1->File(_url) + ".lock").c_str()));
   _createFile(_fc1->File(_url) + ".lock", "99999@" + host);
-  if (stat(procdir.c_str(), &fileStat) == 0) {
-    CPPUNIT_ASSERT(_fc1->Start(_url, available, is_locked));
-    CPPUNIT_ASSERT(!available);
-    CPPUNIT_ASSERT(!is_locked);
-    CPPUNIT_ASSERT(_fc1->Stop(_url));
-  } else {
-    CPPUNIT_ASSERT(!_fc1->Start(_url, available, is_locked));
-    CPPUNIT_ASSERT(!available);
-    CPPUNIT_ASSERT(is_locked);
-    CPPUNIT_ASSERT_EQUAL(0, remove(std::string(_fc1->File(_url) + ".lock").c_str()));
-  }
+  CPPUNIT_ASSERT(_fc1->Start(_url, available, is_locked));
+  CPPUNIT_ASSERT(!available);
+  CPPUNIT_ASSERT(!is_locked);
+  CPPUNIT_ASSERT(_fc1->Stop(_url));
 
   // put different url in meta file
-  _createFile(_fc1->File(_url) + ".meta", "http://badfile 1234567890");
+  _createFile(_fc1->File(_url) + ".meta", "http://badfile");
   CPPUNIT_ASSERT(!_fc1->Start(_url, available, is_locked));
   _createFile(_fc1->File(_url) + ".meta", "rls://rls1.ndgf.org/file1.bad");
   CPPUNIT_ASSERT(!_fc1->Start(_url, available, is_locked));
   CPPUNIT_ASSERT_EQUAL(0, remove(std::string(_fc1->File(_url)+".meta").c_str()));
 
+  // put old validity format in meta file and check it is changed to new
+  _createFile(_fc1->File(_url) + ".meta", _url + " 1234567890");
+  CPPUNIT_ASSERT(_fc1->Start(_url, available, is_locked));
+  meta_url = _readFile(meta_file);
+  CPPUNIT_ASSERT_EQUAL(std::string(_url + '\n' + "1234567890\n"), meta_url);
 }
 
 void FileCacheTest::testRemoteCache() {
@@ -307,22 +303,13 @@ void FileCacheTest::testRemoteCache() {
 
   delete _fc1;
   _fc1 = new Arc::FileCache(caches, remote_caches, draining_caches, _jobid, _uid, _gid);
-  // if /proc exists, start should succeed and delete remote cache file and lock
-  // if not, start should still succeed but download to local cache and
-  // leave remote file and lock alone
-  std::string procdir = "/proc";
+  // Start() should succeed and delete remote cache file and lock
   CPPUNIT_ASSERT(_fc1->Start(_url, available, is_locked));
   CPPUNIT_ASSERT(!available);
   CPPUNIT_ASSERT(!is_locked);
-  if (stat(procdir.c_str(), &fileStat) == 0) {
-    CPPUNIT_ASSERT(stat(remote_cache_lock.c_str(), &fileStat) != 0 );
-    CPPUNIT_ASSERT(stat(remote_cache_file.c_str(), &fileStat) != 0 );
-  } else {
-    CPPUNIT_ASSERT(stat(remote_cache_lock.c_str(), &fileStat) == 0 );
-    CPPUNIT_ASSERT(stat(remote_cache_file.c_str(), &fileStat) == 0 );
-    CPPUNIT_ASSERT_EQUAL(0, remove(remote_cache_lock.c_str()));
-    CPPUNIT_ASSERT_EQUAL(0, remove(remote_cache_file.c_str()));
-  }
+  CPPUNIT_ASSERT(stat(remote_cache_lock.c_str(), &fileStat) != 0 );
+  CPPUNIT_ASSERT(stat(remote_cache_file.c_str(), &fileStat) != 0 );
+
   // stop cache to release locks
   CPPUNIT_ASSERT(_fc1->Stop(_url));
 
@@ -691,7 +678,7 @@ void FileCacheTest::testCheckDN() {
   CPPUNIT_ASSERT(_fc1->AddDN(_url, dn2, futuretime));
   CPPUNIT_ASSERT(_fc1->CheckDN(_url, dn1));
   CPPUNIT_ASSERT(_fc1->CheckDN(_url, dn2));
-  CPPUNIT_ASSERT_EQUAL(_url + "\n" + dn2 + " " + futuretime.str(Arc::MDSTime) + "\n" + dn1 + " " + futuretime.str(Arc::MDSTime) + '\n', _readFile(meta_file));
+  CPPUNIT_ASSERT_EQUAL(_url + "\n" + dn1 + " " + futuretime.str(Arc::MDSTime) + "\n" + dn2 + " " + futuretime.str(Arc::MDSTime) + '\n', _readFile(meta_file));
 
   // create expired DN and check it gets removed
   pasttime = Arc::Time(now.GetTime() - 86401);
@@ -703,7 +690,7 @@ void FileCacheTest::testCheckDN() {
   // add with no specified expiry time
   CPPUNIT_ASSERT(_fc1->AddDN(_url, dn2, Arc::Time(0)));
   // test should not fail if time changes during the test
-  CPPUNIT_ASSERT((_url + "\n" + dn2 + " " + futuretime.str(Arc::MDSTime) + "\n" + dn1 + " " + futuretime.str(Arc::MDSTime) + '\n') == _readFile(meta_file) ||
+  CPPUNIT_ASSERT((_url + "\n" + dn1 + " " + futuretime.str(Arc::MDSTime) + "\n" + dn2 + " " + futuretime.str(Arc::MDSTime) + '\n') == _readFile(meta_file) ||
                  (Arc::Time().GetTime() != now.GetTime()));
 }
 
@@ -856,24 +843,24 @@ void FileCacheTest::testValidityDate() {
   CPPUNIT_ASSERT(_fc1->SetValid(_url, now));
   CPPUNIT_ASSERT(_fc1->CheckValid(_url));
   CPPUNIT_ASSERT_EQUAL(now, _fc1->GetValid(_url));
-  CPPUNIT_ASSERT_EQUAL(_url + " " + now.str(Arc::MDSTime), _readFile(meta_file));
+  CPPUNIT_ASSERT_EQUAL(_url + '\n' + now.str(Arc::MDSTime), _readFile(meta_file));
 
   // put bad format inside metafile
   CPPUNIT_ASSERT(_createFile(_fc1->File(_url) + ".meta", _url + " abcd"));
   CPPUNIT_ASSERT_EQUAL(0, (int)(_fc1->GetValid(_url).GetTime()));
-  CPPUNIT_ASSERT(_createFile(_fc1->File(_url) + ".meta", _url + "abc 1234567890"));
+  CPPUNIT_ASSERT(_createFile(_fc1->File(_url) + ".meta", _url + "\nabc 1234567890"));
   CPPUNIT_ASSERT_EQUAL(0, (int)(_fc1->GetValid(_url).GetTime()));
-  CPPUNIT_ASSERT(_createFile(_fc1->File(_url) + ".meta", _url + "abc1234567890"));
+  CPPUNIT_ASSERT(_createFile(_fc1->File(_url) + ".meta", _url + "\nabc1234567890"));
   CPPUNIT_ASSERT_EQUAL(0, (int)(_fc1->GetValid(_url).GetTime()));
   // cannot be more than MAX_INT
-  CPPUNIT_ASSERT(_createFile(_fc1->File(_url) + ".meta", _url + " 1234567890123"));
+  CPPUNIT_ASSERT(_createFile(_fc1->File(_url) + ".meta", _url + "\n1234567890123"));
   CPPUNIT_ASSERT_EQUAL(0, (int)(_fc1->GetValid(_url).GetTime()));
 
   // set new time
   Arc::Time newtime(time(NULL) + 10);
   CPPUNIT_ASSERT(_fc1->SetValid(_url, newtime));
   CPPUNIT_ASSERT_EQUAL(newtime, _fc1->GetValid(_url));
-  CPPUNIT_ASSERT_EQUAL(_url + " " + newtime.str(Arc::MDSTime), _readFile(meta_file));
+  CPPUNIT_ASSERT_EQUAL(_url + '\n' + newtime.str(Arc::MDSTime), _readFile(meta_file));
 
   // Stop cache to release lock
   CPPUNIT_ASSERT(_fc1->Stop(_url));
