@@ -33,9 +33,9 @@ static Arc::Plugin* get_mcc_client(Arc::PluginArgument* arg) {
 }
 
 Arc::PluginDescriptor PLUGINS_TABLE_NAME[] = {
-    { "http.service", "HED:MCC", 0, &get_mcc_service },
-    { "http.client",  "HED:MCC", 0, &get_mcc_client  },
-    { NULL, NULL, 0, NULL }
+    { "http.service", "HED:MCC", NULL, 0, &get_mcc_service },
+    { "http.client",  "HED:MCC", NULL, 0, &get_mcc_client  },
+    { NULL, NULL, NULL, 0, NULL }
 };
 
 namespace Arc {
@@ -48,6 +48,7 @@ class HTTPSecAttr: public SecAttr {
   virtual ~HTTPSecAttr(void);
   virtual operator bool(void) const;
   virtual bool Export(SecAttrFormat format,XMLNode &val) const;
+  virtual std::string get(const std::string& id) const;
  protected:
   std::string action_;
   std::string object_;
@@ -76,6 +77,12 @@ HTTPSecAttr::operator bool(void) const {
   return true;
 }
 
+std::string HTTPSecAttr::get(const std::string& id) const {
+  if(id == "ACTION") return action_;
+  if(id == "OBJECT") return object_;
+  return "";
+}
+
 bool HTTPSecAttr::equal(const SecAttr &b) const {
   try {
     const HTTPSecAttr& a = (const HTTPSecAttr&)b;
