@@ -34,9 +34,9 @@ static Arc::Plugin* get_mcc_client(Arc::PluginArgument* arg) {
 }
 
 Arc::PluginDescriptor PLUGINS_TABLE_NAME[] = {
-    { "soap.service", "HED:MCC", 0, &get_mcc_service },
-    { "soap.client",  "HED:MCC", 0, &get_mcc_client  },
-    { NULL, NULL, 0, NULL }
+    { "soap.service", "HED:MCC", NULL, 0, &get_mcc_service },
+    { "soap.client",  "HED:MCC", NULL, 0, &get_mcc_client  },
+    { NULL, NULL, NULL, 0, NULL }
 };
 
 namespace Arc {
@@ -49,6 +49,7 @@ class SOAPSecAttr: public SecAttr {
   virtual ~SOAPSecAttr(void);
   virtual operator bool(void) const;
   virtual bool Export(SecAttrFormat format,XMLNode &val) const;
+  virtual std::string get(const std::string& id) const;
  protected:
   std::string action_;
   std::string object_;
@@ -69,6 +70,13 @@ SOAPSecAttr::operator bool(void) const {
   return !action_.empty();
 }
 
+std::string SOAPSecAttr::get(const std::string& id) const {
+  if(id == "ACTION") return action_;
+  if(id == "OBJECT") return object_;
+  if(id == "CONTEXT") return context_;
+  return "";
+}
+
 bool SOAPSecAttr::equal(const SecAttr &b) const {
   try {
     const SOAPSecAttr& a = (const SOAPSecAttr&)b;
