@@ -6,7 +6,6 @@
 #include <unistd.h>
 
 #ifdef WIN32
-#include <arc/win32.h>
 #include <winsock2.h>
 typedef int socklen_t;
 #define ErrNo WSAGetLastError()
@@ -87,9 +86,9 @@ static Arc::Plugin* get_mcc_client(Arc::PluginArgument* arg) {
 }
 
 Arc::PluginDescriptor PLUGINS_TABLE_NAME[] = {
-    { "tcp.service", "HED:MCC", 0, &get_mcc_service },
-    { "tcp.client",  "HED:MCC", 0, &get_mcc_client  },
-    { NULL, NULL, 0, NULL }
+    { "tcp.service", "HED:MCC", NULL, 0, &get_mcc_service },
+    { "tcp.client",  "HED:MCC", NULL, 0, &get_mcc_client  },
+    { NULL, NULL, NULL, 0, NULL }
 };
 
 
@@ -366,6 +365,7 @@ class TCPSecAttr: public SecAttr {
   virtual ~TCPSecAttr(void);
   virtual operator bool(void);
   virtual bool Export(SecAttrFormat format,XMLNode &val) const;
+  virtual std::string get(const std::string& id) const;
  protected:
   std::string local_ip_;
   std::string local_port_;
@@ -385,6 +385,14 @@ TCPSecAttr::operator bool(void) {
   return true;
 }
 
+std::string TCPSecAttr::get(const std::string& id) const {
+  if(id == "LOCALIP") return local_ip_;
+  if(id == "LOCALPORT") return local_port_;
+  if(id == "REMOTEIP") return remote_ip_;
+  if(id == "REMOTEPORT") return remote_port_;
+  return "";
+}
+
 bool TCPSecAttr::equal(const SecAttr &b) const {
   try {
     const TCPSecAttr& a = (const TCPSecAttr&)b;
