@@ -2,10 +2,6 @@
 #include <config.h>
 #endif
 
-#ifdef WIN32
-#include <arc/win32.h>
-#endif
-
 #include <openssl/ssl.h>
 
 #include <arc/message/PayloadRaw.h>
