@@ -62,23 +62,63 @@ class TLSSecAttr: public SecAttr {
   virtual operator bool(void) const;
   virtual bool Export(SecAttrFormat format,XMLNode &val) const;
   std::string Identity(void) { return identity_; };
-  std::string Subject(void) {
+  std::string Subject(void) const {
     if(subjects_.size() <= 0) return "";
     return *(--(subjects_.end()));
   };
-  std::string CA(void) {
+  std::string CA(void) const {
     if(subjects_.size() <= 0) return "";
     return *(subjects_.begin());
   };
-  std::string X509Str(void) {
+  std::string X509Str(void) const {
     return x509str_;
   };
+  std::string X509ChainStr(void) const {
+    return x509chainstr_;
+  };
+  virtual std::string get(const std::string& id) const {
+    if(id == "IDENTITY") return identity_;
+    if(id == "SUBJECT") return Subject();
+    if(id == "CA") return CA();
+    if(id == "CERTIFICATE") return x509str_;
+    if(id == "CERTIFICATECHAIN") return x509chainstr_;
+    if(id == "LOCALSUBJECT") return target_;
+    if((id == "VOMS") || (id == "VO")) {
+      std::list<std::string> items = getAll(id);
+      if(!items.empty()) return *items.begin();
+      return "";
+    };
+    return "";
+  };
+  virtual std::list<std::string> getAll(const std::string& id) const {
+    std::list<std::string> items;
+    if(id == "VOMS") {
+      for(std::vector<VOMSACInfo>::const_iterator v = voms_attributes_.begin();
+                                   v != voms_attributes_.end();++v) { 
+        for(std::vector<std::string>::const_iterator a = v->attributes.begin();
+                                   a != v->attributes.end();++a) {
+          std::string vattr = VOMSFQANToFull(v->voname,*a);
+          items.push_back(vattr);
+        };
+      };
+    } else if(id == "VO") {
+      for(std::vector<VOMSACInfo>::const_iterator v = voms_attributes_.begin();
+                                   v != voms_attributes_.end();++v) { 
+        items.push_back(v->voname);
+      };
+    } else {
+      return SecAttr::getAll(id);
+    };
+    return items;
+  };
  protected:
   std::string identity_; // Subject of last non-proxy certificate
   std::list<std::string> subjects_; // Subjects of all certificates in chain
   std::vector<VOMSACInfo> voms_attributes_; // VOMS attributes from the VOMS extension of proxy
   std::string target_; // Subject of host certificate
-  std::string x509str_; // The last certificate (in string format)
+  std::string x509str_; // The last certificate (in string format). TODO: extract on demand.
+  std::string x509chainstr_; // Other certificates (in string format). TODO: extract on demand.
+  bool processing_failed_;
   virtual bool equal(const SecAttr &b) const;
 };
 }
@@ -104,10 +144,10 @@ static Arc::Plugin* get_mcc_client(Arc::PluginArgument* arg) {
 }
 
 Arc::PluginDescriptor PLUGINS_TABLE_NAME[] = {
-    { "tls.service", "HED:MCC", 0, &get_mcc_service },
-    { "tls.client",  "HED:MCC", 0, &get_mcc_client  },
-    { "delegation.collector", "HED:SHC", 0, &ArcSec::DelegationCollector::get_sechandler},
-    { NULL, NULL, 0, NULL }
+    { "tls.service", "HED:MCC", NULL, 0, &get_mcc_service },
+    { "tls.client",  "HED:MCC", NULL, 0, &get_mcc_client  },
+    { "delegation.collector", "HED:SHC", NULL, 0, &ArcSec::DelegationCollector::get_sechandler},
+    { NULL, NULL, NULL, 0, NULL }
 };
 
 using namespace Arc;
@@ -119,6 +159,7 @@ using namespace Arc;
 TLSSecAttr::TLSSecAttr(PayloadTLSStream& payload, ConfigTLSMCC& config, Logger& logger) {
    char buf[100];
    std::string subject;
+   processing_failed_ = false;
    STACK_OF(X509)* peerchain = payload.GetPeerChain();
    voms_attributes_.clear();
    if(peerchain != NULL) {
@@ -138,16 +179,20 @@ TLSSecAttr::TLSSecAttr(PayloadTLSStream& payload, ConfigTLSMCC& config, Logger&
          X509_NAME_oneline(X509_get_subject_name(cert),buf,sizeof(buf));
          subject=buf;
          subjects_.push_back(subject);
+         std::string certstr;
+         x509_to_string(cert, certstr);
+         x509chainstr_=certstr+x509chainstr_;
 #ifdef HAVE_OPENSSL_PROXY
          if(X509_get_ext_by_NID(cert,NID_proxyCertInfo,-1) < 0) {
             identity_=subject;
          };
 #endif
-        // Parse VOMS attributes from each certificate of the peer chain.
-        bool res = parseVOMSAC(cert, config.CADir(), config.CAFile(), config.VOMSCertTrustDN(), voms_attributes_);
-        if(!res) {
-          logger.msg(ERROR,"VOMS attribute parsing failed");
-        };
+         // Parse VOMS attributes from each certificate of the peer chain.
+         Arc::VOMSTrustList trust_list(config.VOMSCertTrustDN());
+         bool res = parseVOMSAC(cert, config.CADir(), config.CAFile(), trust_list, voms_attributes_, true, true);
+         if(!res) {
+            logger.msg(ERROR,"VOMS attribute parsing failed");
+         };
       };
    };
    X509* peercert = payload.GetPeerCert();
@@ -172,7 +217,8 @@ TLSSecAttr::TLSSecAttr(PayloadTLSStream& payload, ConfigTLSMCC& config, Logger&
       };
 #endif
       // Parse VOMS attributes from peer certificate
-      bool res = parseVOMSAC(peercert, config.CADir(), config.CAFile(), config.VOMSCertTrustDN(), voms_attributes_);
+      Arc::VOMSTrustList trust_list(config.VOMSCertTrustDN());
+      bool res = parseVOMSAC(peercert, config.CADir(), config.CAFile(), trust_list, voms_attributes_, true, true);
       if(!res) {
         logger.msg(ERROR,"VOMS attribute parsing failed");
       };
@@ -188,13 +234,34 @@ TLSSecAttr::TLSSecAttr(PayloadTLSStream& payload, ConfigTLSMCC& config, Logger&
       target_=buf;
       //logger.msg(VERBOSE, "Host name: %s", peer_dn);
    };
+   // Cleaning collected VOMS attributes
+   for(std::vector<VOMSACInfo>::iterator v = voms_attributes_.begin();
+                                           v != voms_attributes_.end();) {
+     if(v->status & VOMSACInfo::Error) {
+       if(config.IfCheckVOMSCritical() && (v->status & VOMSACInfo::IsCritical)) {
+         processing_failed_ = true; 
+         logger.msg(ERROR,"Critical VOMS attribute processing failed");
+       };
+       if(config.IfFailOnVOMSParsing() && (v->status & VOMSACInfo::ParsingError)) {
+         processing_failed_ = true; 
+         logger.msg(ERROR,"VOMS attribute parsing failed");
+       };
+       if(config.IfFailOnVOMSInvalid() && (v->status & VOMSACInfo::ValidationError)) {
+         processing_failed_ = true; 
+         logger.msg(ERROR,"VOMS attribute validation failed");
+       };
+       v = voms_attributes_.erase(v);
+     } else {
+       ++v;
+     };
+   };
 }
 
 TLSSecAttr::~TLSSecAttr(void) {
 }
 
 TLSSecAttr::operator bool(void) const {
-  return true;
+  return !processing_failed_;
 }
 
 bool TLSSecAttr::equal(const SecAttr &b) const {
@@ -245,7 +312,8 @@ bool TLSSecAttr::Export(SecAttrFormat format,XMLNode &val) const {
     if(!voms_attributes_.empty()) {
       for(int k=0; k < voms_attributes_.size(); k++) {
         for(int n=0; n < voms_attributes_[k].attributes.size(); n++) {
-          add_arc_subject_attribute(subj, voms_attributes_[k].attributes[n],"http://www.nordugrid.org/schemas/policy-arc/types/tls/vomsattribute");
+          std::string vattr = VOMSFQANToFull(voms_attributes_[k].voname,voms_attributes_[k].attributes[n]);
+          add_arc_subject_attribute(subj,vattr,"http://www.nordugrid.org/schemas/policy-arc/types/tls/vomsattribute");
         };
       };
     };
@@ -282,7 +350,8 @@ bool TLSSecAttr::Export(SecAttrFormat format,XMLNode &val) const {
     if(!voms_attributes_.empty()) {
       for(int k=0; k < voms_attributes_.size(); k++) {
         for(int n=0; n < voms_attributes_[k].attributes.size(); n++) {
-          add_xacml_subject_attribute(subj, voms_attributes_[k].attributes[n],"http://www.nordugrid.org/schemas/policy-arc/types/tls/vomsattribute");
+          std::string vattr = VOMSFQANToFull(voms_attributes_[k].voname,voms_attributes_[k].attributes[n]);
+          add_xacml_subject_attribute(subj,vattr,"http://www.nordugrid.org/schemas/policy-arc/types/tls/vomsattribute");
         };
       };
     };
@@ -367,7 +436,6 @@ MCC_Status MCC_TLS_Service::process(Message& inmsg,Message& outmsg) {
    // Filling security attributes
    if(tstream && (config_.IfClientAuthn())) {
       TLSSecAttr* sattr = new TLSSecAttr(*tstream, config_, logger);
-      nextinmsg.Auth()->set("TLS",sattr);
       //Getting the subject name of peer(client) certificate
       logger.msg(VERBOSE, "Peer name: %s", sattr->Subject());
       nextinmsg.Attributes()->set("TLS:PEERDN",sattr->Subject());
@@ -375,10 +443,15 @@ MCC_Status MCC_TLS_Service::process(Message& inmsg,Message& outmsg) {
       nextinmsg.Attributes()->set("TLS:IDENTITYDN",sattr->Identity());
       logger.msg(VERBOSE, "CA name: %s", sattr->CA());
       nextinmsg.Attributes()->set("TLS:CADN",sattr->CA());
-
-      nextinmsg.Attributes()->set("TLS:PEERCERT",sattr->X509Str());
-      if(!((sattr->target_).empty()))
+      if(!((sattr->target_).empty())) {
         nextinmsg.Attributes()->set("TLS:LOCALDN",sattr->target_);
+      }
+      if(!*sattr) {
+        logger.msg(ERROR, "Failed to process security attributes in TLS MCC for incoming message");
+        delete sattr;
+        return MCC_Status();
+      }
+      nextinmsg.Auth()->set("TLS",sattr);
    }
 
    // Checking authentication and authorization;
