@@ -26,6 +26,8 @@ namespace DataStaging {
     return owner_name[proc];
   }
 
+  const Arc::URL DTR::LOCAL_DELIVERY("file:/local");
+
   DTR::DTR(const std::string& source,
            const std::string& destination,
            const Arc::UserConfig& usercfg,
@@ -47,16 +49,19 @@ namespace DataStaging {
        replication(false),
        force_registration(false),
        status(DTRStatus::NEW,"Created by the generator"),
+       bytes_transferred(0),
        created(time(NULL)),
        cancel_request(false),
+       delivery_endpoint(LOCAL_DELIVERY),
+       use_host_cert_for_remote_delivery(false),
        current_owner(GENERATOR),
-       logger(log),
-       log_destinations(logger->getDestinations())
+       logger(log)
   {
     if (!logger) {
       // use root logger if none is supplied
       logger = new Arc::Logger(Arc::Logger::getRootLogger(), "DTR");
     }
+    log_destinations = logger->getDestinations();
 
     // check that endpoints can be handled
     if (!source_endpoint) {
@@ -119,14 +124,18 @@ namespace DataStaging {
       mapped_source(dtr.mapped_source),
       status(dtr.status),
       error_status(dtr.error_status),
+      bytes_transferred(dtr.bytes_transferred),
       timeout(dtr.timeout),
       created(dtr.created),
       next_process_time(dtr.next_process_time),
       cancel_request(dtr.cancel_request),
+      delivery_endpoint(dtr.delivery_endpoint),
+      use_host_cert_for_remote_delivery(dtr.use_host_cert_for_remote_delivery),
       current_owner(dtr.current_owner),
       logger(dtr.logger),
       log_destinations(dtr.log_destinations),
-      proc_callback(dtr.proc_callback)
+      proc_callback(dtr.proc_callback),
+      lock()
   {
     // set insecure by default. Real value will come from configuration
     source_endpoint->SetSecure(false);
@@ -143,46 +152,41 @@ namespace DataStaging {
 
   }
 
-  DTR& DTR::operator=(const DTR& dtr) {
-    DTR_ID = dtr.DTR_ID;
-    source_url = dtr.source_url;
-    destination_url = dtr.destination_url;
-    cfg = dtr.cfg;
-    Arc::DataHandle src(source_url, cfg);
-    source_endpoint = src;
-    Arc::DataHandle dest(destination_url, cfg);
-    destination_endpoint = dest;
-    cache_file = dtr.cache_file;
-    cache_parameters = dtr.cache_parameters;
-    cache_state = dtr.cache_state;
-    user = dtr.user;
-    parent_job_id = dtr.parent_job_id;
-    priority = dtr.priority;
-    transfershare = dtr.transfershare;
-    sub_share = dtr.sub_share;
-    tries_left = dtr.tries_left;
-    replication = dtr.replication;
-    force_registration = dtr.force_registration;
-    mapped_source = dtr.mapped_source;
-    status = dtr.status;
-    error_status = dtr.error_status;
-    timeout = dtr.timeout;
-    created = dtr.created;
-    next_process_time = dtr.next_process_time;
-    cancel_request = dtr.cancel_request;
-    current_owner = dtr.current_owner;
-    logger = dtr.logger;
-    log_destinations = dtr.log_destinations;
-    proc_callback = dtr.proc_callback;
-    return *this;
-  }
-
   void DTR::registerCallback(DTRCallback* cb, StagingProcesses owner) {
     lock.lock();
     proc_callback[owner].push_back(cb);
     lock.unlock();
   }
 
+  void DTR::reset() {
+    // remove resolved locations
+    if (source_endpoint->IsIndex()) {
+      source_endpoint->ClearLocations();
+    }
+    // reset retry count to 1
+    source_endpoint->SetTries(1);
+
+    if (destination_endpoint->IsIndex()) {
+      destination_endpoint->ClearLocations();
+    }
+    destination_endpoint->SetTries(1);
+
+    // empty cache and map info
+    cache_file.clear();
+    mapped_source.clear();
+    bytes_transferred = 0;
+    reset_error_status();
+  }
+
+  void DTR::set_id(const std::string& id) {
+    // sanity check - regular expressions would be useful here
+    if (id.length() != DTR_ID.length()) {
+      logger->msg(Arc::WARNING, "Invalid ID: %s", id);
+    } else {
+      DTR_ID = id;
+    }
+  }
+
   std::string DTR::get_short_id() const {
     if(DTR_ID.length() < 8) return DTR_ID;
     std::string short_id(DTR_ID.substr(0,4)+"..."+DTR_ID.substr(DTR_ID.length()-4));
@@ -247,6 +251,11 @@ namespace DataStaging {
     return s;
   }
 
+  void DTR::set_bytes_transferred(unsigned long long int bytes) {
+    bytes_transferred = bytes;
+    mark_modification();
+  }
+
   void DTR::set_cache_file(const std::string& filename)
   {
     cache_file = filename;
@@ -391,41 +400,4 @@ namespace DataStaging {
        drain_cache_dirs(drain_caches) {
 }
 
-std::ostream& operator<<(std::ostream& stream, const CacheParameters& obj) {
-  for(std::vector<std::string>::const_iterator d = obj.cache_dirs.begin();
-                      d != obj.cache_dirs.end();++d) {
-    stream<<"cache="<<*d<<std::endl;
-  };
-  for(std::vector<std::string>::const_iterator d = obj.remote_cache_dirs.begin();
-                      d != obj.remote_cache_dirs.end();++d) {
-    stream<<"remotecache="<<*d<<std::endl;
-  };
-  for(std::vector<std::string>::const_iterator d = obj.drain_cache_dirs.begin();
-                      d != obj.drain_cache_dirs.end();++d) {
-    stream<<"draincache="<<*d<<std::endl;
-  };
-  return stream;
-}
-
-std::istream& operator>>(std::istream& stream, CacheParameters& obj) {
-  obj.cache_dirs.clear();
-  obj.remote_cache_dirs.clear();
-  obj.drain_cache_dirs.clear();
-  std::string s;
-  while(std::getline(stream,s)) {
-    std::string::size_type p = s.find('=');
-    if(p == std::string::npos) continue;
-    std::string key = s.substr(0,p);
-    if(key == "cache") {
-      obj.cache_dirs.push_back(s.substr(p+1));
-    } else if(key == "remotecache") {
-      obj.remote_cache_dirs.push_back(s.substr(p+1));
-    } else if(key == "draincache") {
-      obj.drain_cache_dirs.push_back(s.substr(p+1));
-    };
-  };
-  return stream;
-}
-
-
 } // namespace DataStaging
