@@ -3,23 +3,29 @@
 #endif
 
 #include "DataDeliveryComm.h"
-
 #include "DataDelivery.h"
 
 namespace DataStaging {
 
   Arc::Logger DataDelivery::logger(Arc::Logger::getRootLogger(), "DataStaging.DataDelivery");
 
+  /// Wrapper class around DataDeliveryComm
   class DataDelivery::delivery_pair_t {
     public:
     DTR* dtr;
-    DataDeliveryComm comm;
+    DataDeliveryComm* comm;
     bool cancelled;
     delivery_pair_t(DTR* request, const TransferParameters& params);
+    ~delivery_pair_t();
   };
 
   DataDelivery::delivery_pair_t::delivery_pair_t(DTR* request, const TransferParameters& params)
-    :dtr(request),comm(*request, params),cancelled(false) {
+    :dtr(request),cancelled(false) {
+    comm = DataDeliveryComm::CreateInstance(*request, params);
+  }
+
+  DataDelivery::delivery_pair_t::~delivery_pair_t() {
+    delete comm;
   }
 
   DataDelivery::DataDelivery(): delivery_state(INITIATED) {
@@ -35,27 +41,23 @@ namespace DataStaging {
   void DataDelivery::receiveDTR(DTR& dtr) {
     if(!dtr) {
       logger.msg(Arc::ERROR, "Received invalid DTR");
-      dtr.set_error_status(DTRErrorStatus::INTERNAL_ERROR, DTRErrorStatus::ERROR_UNKNOWN, "Invalid DTR");
+      dtr.set_error_status(DTRErrorStatus::INTERNAL_LOGIC_ERROR, DTRErrorStatus::ERROR_UNKNOWN, "Invalid DTR");
       dtr.set_status(DTRStatus::TRANSFERRED);
       dtr.push(SCHEDULER);
       return;
     }
     dtr.get_logger()->msg(Arc::INFO, "Delivery received new DTR %s with source: %s, destination: %s",
                dtr.get_id(), dtr.get_source()->CurrentLocation().str(), dtr.get_destination()->CurrentLocation().str());
-    /*
-     *  Change the status of the dtr to TRANSFERRING	 
-     *  Start reading from the source into a buffer
-     *  TODO: Complete reading the file   
-     *  TODO: Do the checksome 
-     */
+
     dtr.set_status(DTRStatus::TRANSFERRING);
     delivery_pair_t* d = new delivery_pair_t(&dtr, transfer_params);
-    if(d->comm) {
+    if(*(d->comm)) {
       dtr_list_lock.lock();
       dtr_list.push_back(d);
       dtr_list_lock.unlock();
     } else {
-      dtr.set_error_status(DTRErrorStatus::INTERNAL_ERROR, DTRErrorStatus::ERROR_UNKNOWN, "Failed to start Delivery process");
+      delete d;
+      dtr.set_error_status(DTRErrorStatus::INTERNAL_PROCESS_ERROR, DTRErrorStatus::ERROR_UNKNOWN, "Failed to start Delivery process");
       dtr.set_status(DTRStatus::TRANSFERRED);
       dtr.push(SCHEDULER);
     }
@@ -129,12 +131,8 @@ namespace DataStaging {
         dtr_list_lock.unlock();
         delivery_pair_t* dp = *d;
         DataDeliveryComm::Status status;
-        status = dp->comm.GetStatus();
-        // TODO: fill status into DTR
-        //std::cerr<<"Time: "<<status.timestamp
-        //         <<", Comm. Status: "<<status.commstatus
-        //         <<", Status: "<<status.status
-        //         <<", Bytes: "<<status.transfered<<"/"<<status.size<<std::endl;
+        status = dp->comm->GetStatus();
+        dp->dtr->set_bytes_transferred(status.transferred);
 
         // check for cancellation
         if (dp->cancelled) {
@@ -159,23 +157,28 @@ namespace DataStaging {
           if((status.commstatus == DataDeliveryComm::CommFailed) ||
              (status.error != DTRErrorStatus::NONE_ERROR)) {
             if(status.error == DTRErrorStatus::NONE_ERROR)
-              status.error = DTRErrorStatus::INTERNAL_ERROR;
+              status.error = DTRErrorStatus::INTERNAL_PROCESS_ERROR;
             dp->dtr->set_error_status(status.error,status.error_location,
-                     status.error_desc[0]?status.error_desc:dp->comm.GetError().c_str());
+                     status.error_desc[0]?status.error_desc:dp->comm->GetError().c_str());
+          } else if (status.checksum) {
+            dp->dtr->get_destination()->SetCheckSum(status.checksum);
           }
+          dp->dtr->get_logger()->msg(Arc::INFO, "DTR %s: Transfer finished: %llu bytes transferred %s",
+                                     dp->dtr->get_short_id(), status.transferred,
+                                     (status.checksum[0] ? ": checksum "+std::string(status.checksum) : " "));
           dp->dtr->set_status(DTRStatus::TRANSFERRED);
           dp->dtr->push(SCHEDULER);
           delete dp;
           continue;
         }
-        if(!(dp->comm)) {
+        if(!(*(dp->comm))) {
           // Error happened
           // comm.GetError()
           dtr_list_lock.lock();
           d = dtr_list.erase(d);
           dtr_list_lock.unlock();
-          dp->dtr->set_error_status(DTRErrorStatus::INTERNAL_ERROR,DTRErrorStatus::ERROR_TRANSFER,
-                   dp->comm.GetError().empty()?"Connection with delivery process lost":dp->comm.GetError());
+          dp->dtr->set_error_status(DTRErrorStatus::INTERNAL_PROCESS_ERROR,DTRErrorStatus::ERROR_TRANSFER,
+                   dp->comm->GetError().empty()?"Connection with delivery process lost":dp->comm->GetError());
           dp->dtr->set_status(DTRStatus::TRANSFERRED);
           dp->dtr->push(SCHEDULER);
           delete dp;
@@ -189,6 +192,14 @@ namespace DataStaging {
       // TODO: replace with condition
       Glib::usleep(500000);
     }
+    // Kill any transfers still running
+    dtr_list_lock.lock();
+    for (std::list<delivery_pair_t*>::iterator d = dtr_list.begin(); d != dtr_list.end();) {
+      delete *d;
+      d = dtr_list.erase(d);
+    }
+    dtr_list_lock.unlock();
+
     logger.msg(Arc::INFO, "Data delivery loop exited");
     run_signal.signal();
   }
