@@ -32,6 +32,7 @@ namespace DataStaging {
   void Processor::DTRCheckCache(void* arg) {
     ThreadArgument* targ = (ThreadArgument*)arg;
     DTR* request = targ->dtr;
+    delete targ;
 
     setUpLogger(request);
 
@@ -40,6 +41,8 @@ namespace DataStaging {
 
     // Create cache using configuration
     Arc::FileCache cache(request->get_cache_parameters().cache_dirs,
+                         request->get_cache_parameters().remote_cache_dirs,
+                         request->get_cache_parameters().drain_cache_dirs,
                          request->get_parent_job_id(),
                          request->get_local_user().get_uid(),
                          request->get_local_user().get_gid());
@@ -70,8 +73,9 @@ namespace DataStaging {
 
     bool is_in_cache = false;
     bool is_locked = false;
+    bool use_remote = true;
     for (;;) {
-      if (!cache.Start(canonic_url, is_in_cache, is_locked)) {
+      if (!cache.Start(canonic_url, is_in_cache, is_locked, use_remote)) {
         if (is_locked) {
           request->get_logger()->msg(Arc::WARNING, "DTR %s: Cached file is locked - should retry", request->get_short_id());
           request->set_cache_state(CACHE_LOCKED);
@@ -106,6 +110,7 @@ namespace DataStaging {
         if (cache_option == "renew") {
           request->get_logger()->msg(Arc::VERBOSE, "DTR %s: Forcing re-download of file %s", request->get_short_id(), canonic_url);
           cache.StopAndDelete(canonic_url);
+          use_remote = false;
           continue;
         }
         // just need to check permissions
@@ -152,6 +157,7 @@ namespace DataStaging {
         if (outdated) {
           cache.StopAndDelete(canonic_url);
           request->get_logger()->msg(Arc::INFO, "DTR %s: Cached file is outdated, will re-download", request->get_short_id());
+          use_remote = false;
           continue;
         }
         // cached file is present and valid
@@ -160,6 +166,7 @@ namespace DataStaging {
       }
       else { // file is not there but we are ready to download it
         request->get_logger()->msg(Arc::VERBOSE, "DTR %s: Will download to cache file %s", request->get_short_id(), request->get_cache_file());
+        request->set_cache_state(CACHEABLE);
       }
       break;
     }
@@ -174,6 +181,7 @@ namespace DataStaging {
     // call request.destination.PreRegister() to lock destination LFN
     ThreadArgument* targ = (ThreadArgument*)arg;
     DTR* request = targ->dtr;
+    delete targ;
 
     setUpLogger(request);
 
@@ -243,6 +251,7 @@ namespace DataStaging {
     // check source is ok and obtain metadata
     ThreadArgument* targ = (ThreadArgument*)arg;
     DTR* request = targ->dtr;
+    delete targ;
 
     setUpLogger(request);
 
@@ -285,6 +294,7 @@ namespace DataStaging {
     // only if the entry already exists
     ThreadArgument* targ = (ThreadArgument*)arg;
     DTR* request = targ->dtr;
+    delete targ;
     setUpLogger(request);
 
     Arc::DataStatus res = Arc::DataStatus::Success;
@@ -357,6 +367,7 @@ namespace DataStaging {
     // and/or request.destination.PrepareWriting() to get TURL for writing or query status of request
     ThreadArgument* targ = (ThreadArgument*)arg;
     DTR* request = targ->dtr;
+    delete targ;
 
     setUpLogger(request);
 
@@ -375,6 +386,7 @@ namespace DataStaging {
       else if (res == Arc::DataStatus::ReadPrepareWait) {
         // if timeout then don't wait - scheduler will deal with it immediately
         if (Arc::Time() < request->get_timeout()) {
+          if (source_wait_time > 60) source_wait_time = 60;
           request->set_process_time(source_wait_time);
           request->get_logger()->msg(Arc::VERBOSE, "DTR %s: Source is not ready, will wait %u seconds", request->get_short_id(), source_wait_time);
         }
@@ -412,6 +424,7 @@ namespace DataStaging {
       else if (res == Arc::DataStatus::WritePrepareWait) {
         // if timeout then don't wait - scheduler will deal with it immediately
         if (Arc::Time() < request->get_timeout()) {
+          if (dest_wait_time > 60) dest_wait_time = 60;
           request->set_process_time(dest_wait_time);
           request->get_logger()->msg(Arc::VERBOSE, "DTR %s: Destination is not ready, will wait %u seconds", request->get_short_id(), dest_wait_time);
         }
@@ -440,6 +453,7 @@ namespace DataStaging {
     // request.destination.FinishWriting() to release or abort requests
     ThreadArgument* targ = (ThreadArgument*)arg;
     DTR* request = targ->dtr;
+    delete targ;
     setUpLogger(request);
 
     Arc::DataStatus res;
@@ -481,6 +495,7 @@ namespace DataStaging {
     // RESOLVE stage for error workflow
     ThreadArgument* targ = (ThreadArgument*)arg;
     DTR* request = targ->dtr;
+    delete targ;
     setUpLogger(request);
 
     // TODO: If the copy completed before request was cancelled, unregistering
@@ -516,9 +531,12 @@ namespace DataStaging {
     // of error or deciding not to use cache (for example because of a mapped link)
     ThreadArgument* targ = (ThreadArgument*)arg;
     DTR* request = targ->dtr;
+    delete targ;
     setUpLogger(request);
 
     Arc::FileCache cache(request->get_cache_parameters().cache_dirs,
+                         request->get_cache_parameters().remote_cache_dirs,
+                         request->get_cache_parameters().drain_cache_dirs,
                          request->get_parent_job_id(),
                          request->get_local_user().get_uid(),
                          request->get_local_user().get_gid());
@@ -539,7 +557,12 @@ namespace DataStaging {
 
     // check for error, cancellation or cache not being used
     if (request->error() || request->cancel_requested() || request->get_cache_state() == CACHE_NOT_USED) {
-      (request->error() || request->cancel_requested()) ? cache.StopAndDelete(canonic_url) : cache.Stop(canonic_url);
+      // don't delete cache file if it was already present or successfully downloaded
+      if (request->error() || (request->cancel_requested() && request->get_cache_state() == CACHEABLE)) {
+        cache.StopAndDelete(canonic_url);
+      } else {
+        cache.Stop(canonic_url);
+      }
       request->set_status(DTRStatus::CACHE_PROCESSED);
       request->connect_logger();
       request->push(SCHEDULER);
@@ -621,7 +644,7 @@ namespace DataStaging {
 
       default: {
         // unexpected state - report error
-        request.set_error_status(DTRErrorStatus::INTERNAL_ERROR,
+        request.set_error_status(DTRErrorStatus::INTERNAL_LOGIC_ERROR,
                               DTRErrorStatus::ERROR_UNKNOWN,
                               "Received a DTR in an unexpected state ("+request.get_status().str()+") in processor");
         request.push(SCHEDULER);
