@@ -11,6 +11,7 @@
 #include <arc/Utils.h>
 
 #include "Scheduler.h"
+#include "DataDeliveryRemoteComm.h"
 
 namespace DataStaging {
 	
@@ -73,12 +74,29 @@ namespace DataStaging {
     delivery.SetTransferParameters(params);
   }
 
+  void Scheduler::SetDeliveryServices(const std::vector<Arc::URL>& endpoints) {
+    if (scheduler_state == INITIATED)
+      delivery_services = endpoints;
+  }
+
+  void Scheduler::SetDumpLocation(const std::string& location) {
+    dumplocation = location;
+  }
+
 
   bool Scheduler::start(void) {
     if(scheduler_state == RUNNING || scheduler_state == TO_STOP) return false;
     scheduler_state = RUNNING;
     processor.start();
     delivery.start();
+    // if no delivery services set, then use local
+    if (delivery_services.empty()) {
+      std::vector<Arc::URL> services;
+      services.push_back(DTR::LOCAL_DELIVERY);
+      delivery_services = services;
+    }
+    DeliverySlots *= delivery_services.size();
+    DeliveryEmergencySlots *= delivery_services.size();
     Arc::CreateThreadFunction(&main_thread, this);
     return true;
   }
@@ -95,7 +113,7 @@ namespace DataStaging {
 
   void Scheduler::next_replica(DTR* request) {
     if (!request->error()) { // bad logic
-      request->set_error_status(DTRErrorStatus::INTERNAL_ERROR,
+      request->set_error_status(DTRErrorStatus::INTERNAL_LOGIC_ERROR,
                                 DTRErrorStatus::ERROR_UNKNOWN,
                                 "Bad logic: next_replica called when there is no error");
       // TODO: how to deal with these internal errors?
@@ -216,6 +234,11 @@ namespace DataStaging {
     // Fresh DTRs should receive the initial priority
     // This is to be implemented
     //compute_priority(request);
+    request->get_logger()->msg(Arc::INFO, "Scheduler received new DTR %s with source: %s,"
+        " destination: %s, assigned to transfer share %s with priority %d",
+        request->get_id(), request->get_source()->str(), request->get_destination()->str(),
+        request->get_transfer_share(), request->get_priority());
+
     // Normal workflow is CHECK_CACHE
     if (request->get_cache_state() == NON_CACHEABLE || request->get_cache_parameters().cache_dirs.empty()) {
       request->get_logger()->msg(Arc::VERBOSE, "DTR %s: File is not cacheable, was requested not to be cached or no cache available, skipping cache check", request->get_short_id());
@@ -346,12 +369,24 @@ namespace DataStaging {
     if (request->get_source()->IsStageable() || request->get_destination()->IsStageable()) {
       // Normal workflow is STAGE_PREPARE
 
-      // Apply primitive limit to staging - in future may be better to limit per remote host
-      // TODO: non-stageable transfers can block staging ones, so only count
-      // stageable transfers in the delivery queue. Also one share may block others.
+      // Apply limit to staging to avoid preparing too many files and then
+      // pins expire while in the transfer queue. In future it may be better
+      // to limit per remote host. For now count staging transfers in this
+      // share already in transfer queue and apply limit. In order not to block
+      // the highest priority DTRs here we allow them to bypass the limit.
       std::list<DTR*> DeliveryQueue;
       DtrList.filter_dtrs_by_next_receiver(DELIVERY,DeliveryQueue);
-      if (DeliveryQueue.size() >= DeliverySlots*2) {
+
+      int share_queue = 0, highest_priority = 0;
+      for (std::list<DTR*>::iterator dtr = DeliveryQueue.begin(); dtr != DeliveryQueue.end(); ++dtr) {
+        if ((*dtr)->get_transfer_share() == request->get_transfer_share() &&
+            ((*dtr)->get_source()->IsStageable() ||
+             (*dtr)->get_destination()->IsStageable())) {
+          ++share_queue;
+          if ((*dtr)->get_priority() > highest_priority) highest_priority = (*dtr)->get_priority();
+        }
+      }
+      if (share_queue >= DeliverySlots*2 && request->get_priority() <= highest_priority) {
         request->get_logger()->msg(Arc::INFO, "DTR %s: Large transfer queue - will wait 10s before staging", request->get_short_id());
         request->set_process_time(10);
       }
@@ -541,18 +576,23 @@ namespace DataStaging {
         // Here we decide to retry based on whether the error is
         // temporary or not and the configured retry strategy
         if (request->get_error_status().GetErrorStatus() == DTRErrorStatus::TEMPORARY_REMOTE_ERROR ||
-            request->get_error_status().GetErrorStatus() == DTRErrorStatus::TRANSFER_SPEED_ERROR) {
+            request->get_error_status().GetErrorStatus() == DTRErrorStatus::TRANSFER_SPEED_ERROR ||
+            request->get_error_status().GetErrorStatus() == DTRErrorStatus::INTERNAL_PROCESS_ERROR) {
           if (request->get_tries_left() > 0) {
             request->set_process_time(10);
             request->get_logger()->msg(Arc::INFO, "DTR %s: %i retries left, will wait until %s before next attempt",
                                        request->get_short_id(), request->get_tries_left(), request->get_process_time().str());
             // set state depending on where the error occurred
-            if (request->get_error_status().GetLastErrorState() == DTRStatus::REGISTERING_REPLICA)
+            if (request->get_error_status().GetLastErrorState() == DTRStatus::REGISTERING_REPLICA) {
               request->set_status(DTRStatus::REGISTER_REPLICA);
-            else if (request->get_error_status().GetLastErrorState() == DTRStatus::RELEASING_REQUEST)
+            } else if (request->get_error_status().GetLastErrorState() == DTRStatus::RELEASING_REQUEST) {
               request->set_status(DTRStatus::RELEASE_REQUEST);
-            else // if error happened before or during transfer set back to NEW
+            } else {
+              // If error happened before or during transfer set back to NEW
+              // Reset DTR information set during this transfer
+              request->reset();
               request->set_status(DTRStatus::NEW);
+            }
             return;
           }
           else
@@ -699,13 +739,19 @@ namespace DataStaging {
   void Scheduler::revise_pre_processor_queue()
   {
     std::list<DTR*> PreProcessorQueue;
-    std::list<DTR*>::iterator dtr;
-
     DtrList.filter_dtrs_by_next_receiver(PRE_PROCESSOR,PreProcessorQueue);
 
+    if (PreProcessorQueue.empty()) return;
+
+    // Sort the queue by priority
+    PreProcessorQueue.sort(dtr_sort_predicate);
+
     DTR* tmp;
 
-    for(dtr = PreProcessorQueue.begin(); dtr != PreProcessorQueue.end();){
+    std::list<DTR*>::iterator dtr = PreProcessorQueue.begin();
+    int highest_priority = (*dtr)->get_priority();
+
+    while (dtr != PreProcessorQueue.end()) {
 
       tmp = *dtr;
       // The cancellation requests break the normal workflow. A cancelled
@@ -717,31 +763,45 @@ namespace DataStaging {
         continue;
       }
 
-      // DTRs that have passed their timeout should have the
-      // priority boosted high. Alternative solution -- recompute
-      // the priority with compute_priority() function, so this
-      // function contains all the logic, but it may
-      // take too long time
-      if(tmp->get_timeout() < time(NULL)){
-        tmp->set_priority(tmp->get_priority() + 100);
+      // To avoid the situation where DTRs get blocked due to higher
+      // priority DTRs, DTRs that have passed their timeout should have their
+      // priority boosted. But this should only happen if there are higher
+      // priority DTRs, since there could be a large queue of low priority DTRs
+      // which, after having their priority boosted, would then block new
+      // high priority requests.
+      // The simple solution here is to increase priority by 1 every 5 minutes.
+      // There is plenty of scope for more intelligent solutions.
+      // TODO reset priority back to original value after pre-processing
+      if(tmp->get_timeout() < time(NULL) && tmp->get_priority() < highest_priority){
+        tmp->set_priority(tmp->get_priority() + 1);
+        tmp->set_timeout(300);
       }
       ++dtr;
     }
 
-    // No sorting of the queue -- make it FIFO for the time being
-    // PreProcessorQueue.sort(dtr_sort_predicate);
+    transferShares.calculate_shares(PreProcessorSlots);
 
-    // Reset the number of the DTRs running in the pre-processor
-    int PreProcessorRunning = DtrList.number_of_dtrs_by_owner(PRE_PROCESSOR);
+    std::list<DTR*> InPreProcessor;
+    DtrList.filter_dtrs_by_owner(PRE_PROCESSOR, InPreProcessor);
 
-    // Now at the end of the list are DTRs that should be launched first
+    // Number of the DTRs running in the pre-processor
+    int PreProcessorRunning = InPreProcessor.size();
+    if (PreProcessorRunning == PreProcessorSlots) return;
+
+    // Decrease shares for those already in the pre-processor
+    for (dtr = InPreProcessor.begin(); dtr != InPreProcessor.end(); ++dtr) {
+      transferShares.decrease_number_of_slots((*dtr)->get_transfer_share());
+    }
+
+    // Send to pre-processor according to share and place in the queue
     while(PreProcessorRunning < PreProcessorSlots && !PreProcessorQueue.empty()){
-      // Push to the pre-processor, register in the list
-      // of processed DTRs and delete from the queue
-      tmp = PreProcessorQueue.back();
-      tmp->push(PRE_PROCESSOR);
-      PreProcessorQueue.pop_back();
-      PreProcessorRunning++;
+      tmp = PreProcessorQueue.front();
+      PreProcessorQueue.pop_front();
+      if (transferShares.can_start(tmp->get_transfer_share())) {
+        tmp->push(PRE_PROCESSOR);
+        PreProcessorRunning++;
+        transferShares.decrease_number_of_slots(tmp->get_transfer_share());
+      }
     }
   }
   
@@ -758,52 +818,72 @@ namespace DataStaging {
     // cleanup activities.
 
     std::list<DTR*> PostProcessorQueue;
-    std::list<DTR*>::iterator dtr;
-    DTR* tmp;
-
     DtrList.filter_dtrs_by_next_receiver(POST_PROCESSOR,PostProcessorQueue);
 
-    for(dtr = PostProcessorQueue.begin(); dtr != PostProcessorQueue.end(); dtr++){
+    if (PostProcessorQueue.empty()) return;
+
+    // Sort the queue by priority
+    PostProcessorQueue.sort(dtr_sort_predicate);
+
+    DTR* tmp;
+
+    std::list<DTR*>::iterator dtr = PostProcessorQueue.begin();
+    int highest_priority = (*dtr)->get_priority();
+
+    while (dtr != PostProcessorQueue.end()) {
 
       tmp = *dtr;
 
-      // DTRs that have passed their timeout should have the
-      // priority boosted high. Alternative solution -- recompute
-      // the priority with compute_priority() function, so this
-      // function contains all the logic, but it may
-      // take too long time
-      if(tmp->get_timeout() < time(NULL)){
-        tmp->set_priority(tmp->get_priority()+100);
+      // see revise_pre_processor_queue() for explanation
+      if(tmp->get_timeout() < time(NULL) && tmp->get_priority() < highest_priority){
+        tmp->set_priority(tmp->get_priority() + 1);
+        tmp->set_timeout(300);
       }
-
+      ++dtr;
     }
 
-    // No sorting of the queue -- make it FIFO for the time being
-    // PostProcessorQueue.sort(dtr_sort_predicate);
+    transferShares.calculate_shares(PostProcessorSlots);
 
-    // Reset the number of the DTRs running in the pre-processor
-    int PostProcessorRunning = DtrList.number_of_dtrs_by_owner(POST_PROCESSOR);
+    std::list<DTR*> InPostProcessor;
+    DtrList.filter_dtrs_by_owner(POST_PROCESSOR, InPostProcessor);
 
-    // Now at the end of the list are DTRs that should be launched first
+    // Number of the DTRs running in the post-processor
+    int PostProcessorRunning = InPostProcessor.size();
+    if (PostProcessorRunning == PostProcessorSlots) return;
+
+    // Decrease shares for those already in the post-processor
+    for (dtr = InPostProcessor.begin(); dtr != InPostProcessor.end(); ++dtr) {
+      transferShares.decrease_number_of_slots((*dtr)->get_transfer_share());
+    }
+
+    // Send to pre-processor according to share and place in the queue
     while(PostProcessorRunning < PostProcessorSlots && !PostProcessorQueue.empty()){
-      // Push to the post-processor, register in the list
-      // of processed DTRs and delete from the queue
-      tmp = PostProcessorQueue.back();
-      tmp->push(POST_PROCESSOR);
-      PostProcessorQueue.pop_back();
-      PostProcessorRunning++;
+      tmp = PostProcessorQueue.front();
+      PostProcessorQueue.pop_front();
+      if (transferShares.can_start(tmp->get_transfer_share())) {
+        tmp->push(POST_PROCESSOR);
+        PostProcessorRunning++;
+        transferShares.decrease_number_of_slots(tmp->get_transfer_share());
+      }
     }
   }
   
   void Scheduler::revise_delivery_queue()
   {
     std::list<DTR*> DeliveryQueue;    
-    std::list<DTR*>::iterator dtr;
-    DTR* tmp;
-
     DtrList.filter_dtrs_by_next_receiver(DELIVERY,DeliveryQueue);
 
-    for(dtr = DeliveryQueue.begin(); dtr != DeliveryQueue.end();){
+    // Sort the Delivery Queue according to
+    // the priorities the DTRs have.
+    DeliveryQueue.sort(dtr_sort_predicate);
+
+    DTR* tmp;
+    int highest_priority = 0;
+
+    std::list<DTR*>::iterator dtr = DeliveryQueue.begin();
+
+    while (dtr != DeliveryQueue.end()) {
+      if (dtr == DeliveryQueue.begin()) highest_priority = (*dtr)->get_priority();
 
       tmp = *dtr;
       // The cancellation requests break the normal workflow. A cancelled
@@ -815,13 +895,10 @@ namespace DataStaging {
         continue;
       }
 
-      // DTRs that have passed their timeout should have the
-      // priority boosted high. Alternative solution -- recompute
-      // the priority with compute_priority() function, so this
-      // function contains all the logic, but it may
-      // take too long time
-      if(tmp->get_timeout() < time(NULL)){
-        tmp->set_priority(tmp->get_priority()+100);
+      // see revise_pre_processor_queue() for explanation
+      if(tmp->get_timeout() < time(NULL) && tmp->get_priority() < highest_priority){
+        tmp->set_priority(tmp->get_priority() + 1);
+        tmp->set_timeout(300);
       }
       dtr++;
     }
@@ -864,10 +941,6 @@ namespace DataStaging {
     // Refresh the number of DTRs running in the Delivery,
     int DeliveryRunning = DtrList.number_of_dtrs_by_owner(DELIVERY);
 
-    // Sort the Delivery Queue according to 
-    // the priorities the DTRs have.
-    DeliveryQueue.sort(dtr_sort_predicate);
-    
     // Now at the beginning of the queue we have DTRs that
     // should be launched first. Launch them, but with respect
     // to the transfer shares.
@@ -880,6 +953,11 @@ namespace DataStaging {
           break;
         if ((shares_in_delivery.find(tmp->get_transfer_share()) == shares_in_delivery.end()) &&
             transferShares.can_start(tmp->get_transfer_share())) {
+          // choose delivery service - random for now
+          // if this is a retry, try to use a different service
+          if (!delivery_services.empty()) {
+            tmp->set_delivery_endpoint(delivery_services.at(rand() % delivery_services.size()));
+          }
           transferShares.decrease_number_of_slots(tmp->get_transfer_share());
           tmp->set_status(DTRStatus::TRANSFER);
           tmp->push(DELIVERY);
@@ -888,6 +966,11 @@ namespace DataStaging {
         }
       }
       else if(transferShares.can_start(tmp->get_transfer_share())){
+        // choose delivery service - random for now
+        // if this is a retry, try to use a different service
+        if (!delivery_services.empty()) {
+          tmp->set_delivery_endpoint(delivery_services.at(rand() % delivery_services.size()));
+        }
       	transferShares.decrease_number_of_slots(tmp->get_transfer_share());
         tmp->set_status(DTRStatus::TRANSFER);
         tmp->push(DELIVERY);
@@ -903,8 +986,6 @@ namespace DataStaging {
        // If DTR is not NEW scheduler will pick it up itself.
        return;
     }
-    request.get_logger()->msg(Arc::INFO, "Scheduler received new DTR %s with source: %s, destination: %s",
-               request.get_id(), request.get_source()->str(), request.get_destination()->str());
     
     request.registerCallback(&processor,PRE_PROCESSOR);
     request.registerCallback(&processor,POST_PROCESSOR);
@@ -937,7 +1018,6 @@ namespace DataStaging {
     // This is the priority of the share adjusted by the priority
     // of the parent job
     request.set_priority(int(transferShares.get_basic_priority(DtrTransferShare) * request.get_priority() * 0.01));
-    request.get_logger()->msg(Arc::INFO,"DTR %s: Assigned to transfer share %s with priority %d",request.get_short_id(),DtrTransferShare,request.get_priority());
     /* Shares part ends*/               
     
     DtrList.add_dtr(request);
@@ -984,6 +1064,10 @@ namespace DataStaging {
     logger.msg(Arc::INFO, "  Emergency Delivery slots: %i", DeliveryEmergencySlots);
     logger.msg(Arc::INFO, "  Post-processor slots: %i", PostProcessorSlots);
     logger.msg(Arc::INFO, "  Shares configuration:\n%s", transferShares.conf());
+    for (std::vector<Arc::URL>::iterator i = delivery_services.begin(); i != delivery_services.end(); ++i) {
+      if (*i == DTR::LOCAL_DELIVERY) logger.msg(Arc::INFO, "  Delivery service: LOCAL");
+      else logger.msg(Arc::INFO, "  Delivery service: %s", i->str());
+    }
 
     // disconnect from root logger
     // messages are logged to per-DTR Logger
@@ -1024,15 +1108,16 @@ namespace DataStaging {
                              DtrList.number_of_dtrs_by_owner(DELIVERY),
                              DtrList.number_of_dtrs_by_owner(POST_PROCESSOR));
       // every 5 seconds, dump state
-      if (Arc::Time().GetTime() % 5 == 0) {
+      if (!dumplocation.empty() && Arc::Time().GetTime() % 5 == 0) {
         if (dump) {
-          DtrList.dumpState("/tmp/dtrstate.log");
+          DtrList.dumpState(dumplocation);
           dump = false;
         }
       } else
         dump = true;
-      usleep(50000);
+      Glib::usleep(50000);
     }
+    DtrList.dumpState(dumplocation);
     logger.msg(Arc::INFO, "Scheduler loop exited");
     run_signal.signal();
   }
