@@ -44,8 +44,32 @@ static const std::string BES_MANAGEMENT_NAMESPACE("http://schemas.ggf.org/bes/20
 static const std::string BES_ARC_NPREFIX("a-rex");
 static const std::string BES_ARC_NAMESPACE("http://www.nordugrid.org/schemas/a-rex");
 
-static const std::string BES_GLUE_NPREFIX("glue");
-static const std::string BES_GLUE_NAMESPACE("http://schemas.ogf.org/glue/2008/05/spec_2.0_d41_r01");
+static const std::string DELEG_ARC_NPREFIX("arcdeleg");
+static const std::string DELEG_ARC_NAMESPACE("http://www.nordugrid.org/schemas/delegation");
+
+static const std::string BES_GLUE2PRE_NPREFIX("glue2pre");
+static const std::string BES_GLUE2PRE_NAMESPACE("http://schemas.ogf.org/glue/2008/05/spec_2.0_d41_r01");
+
+static const std::string BES_GLUE2_NPREFIX("glue2");
+static const std::string BES_GLUE2_NAMESPACE("http://schemas.ogf.org/glue/2009/03/spec/2/0");
+
+static const std::string ES_TYPES_NPREFIX("estypes");
+static const std::string ES_TYPES_NAMESPACE("http://www.eu-emi.eu/es/2010/12/types");
+
+static const std::string ES_CREATE_NPREFIX("escreate");
+static const std::string ES_CREATE_NAMESPACE("http://www.eu-emi.eu/es/2010/12/creation");
+
+static const std::string ES_DELEG_NPREFIX("esdeleg");
+static const std::string ES_DELEG_NAMESPACE("http://www.eu-emi.eu/es/2010/12/delegation");
+
+static const std::string ES_RINFO_NPREFIX("esrinfo");
+static const std::string ES_RINFO_NAMESPACE("http://www.eu-emi.eu/es/2010/12/resourceinfo");
+
+static const std::string ES_MANAG_NPREFIX("esmanag");
+static const std::string ES_MANAG_NAMESPACE("http://www.eu-emi.eu/es/2010/12/activitymanagement");
+
+static const std::string ES_AINFO_NPREFIX("esainfo");
+static const std::string ES_AINFO_NAMESPACE("http://www.eu-emi.eu/es/2010/12/activity");
 
 
 #define AREX_POLICY_OPERATION_URN "http://www.nordugrid.org/schemas/policy-arc/types/a-rex/operation"
@@ -81,45 +105,117 @@ ARexSecAttr::ARexSecAttr(const std::string& action) {
 }
 
 ARexSecAttr::ARexSecAttr(const Arc::XMLNode op) {
-  if(MatchXMLName(op,"CreateActivity")) {
-    id_=JOB_POLICY_OPERATION_URN;
-    action_=JOB_POLICY_OPERATION_CREATE;
-  } else if(MatchXMLName(op,"GetActivityStatuses")) {
-    id_=JOB_POLICY_OPERATION_URN;
-    action_=JOB_POLICY_OPERATION_READ;
-  } else if(MatchXMLName(op,"TerminateActivities")) {
-    id_=JOB_POLICY_OPERATION_URN;
-    action_=JOB_POLICY_OPERATION_MODIFY;
-  } else if(MatchXMLName(op,"GetActivityDocuments")) {
-    id_=JOB_POLICY_OPERATION_URN;
-    action_=JOB_POLICY_OPERATION_READ;
-  } else if(MatchXMLName(op,"GetFactoryAttributesDocument")) {
-    id_=AREX_POLICY_OPERATION_URN;
-    action_=AREX_POLICY_OPERATION_INFO;
-  } else if(MatchXMLName(op,"StopAcceptingNewActivities")) {
-    id_=AREX_POLICY_OPERATION_URN;
-    action_=AREX_POLICY_OPERATION_ADMIN;
-  } else if(MatchXMLName(op,"StartAcceptingNewActivities")) {
-    id_=AREX_POLICY_OPERATION_URN;
-    action_=AREX_POLICY_OPERATION_ADMIN;
-  } else if(MatchXMLName(op,"ChangeActivityStatus")) {
-    id_=JOB_POLICY_OPERATION_URN;
-    action_=JOB_POLICY_OPERATION_MODIFY;
-  } else if(MatchXMLName(op,"MigrateActivity")) {
-    id_=JOB_POLICY_OPERATION_URN;
-    action_=JOB_POLICY_OPERATION_MODIFY;
-  } else if(MatchXMLName(op,"CacheCheck")) {
-    id_=AREX_POLICY_OPERATION_URN;
-    action_=AREX_POLICY_OPERATION_INFO;
-  } else if(MatchXMLName(op,"DelegateCredentialsInit")) {
-    id_=JOB_POLICY_OPERATION_URN;
-    action_=JOB_POLICY_OPERATION_CREATE;
-  } else if(MatchXMLName(op,"UpdateCredentials")) {
-    id_=JOB_POLICY_OPERATION_URN;
-    action_=JOB_POLICY_OPERATION_MODIFY;
+  if(MatchXMLNamespace(op,BES_FACTORY_NAMESPACE)) {
+    if(MatchXMLName(op,"CreateActivity")) {
+      id_=JOB_POLICY_OPERATION_URN;
+      action_=JOB_POLICY_OPERATION_CREATE;
+    } else if(MatchXMLName(op,"GetActivityStatuses")) {
+      id_=JOB_POLICY_OPERATION_URN;
+      action_=JOB_POLICY_OPERATION_READ;
+    } else if(MatchXMLName(op,"TerminateActivities")) {
+      id_=JOB_POLICY_OPERATION_URN;
+      action_=JOB_POLICY_OPERATION_MODIFY;
+    } else if(MatchXMLName(op,"GetActivityDocuments")) {
+      id_=JOB_POLICY_OPERATION_URN;
+      action_=JOB_POLICY_OPERATION_READ;
+    } else if(MatchXMLName(op,"GetFactoryAttributesDocument")) {
+      id_=AREX_POLICY_OPERATION_URN;
+      action_=AREX_POLICY_OPERATION_INFO;
+    }
+  } else if(MatchXMLNamespace(op,BES_MANAGEMENT_NAMESPACE)) {
+    if(MatchXMLName(op,"StopAcceptingNewActivities")) {
+      id_=AREX_POLICY_OPERATION_URN;
+      action_=AREX_POLICY_OPERATION_ADMIN;
+    } else if(MatchXMLName(op,"StartAcceptingNewActivities")) {
+      id_=AREX_POLICY_OPERATION_URN;
+      action_=AREX_POLICY_OPERATION_ADMIN;
+    }
+  } else if(MatchXMLNamespace(op,BES_ARC_NAMESPACE)) {
+    if(MatchXMLName(op,"ChangeActivityStatus")) {
+      id_=JOB_POLICY_OPERATION_URN;
+      action_=JOB_POLICY_OPERATION_MODIFY;
+    } else if(MatchXMLName(op,"MigrateActivity")) {
+      id_=JOB_POLICY_OPERATION_URN;
+      action_=JOB_POLICY_OPERATION_MODIFY;
+    } else if(MatchXMLName(op,"CacheCheck")) {
+      id_=AREX_POLICY_OPERATION_URN;
+      action_=AREX_POLICY_OPERATION_INFO;
+    }
+  } else if(MatchXMLNamespace(op,DELEG_ARC_NAMESPACE)) {
+    if(MatchXMLName(op,"DelegateCredentialsInit")) {
+      id_=JOB_POLICY_OPERATION_URN;
+      action_=JOB_POLICY_OPERATION_CREATE;
+    } else if(MatchXMLName(op,"UpdateCredentials")) {
+      id_=JOB_POLICY_OPERATION_URN;
+      action_=JOB_POLICY_OPERATION_MODIFY;
+    }
   } else if(MatchXMLNamespace(op,"http://docs.oasis-open.org/wsrf/rp-2")) {
     id_=AREX_POLICY_OPERATION_URN;
     action_=AREX_POLICY_OPERATION_INFO;
+  } else if(MatchXMLNamespace(op,ES_CREATE_NAMESPACE)) {
+    if(MatchXMLName(op,"CreateActivities")) {
+      id_=JOB_POLICY_OPERATION_URN;
+      action_=JOB_POLICY_OPERATION_CREATE;
+    }
+  } else if(MatchXMLNamespace(op,ES_DELEG_NAMESPACE)) {
+    if(MatchXMLName(op,"InitDelegation")) {
+      id_=JOB_POLICY_OPERATION_URN;
+      action_=JOB_POLICY_OPERATION_CREATE;
+    } else if(MatchXMLName(op,"PutDelegation")) {
+      id_=JOB_POLICY_OPERATION_URN;
+      action_=JOB_POLICY_OPERATION_MODIFY;
+    } else if(MatchXMLName(op,"GetDelegationInfo")) {
+      id_=JOB_POLICY_OPERATION_URN;
+      action_=JOB_POLICY_OPERATION_READ;
+    }
+  } else if(MatchXMLNamespace(op,ES_RINFO_NAMESPACE)) {
+    if(MatchXMLName(op,"GetResourceInfo")) {
+      id_=AREX_POLICY_OPERATION_URN;
+      action_=AREX_POLICY_OPERATION_INFO;
+    } else if(MatchXMLName(op,"QueryResourceInfo")) {
+      id_=AREX_POLICY_OPERATION_URN;
+      action_=AREX_POLICY_OPERATION_INFO;
+    }
+  } else if(MatchXMLNamespace(op,ES_MANAG_NAMESPACE)) {
+    if(MatchXMLName(op,"PauseActivity")) {
+      id_=JOB_POLICY_OPERATION_URN;
+      action_=JOB_POLICY_OPERATION_MODIFY;
+    } else if(MatchXMLName(op,"ResumeActivity")) {
+      id_=JOB_POLICY_OPERATION_URN;
+      action_=JOB_POLICY_OPERATION_MODIFY;
+    } else if(MatchXMLName(op,"ResumeActivity")) {
+      id_=JOB_POLICY_OPERATION_URN;
+      action_=JOB_POLICY_OPERATION_MODIFY;
+    } else if(MatchXMLName(op,"NotifyService")) {
+      id_=JOB_POLICY_OPERATION_URN;
+      action_=JOB_POLICY_OPERATION_MODIFY;
+    } else if(MatchXMLName(op,"CancelActivity")) {
+      id_=JOB_POLICY_OPERATION_URN;
+      action_=JOB_POLICY_OPERATION_MODIFY;
+    } else if(MatchXMLName(op,"WipeActivity")) {
+      id_=JOB_POLICY_OPERATION_URN;
+      action_=JOB_POLICY_OPERATION_MODIFY;
+    } else if(MatchXMLName(op,"RestartActivity")) {
+      id_=JOB_POLICY_OPERATION_URN;
+      action_=JOB_POLICY_OPERATION_MODIFY;
+    } else if(MatchXMLName(op,"GetActivityStatus")) {
+      id_=JOB_POLICY_OPERATION_URN;
+      action_=JOB_POLICY_OPERATION_READ;
+    } else if(MatchXMLName(op,"GetActivityInfo")) {
+      id_=JOB_POLICY_OPERATION_URN;
+      action_=JOB_POLICY_OPERATION_READ;
+    }
+  } else if(MatchXMLNamespace(op,ES_AINFO_NAMESPACE)) {
+    if(MatchXMLName(op,"ListActivities")) {
+      id_=JOB_POLICY_OPERATION_URN;
+      action_=JOB_POLICY_OPERATION_READ;
+    } else if(MatchXMLName(op,"GetActivityStatus")) {
+      id_=JOB_POLICY_OPERATION_URN;
+      action_=JOB_POLICY_OPERATION_READ;
+    } else if(MatchXMLName(op,"GetActivityInfo")) {
+      id_=JOB_POLICY_OPERATION_URN;
+      action_=JOB_POLICY_OPERATION_READ;
+    }
   }
 }
 
@@ -174,6 +270,33 @@ static Arc::XMLNode BESARCResponse(Arc::PayloadSOAP& res,const char* opname) {
   return response;
 }
 
+static Arc::XMLNode ESCreateResponse(Arc::PayloadSOAP& res,const char* opname) {
+  Arc::XMLNode response = res.NewChild(ES_CREATE_NPREFIX + ":" + opname + "Response");
+  return response;
+}
+
+/*
+static Arc::XMLNode ESDelegResponse(Arc::PayloadSOAP& res,const char* opname) {
+  Arc::XMLNode response = res.NewChild(ES_DELEG_NPREFIX + ":" + opname + "Response");
+  return response;
+}
+*/
+
+static Arc::XMLNode ESRInfoResponse(Arc::PayloadSOAP& res,const char* opname) {
+  Arc::XMLNode response = res.NewChild(ES_RINFO_NPREFIX + ":" + opname + "Response");
+  return response;
+}
+
+static Arc::XMLNode ESManagResponse(Arc::PayloadSOAP& res,const char* opname) {
+  Arc::XMLNode response = res.NewChild(ES_MANAG_NPREFIX + ":" + opname + "Response");
+  return response;
+}
+
+static Arc::XMLNode ESAInfoResponse(Arc::PayloadSOAP& res,const char* opname) {
+  Arc::XMLNode response = res.NewChild(ES_AINFO_NPREFIX + ":" + opname + "Response");
+  return response;
+}
+
 //static Arc::LogStream logcerr(std::cerr);
 
 static Arc::Plugin* get_service(Arc::PluginArgument* arg) {
@@ -357,6 +480,13 @@ static std::string GetPath(Arc::Message &inmsg,std::string &base) {
   return path;
 }
 
+#define SOAP_NOT_SUPPORTED { \
+  logger_.msg(Arc::ERROR, "SOAP operation is not supported: %s", op.Name()); \
+  delete outpayload; \
+  return make_soap_fault(outmsg); \
+}
+
+
 Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
   // Split request path into parts: service, job and file path. 
   // TODO: make it HTTP independent
@@ -394,8 +524,6 @@ Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
       logger_.msg(Arc::ERROR, "input is not SOAP");
       return make_soap_fault(outmsg);
     };
-    // Aplying known namespaces
-    inpayload->Namespaces(ns_);
     if(logger_.getThreshold() <= Arc::VERBOSE) {
         std::string str;
         inpayload->GetDoc(str, true);
@@ -447,49 +575,131 @@ Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
       Arc::PayloadSOAP& res = *outpayload;
       // Preparing known namespaces
       outpayload->Namespaces(ns_);
-      if(MatchXMLName(op,"CreateActivity")) {
-        CountedResourceLock cl_lock(beslimit_);
-        CreateActivity(*config,op,BESFactoryResponse(res,"CreateActivity"),clientid);
-      } else if(MatchXMLName(op,"GetActivityStatuses")) {
-        CountedResourceLock cl_lock(beslimit_);
-        GetActivityStatuses(*config,op,BESFactoryResponse(res,"GetActivityStatuses"));
-      } else if(MatchXMLName(op,"TerminateActivities")) {
-        CountedResourceLock cl_lock(beslimit_);
-        TerminateActivities(*config,op,BESFactoryResponse(res,"TerminateActivities"));
-      } else if(MatchXMLName(op,"GetActivityDocuments")) {
-        CountedResourceLock cl_lock(beslimit_);
-        GetActivityDocuments(*config,op,BESFactoryResponse(res,"GetActivityDocuments"));
-      } else if(MatchXMLName(op,"GetFactoryAttributesDocument")) {
-        CountedResourceLock cl_lock(beslimit_);
-        GetFactoryAttributesDocument(*config,op,BESFactoryResponse(res,"GetFactoryAttributesDocument"));
-      } else if(MatchXMLName(op,"StopAcceptingNewActivities")) {
-        CountedResourceLock cl_lock(beslimit_);
-        StopAcceptingNewActivities(*config,op,BESManagementResponse(res,"StopAcceptingNewActivities"));
-      } else if(MatchXMLName(op,"StartAcceptingNewActivities")) {
-        CountedResourceLock cl_lock(beslimit_);
-        StartAcceptingNewActivities(*config,op,BESManagementResponse(res,"StartAcceptingNewActivities"));
-      } else if(MatchXMLName(op,"ChangeActivityStatus")) {
-        CountedResourceLock cl_lock(beslimit_);
-        ChangeActivityStatus(*config,op,BESARCResponse(res,"ChangeActivityStatus"));
-      } else if(MatchXMLName(op,"MigrateActivity")) {
-        CountedResourceLock cl_lock(beslimit_);
-        MigrateActivity(*config,op,BESFactoryResponse(res,"MigrateActivity"),clientid);
-      } else if(MatchXMLName(op,"CacheCheck")) {
-        CacheCheck(*config,*inpayload,*outpayload);
-      } else if(MatchXMLName(op,"DelegateCredentialsInit")) {
-        CountedResourceLock cl_lock(beslimit_);
-        if(!delegations_.DelegateCredentialsInit(*inpayload,*outpayload,config->GridName())) {
-          delete outpayload;
-          return make_soap_fault(outmsg);
-        };
-      } else if(MatchXMLName(op,"UpdateCredentials")) {
+      if(MatchXMLNamespace(op,BES_FACTORY_NAMESPACE)) {
+        // Aplying known namespaces
+        inpayload->Namespaces(ns_);
+        if(MatchXMLName(op,"CreateActivity")) {
+          CountedResourceLock cl_lock(beslimit_);
+          CreateActivity(*config,op,BESFactoryResponse(res,"CreateActivity"),clientid);
+        } else if(MatchXMLName(op,"GetActivityStatuses")) {
+          CountedResourceLock cl_lock(beslimit_);
+          GetActivityStatuses(*config,op,BESFactoryResponse(res,"GetActivityStatuses"));
+        } else if(MatchXMLName(op,"TerminateActivities")) {
+          CountedResourceLock cl_lock(beslimit_);
+          TerminateActivities(*config,op,BESFactoryResponse(res,"TerminateActivities"));
+        } else if(MatchXMLName(op,"GetActivityDocuments")) {
+          CountedResourceLock cl_lock(beslimit_);
+          GetActivityDocuments(*config,op,BESFactoryResponse(res,"GetActivityDocuments"));
+        } else if(MatchXMLName(op,"GetFactoryAttributesDocument")) {
+          CountedResourceLock cl_lock(beslimit_);
+          GetFactoryAttributesDocument(*config,op,BESFactoryResponse(res,"GetFactoryAttributesDocument"));
+        } else {
+          SOAP_NOT_SUPPORTED;
+        }
+      } else if(MatchXMLNamespace(op,BES_MANAGEMENT_NAMESPACE)) {
+        // Aplying known namespaces
+        inpayload->Namespaces(ns_);
+        if(MatchXMLName(op,"StopAcceptingNewActivities")) {
+          CountedResourceLock cl_lock(beslimit_);
+          StopAcceptingNewActivities(*config,op,BESManagementResponse(res,"StopAcceptingNewActivities"));
+        } else if(MatchXMLName(op,"StartAcceptingNewActivities")) {
+          CountedResourceLock cl_lock(beslimit_);
+          StartAcceptingNewActivities(*config,op,BESManagementResponse(res,"StartAcceptingNewActivities"));
+        } else {
+          SOAP_NOT_SUPPORTED;
+        }
+      } else if(MatchXMLNamespace(op,ES_CREATE_NAMESPACE)) {
+        // Aplying known namespaces
+        inpayload->Namespaces(ns_);
+        if(MatchXMLName(op,"CreateActivities")) {
+          CountedResourceLock cl_lock(beslimit_);
+          ESCreateActivities(*config,op,ESCreateResponse(res,"CreateActivities"),clientid);
+        } else {
+          SOAP_NOT_SUPPORTED;
+        }
+      } else if(MatchXMLNamespace(op,ES_RINFO_NAMESPACE)) {
+        // Aplying known namespaces
+        inpayload->Namespaces(ns_);
+        if(MatchXMLName(op,"GetResourceInfo")) {
+          CountedResourceLock cl_lock(infolimit_);
+          ESGetResourceInfo(*config,op,ESRInfoResponse(res,"GetResourceInfo"));
+        } else if(MatchXMLName(op,"QueryResourceInfo")) {
+          CountedResourceLock cl_lock(infolimit_);
+          ESQueryResourceInfo(*config,op,ESRInfoResponse(res,"QueryResourceInfo"));
+        } else {
+          SOAP_NOT_SUPPORTED;
+        }
+      } else if(MatchXMLNamespace(op,ES_MANAG_NAMESPACE)) {
+        // Aplying known namespaces
+        inpayload->Namespaces(ns_);
+        if(MatchXMLName(op,"PauseActivity")) {
+          CountedResourceLock cl_lock(beslimit_);
+          ESPauseActivity(*config,op,ESManagResponse(res,"PauseActivity"));
+        } else if(MatchXMLName(op,"ResumeActivity")) {
+          CountedResourceLock cl_lock(beslimit_);
+          ESResumeActivity(*config,op,ESManagResponse(res,"ResumeActivity"));
+        } else if(MatchXMLName(op,"NotifyService")) {
+          CountedResourceLock cl_lock(beslimit_);
+          ESNotifyService(*config,op,ESManagResponse(res,"NotifyService"));
+        } else if(MatchXMLName(op,"CancelActivity")) {
+          CountedResourceLock cl_lock(beslimit_);
+          ESCancelActivity(*config,op,ESManagResponse(res,"CancelActivity"));
+        } else if(MatchXMLName(op,"WipeActivity")) {
+          CountedResourceLock cl_lock(beslimit_);
+          ESWipeActivity(*config,op,ESManagResponse(res,"WipeActivity"));
+        } else if(MatchXMLName(op,"RestartActivity")) {
+          CountedResourceLock cl_lock(beslimit_);
+          ESRestartActivity(*config,op,ESManagResponse(res,"RestartActivity"));
+        } else if(MatchXMLName(op,"GetActivityStatus")) {
+          CountedResourceLock cl_lock(beslimit_);
+          ESGetActivityStatus(*config,op,ESManagResponse(res,"GetActivityStatus"));
+        } else if(MatchXMLName(op,"GetActivityInfo")) {
+          CountedResourceLock cl_lock(beslimit_);
+          ESGetActivityInfo(*config,op,ESManagResponse(res,"GetActivityInfo"));
+        } else {
+          SOAP_NOT_SUPPORTED;
+        }
+      } else if(MatchXMLNamespace(op,ES_AINFO_NAMESPACE)) {
+        // Aplying known namespaces
+        inpayload->Namespaces(ns_);
+        if(MatchXMLName(op,"ListActivities")) {
+          CountedResourceLock cl_lock(beslimit_);
+          ESListActivities(*config,op,ESAInfoResponse(res,"ListActivities"));
+        } else if(MatchXMLName(op,"GetActivityStatus")) {
+          CountedResourceLock cl_lock(beslimit_);
+          ESGetActivityStatus(*config,op,ESAInfoResponse(res,"GetActivityStatus"));
+        } else if(MatchXMLName(op,"GetActivityInfo")) {
+          CountedResourceLock cl_lock(beslimit_);
+          ESGetActivityInfo(*config,op,ESAInfoResponse(res,"GetActivityInfo"));
+        } else {
+          SOAP_NOT_SUPPORTED;
+        }
+      } else if(MatchXMLNamespace(op,BES_ARC_NAMESPACE)) {
+        // Aplying known namespaces
+        inpayload->Namespaces(ns_);
+        if(MatchXMLName(op,"ChangeActivityStatus")) {
+          CountedResourceLock cl_lock(beslimit_);
+          ChangeActivityStatus(*config,op,BESARCResponse(res,"ChangeActivityStatus"));
+        } else if(MatchXMLName(op,"MigrateActivity")) {
+          CountedResourceLock cl_lock(beslimit_);
+          MigrateActivity(*config,op,BESFactoryResponse(res,"MigrateActivity"),clientid);
+        } else if(MatchXMLName(op,"CacheCheck")) {
+          CacheCheck(*config,*inpayload,*outpayload);
+        } else {
+          SOAP_NOT_SUPPORTED;
+        }
+      } else if(delegations_.MatchNamespace(*inpayload)) {
+        // Aplying known namespaces
+        inpayload->Namespaces(ns_);
         CountedResourceLock cl_lock(beslimit_);
         std::string credentials;
-        if(!delegations_.UpdateCredentials(credentials,*inpayload,*outpayload,config->GridName())) {
+        if(!delegations_.Process(credentials,*inpayload,*outpayload,config->GridName())) {
           delete outpayload;
           return make_soap_fault(outmsg);
         };
-        UpdateCredentials(*config,op,outpayload->Child(),credentials);
+        if(!credentials.empty()) {
+          UpdateCredentials(*config,op,outpayload->Child(),credentials);
+        };
       } else if(MatchXMLNamespace(op,"http://docs.oasis-open.org/wsrf/rp-2")) {
         CountedResourceLock cl_lock(infolimit_);
         /*
@@ -527,9 +737,7 @@ Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
         };
         return Arc::MCC_Status(Arc::STATUS_OK);
       } else {
-        logger_.msg(Arc::ERROR, "SOAP operation is not supported: %s", op.Name());
-        delete outpayload;
-        return make_soap_fault(outmsg);
+        SOAP_NOT_SUPPORTED;
       };
       if(logger_.getThreshold() <= Arc::VERBOSE) {
         std::string str;
@@ -606,20 +814,27 @@ ARexService::ARexService(Arc::Config *cfg):RegisteredService(cfg),
               infodoc_(true),
               inforeg_(*cfg,this),
               gmconfig_temporary_(false),
+              infoprovider_wakeup_period_(0),
+              all_jobs_count_(0),
               job_log_(NULL),
               jobs_cfg_(NULL),
               gm_env_(NULL),
               gm_(NULL),
-              infoprovider_wakeup_period_(0),
-              all_jobs_count_(0),
               valid_(false) {
   // logger_.addDestination(logcerr);
   // Define supported namespaces
   ns_[BES_ARC_NPREFIX]=BES_ARC_NAMESPACE;
-  ns_[BES_GLUE_NPREFIX]=BES_GLUE_NAMESPACE;
+  ns_[BES_GLUE2_NPREFIX]=BES_GLUE2_NAMESPACE;
+  ns_[BES_GLUE2PRE_NPREFIX]=BES_GLUE2PRE_NAMESPACE;
   ns_[BES_FACTORY_NPREFIX]=BES_FACTORY_NAMESPACE;
   ns_[BES_MANAGEMENT_NPREFIX]=BES_MANAGEMENT_NAMESPACE;
-  ns_["deleg"]="http://www.nordugrid.org/schemas/delegation";
+  ns_[DELEG_ARC_NPREFIX]=DELEG_ARC_NAMESPACE;
+  ns_[ES_TYPES_NPREFIX]=ES_TYPES_NAMESPACE;
+  ns_[ES_CREATE_NPREFIX]=ES_CREATE_NAMESPACE;
+  ns_[ES_DELEG_NPREFIX]=ES_DELEG_NAMESPACE;
+  ns_[ES_RINFO_NPREFIX]=ES_RINFO_NAMESPACE;
+  ns_[ES_MANAG_NPREFIX]=ES_MANAG_NAMESPACE;
+  ns_[ES_AINFO_NPREFIX]=ES_AINFO_NAMESPACE;
   ns_["wsa"]="http://www.w3.org/2005/08/addressing";
   ns_["jsdl"]="http://schemas.ggf.org/jsdl/2005/11/jsdl";
   ns_["wsrf-bf"]="http://docs.oasis-open.org/wsrf/bf-2";
@@ -732,7 +947,11 @@ ARexService::ARexService(Arc::Config *cfg):RegisteredService(cfg),
   lrms_name_ = (std::string)((*cfg)["LRMSName"]);
   // Must be URI. URL may be too restrictive, but is safe.
   if(!Arc::URL(lrms_name_)) {
-    logger_.msg(Arc::ERROR, "Provided LRMSName is not a valid URL: %s",lrms_name_);
+    if (!lrms_name_.empty()) {
+      logger_.msg(Arc::ERROR, "Provided LRMSName is not a valid URL: %s",lrms_name_);
+    } else {
+      logger_.msg(Arc::WARNING, "No LRMSName is provided. This is needed if you wish to support the BES interface to A-REX.");
+    };
     // Filling something to make it follow BES specs
     lrms_name_ = "uri:undefined";
   };
@@ -787,7 +1006,7 @@ ARexService::~ARexService(void) {
 } // namespace ARex
 
 Arc::PluginDescriptor PLUGINS_TABLE_NAME[] = {
-    { "a-rex", "HED:SERVICE", 0, &ARex::get_service },
-    { NULL, NULL, 0, NULL }
+    { "a-rex", "HED:SERVICE", NULL, 0, &ARex::get_service },
+    { NULL, NULL, NULL, 0, NULL }
 };
 
