@@ -52,6 +52,7 @@ class OptimizedInformationContainer: public Arc::InformationContainer {
   void Assign(const std::string& xml);
 };
 
+#define AREXOP(NAME) Arc::MCC_Status NAME(ARexGMConfig& config,Arc::XMLNode in,Arc::XMLNode out)
 class ARexService: public Arc::RegisteredService {
  protected:
   Arc::ThreadRegistry thread_count_;
@@ -83,24 +84,48 @@ class ARexService: public Arc::RegisteredService {
   GridManager* gm_;
   bool valid_;
   ARexConfigContext* get_configuration(Arc::Message& inmsg);
+
+  // A-REX operations
   Arc::MCC_Status CreateActivity(ARexGMConfig& config,Arc::XMLNode in,Arc::XMLNode out,const std::string& clientid);
-  Arc::MCC_Status GetActivityStatuses(ARexGMConfig& config,Arc::XMLNode in,Arc::XMLNode out);
-  Arc::MCC_Status TerminateActivities(ARexGMConfig& config,Arc::XMLNode in,Arc::XMLNode out);
-  Arc::MCC_Status GetActivityDocuments(ARexGMConfig& config,Arc::XMLNode in,Arc::XMLNode out);
-  Arc::MCC_Status GetFactoryAttributesDocument(ARexGMConfig& config,Arc::XMLNode in,Arc::XMLNode out);
-  Arc::MCC_Status StopAcceptingNewActivities(ARexGMConfig& config,Arc::XMLNode in,Arc::XMLNode out);
-  Arc::MCC_Status StartAcceptingNewActivities(ARexGMConfig& config,Arc::XMLNode in,Arc::XMLNode out);
-  Arc::MCC_Status ChangeActivityStatus(ARexGMConfig& config,Arc::XMLNode in,Arc::XMLNode out);
+  AREXOP(GetActivityStatuses);
+  AREXOP(TerminateActivities);
+  AREXOP(GetActivityDocuments);
+  AREXOP(GetFactoryAttributesDocument);
+
+  AREXOP(StopAcceptingNewActivities);
+  AREXOP(StartAcceptingNewActivities);
+
+  AREXOP(ChangeActivityStatus);
   Arc::MCC_Status MigrateActivity(ARexGMConfig& config,Arc::XMLNode in,Arc::XMLNode out,const std::string& clientid);
-  Arc::MCC_Status CacheCheck(ARexGMConfig& config,Arc::XMLNode in,Arc::XMLNode out);
+  AREXOP(CacheCheck);
   Arc::MCC_Status UpdateCredentials(ARexGMConfig& config,Arc::XMLNode in,Arc::XMLNode out,const std::string& credentials);
+
+  // EMI ES operations
+  Arc::MCC_Status ESCreateActivities(ARexGMConfig& config,Arc::XMLNode in,Arc::XMLNode out,const std::string& clientid);
+  AREXOP(ESGetResourceInfo);
+  AREXOP(ESQueryResourceInfo);
+  AREXOP(ESPauseActivity);
+  AREXOP(ESResumeActivity);
+  AREXOP(ESNotifyService);
+  AREXOP(ESCancelActivity);
+  AREXOP(ESWipeActivity);
+  AREXOP(ESRestartActivity);
+  AREXOP(ESListActivities);
+  AREXOP(ESGetActivityStatus);
+  AREXOP(ESGetActivityInfo);
+
+  // Convenience methods
   Arc::MCC_Status make_empty_response(Arc::Message& outmsg);
   Arc::MCC_Status make_fault(Arc::Message& outmsg);
   Arc::MCC_Status make_http_fault(Arc::Message& outmsg,int code,const char* resp);
   Arc::MCC_Status make_soap_fault(Arc::Message& outmsg);
-  Arc::MCC_Status Get(Arc::Message& inmsg,Arc::Message& outmsg,ARexGMConfig& config,const std::string& id,const std::string& subpath);
-  Arc::MCC_Status Head(Arc::Message& inmsg,Arc::Message& outmsg,ARexGMConfig& config,const std::string& id,const std::string& subpath);
-  Arc::MCC_Status Put(Arc::Message& inmsg,Arc::Message& outmsg,ARexGMConfig& config,const std::string& id,const std::string& subpath);
+
+  // HTTP operations
+  Arc::MCC_Status Get(Arc::Message& inmsg,Arc::Message& outmsg,ARexGMConfig& config,std::string id,std::string subpath);
+  Arc::MCC_Status Head(Arc::Message& inmsg,Arc::Message& outmsg,ARexGMConfig& config,std::string id,std::string subpath);
+  Arc::MCC_Status Put(Arc::Message& inmsg,Arc::Message& outmsg,ARexGMConfig& config,std::string id,std::string subpath);
+
+  // A-REX faults
   void GenericFault(Arc::SOAPFault& fault);
   void NotAuthorizedFault(Arc::XMLNode fault);
   void NotAuthorizedFault(Arc::SOAPFault& fault);
@@ -116,6 +141,29 @@ class ARexService: public Arc::RegisteredService {
   void UnknownActivityIdentifierFault(Arc::SOAPFault& fault,const std::string& message);
   void InvalidRequestMessageFault(Arc::XMLNode fault,const std::string& element,const std::string& message);
   void InvalidRequestMessageFault(Arc::SOAPFault& fault,const std::string& element,const std::string& message);
+
+  // EMI ES faults
+  void ESInternalBaseFault(Arc::XMLNode fault,const std::string& message,const std::string& desc = "");
+  void ESInternalBaseFault(Arc::SOAPFault& fault,const std::string& message,const std::string& desc = "");
+  void ESVectorLimitExceededFault(Arc::XMLNode fault,unsigned long limit,const std::string& message = "",const std::string& desc = "");
+  void ESVectorLimitExceededFault(Arc::SOAPFault& fault,unsigned long limit,const std::string& message = "",const std::string& desc = "");
+  void ESAccessControlFault(Arc::XMLNode fault,const std::string& message = "",const std::string& desc = "");
+  void ESAccessControlFault(Arc::SOAPFault& fault,const std::string& message = "",const std::string& desc = "");
+  void ESUnsupportedCapabilityFault(Arc::XMLNode fault,const std::string& message = "",const std::string& desc = "");
+  void ESUnsupportedCapabilityFault(Arc::SOAPFault& fault,const std::string& message = "",const std::string& desc = "");
+  void ESInvalidActivityDescriptionSemanticFault(Arc::XMLNode fault,const std::string& message = "",const std::string& desc = "");
+  void ESInvalidActivityDescriptionSemanticFault(Arc::SOAPFault& fault,const std::string& message = "",const std::string& desc = "");
+  void ESInvalidActivityDescriptionFault(Arc::XMLNode fault,const std::string& message = "",const std::string& desc = "");
+  void ESInvalidActivityDescriptionFault(Arc::SOAPFault& fault,const std::string& message = "",const std::string& desc = "");
+  void ESInternalResourceInfoFault(Arc::XMLNode fault,const std::string& message = "",const std::string& desc = "");
+  void ESInternalResourceInfoFault(Arc::SOAPFault& fault,const std::string& message = "",const std::string& desc = "");
+  void ESInvalidActivityIDFault(Arc::XMLNode fault,const std::string& message = "",const std::string& desc = "");
+  void ESInvalidActivityIDFault(Arc::SOAPFault& fault,const std::string& message = "",const std::string& desc = "");
+  void ESUnknownActivityIDFault(Arc::XMLNode fault,const std::string& message = "",const std::string& desc = "");
+  void ESUnknownActivityIDFault(Arc::SOAPFault& fault,const std::string& message = "",const std::string& desc = "");
+  void ESInvalidActivityStateFault(Arc::XMLNode fault,const std::string& message = "",const std::string& desc = "");
+  void ESInvalidActivityStateFault(Arc::SOAPFault& fault,const std::string& message = "",const std::string& desc = "");
+
  public:
   ARexService(Arc::Config *cfg);
   virtual ~ARexService(void);
