@@ -76,7 +76,7 @@ static Arc::Logger& logger = Arc::Logger::getRootLogger();
 
 static job_state_t job_state_read_file(const std::string &fname,bool &pending);
 static bool job_state_write_file(const std::string &fname,job_state_t state,bool pending = false);
-static bool job_Xput_read_file(std::list<FileData> &files);
+// static bool job_Xput_read_file(std::list<FileData> &files);
 static bool job_strings_write_file(const std::string &fname,std::list<std::string> &str);
 
 
@@ -310,8 +310,10 @@ static int job_dir_remove_callback(void* arg) {
 bool job_diagnostics_mark_put(const JobDescription &desc,const JobUser &user) {
   std::string fname = desc.SessionDir() + sfx_diag;
   if(user.StrictSession()) {
-    JobUser tmp_user(user.Env(),user.get_uid()==0?desc.get_uid():user.get_uid());
-    return (RunFunction::run(tmp_user,"job_diagnostics_mark_put",&job_mark_put_callback,&fname,10) == 0);
+    uid_t uid = user.get_uid()==0?desc.get_uid():user.get_uid();
+    uid_t gid = user.get_uid()==0?desc.get_gid():user.get_gid();
+    JobUser tmp_user(user.Env(),uid,gid);
+    return (RunFunction::run(tmp_user,"job_diagnostics_mark_put",&job_mark_put_callback,&fname,-1) == 0);
   };
   return job_mark_put(fname) & fix_file_owner(fname,desc,user) & fix_file_permissions(fname);
 }
@@ -319,8 +321,10 @@ bool job_diagnostics_mark_put(const JobDescription &desc,const JobUser &user) {
 bool job_session_create(const JobDescription &desc,const JobUser &user) {
   std::string dname = desc.SessionDir();
   if(user.StrictSession()) {
-    JobUser tmp_user(user.Env(),user.get_uid()==0?desc.get_uid():user.get_uid());
-    return (RunFunction::run(tmp_user,"job_session_create",&job_dir_create_callback,&dname,10) == 0);
+    uid_t uid = user.get_uid()==0?desc.get_uid():user.get_uid();
+    uid_t gid = user.get_uid()==0?desc.get_gid():user.get_gid();
+    JobUser tmp_user(user.Env(),uid,gid);
+    return (RunFunction::run(tmp_user,"job_session_create",&job_dir_create_callback,&dname,-1) == 0);
   };
   return job_dir_create(dname) & fix_file_owner(dname,desc,user) & fix_file_permissions(dname,true);
 }
@@ -335,7 +339,7 @@ bool job_controldiag_mark_put(const JobDescription &desc,const JobUser &user,cha
   if(h == -1) return false;
   int r;
   int t = 10;
-  JobUser tmp_user(user.Env(),(uid_t)0);
+  JobUser tmp_user(user.Env(),(uid_t)0,(gid_t)0);
   r=RunRedirected::run(tmp_user,"job_controldiag_mark_put",-1,h,-1,(char**)args,t);
   close(h);
   if(r != 0) return false;
@@ -345,8 +349,10 @@ bool job_controldiag_mark_put(const JobDescription &desc,const JobUser &user,cha
 bool job_lrmsoutput_mark_put(const JobDescription &desc,const JobUser &user) {
   std::string fname = desc.SessionDir() + sfx_lrmsoutput;
   if(user.StrictSession()) {
-    JobUser tmp_user(user.Env(),user.get_uid()==0?desc.get_uid():user.get_uid());
-    return (RunFunction::run(tmp_user,"job_lrmsoutput_mark_put",&job_mark_put_callback,&fname,10) == 0);
+    uid_t uid = user.get_uid()==0?desc.get_uid():user.get_uid();
+    uid_t gid = user.get_uid()==0?desc.get_gid():user.get_gid();
+    JobUser tmp_user(user.Env(),uid,gid);
+    return (RunFunction::run(tmp_user,"job_lrmsoutput_mark_put",&job_mark_put_callback,&fname,-1) == 0);
   };
   return job_mark_put(fname) & fix_file_owner(fname,desc,user) & fix_file_permissions(fname);
 }
@@ -354,9 +360,11 @@ bool job_lrmsoutput_mark_put(const JobDescription &desc,const JobUser &user) {
 bool job_diagnostics_mark_add(const JobDescription &desc,const JobUser &user,const std::string &content) {
   std::string fname = desc.SessionDir() + sfx_diag;
   if(user.StrictSession()) {
-    JobUser tmp_user(user.Env(),user.get_uid()==0?desc.get_uid():user.get_uid());
+    uid_t uid = user.get_uid()==0?desc.get_uid():user.get_uid();
+    uid_t gid = user.get_uid()==0?desc.get_gid():user.get_gid();
+    JobUser tmp_user(user.Env(),uid,gid);
     job_mark_add_t arg; arg.fname=&fname; arg.content=&content;
-    return (RunFunction::run(tmp_user,"job_diagnostics_mark_add",&job_mark_add_callback,&arg,10) == 0);
+    return (RunFunction::run(tmp_user,"job_diagnostics_mark_add",&job_mark_add_callback,&arg,-1) == 0);
   };
   return job_mark_add_s(fname,content) & fix_file_owner(fname,desc,user) & fix_file_permissions(fname);
 }
@@ -366,8 +374,10 @@ bool job_diagnostics_mark_remove(const JobDescription &desc,const JobUser &user)
   bool res1 = job_mark_remove(fname);
   fname = desc.SessionDir() + sfx_diag;
   if(user.StrictSession()) {
-    JobUser tmp_user(user.Env(),user.get_uid()==0?desc.get_uid():user.get_uid());
-    return (res1 | (RunFunction::run(tmp_user,"job_diagnostics_mark_remove",&job_mark_remove_callback,&fname,10) == 0));
+    uid_t uid = user.get_uid()==0?desc.get_uid():user.get_uid();
+    uid_t gid = user.get_uid()==0?desc.get_gid():user.get_gid();
+    JobUser tmp_user(user.Env(),uid,gid);
+    return (res1 | (RunFunction::run(tmp_user,"job_diagnostics_mark_remove",&job_mark_remove_callback,&fname,-1) == 0));
   };
   return (res1 | job_mark_remove(fname));
 }
@@ -375,8 +385,10 @@ bool job_diagnostics_mark_remove(const JobDescription &desc,const JobUser &user)
 bool job_lrmsoutput_mark_remove(const JobDescription &desc,const JobUser &user) {
   std::string fname = desc.SessionDir() + sfx_lrmsoutput;
   if(user.StrictSession()) {
-    JobUser tmp_user(user.Env(),user.get_uid()==0?desc.get_uid():user.get_uid());
-    return (RunFunction::run(tmp_user,"job_lrmsoutpur_mark_remove",&job_mark_remove_callback,&fname,10) == 0);
+    uid_t uid = user.get_uid()==0?desc.get_uid():user.get_uid();
+    uid_t gid = user.get_uid()==0?desc.get_gid():user.get_gid();
+    JobUser tmp_user(user.Env(),uid,gid);
+    return (RunFunction::run(tmp_user,"job_lrmsoutpur_mark_remove",&job_mark_remove_callback,&fname,-1) == 0);
   };
   return job_mark_remove(fname);
 }
@@ -411,9 +423,11 @@ bool job_diagnostics_mark_move(const JobDescription &desc,const JobUser &user) {
   fix_file_permissions(fname2,desc,user);
   std::string fname1 = user.SessionRoot(desc.get_id()) + "/" + desc.get_id() + sfx_diag;
   if(user.StrictSession()) {
-    JobUser tmp_user(user.Env(),user.get_uid()==0?desc.get_uid():user.get_uid());
+    uid_t uid = user.get_uid()==0?desc.get_uid():user.get_uid();
+    uid_t gid = user.get_uid()==0?desc.get_gid():user.get_gid();
+    JobUser tmp_user(user.Env(),uid,gid);
     job_file_read_t arg; arg.h=h2; arg.fname=&fname1;
-    RunFunction::run(tmp_user,"job_diagnostics_mark_move",&job_file_read_callback,&arg,10);
+    RunFunction::run(tmp_user,"job_diagnostics_mark_move",&job_file_read_callback,&arg,-1);
     close(h2);
     return true;
   };
@@ -1160,16 +1174,18 @@ bool job_Xput_write_file(const std::string &fname,std::list<FileData> &files) {
   return true;
 }
 
+/*
 static bool job_Xput_read_file(std::list<FileData> &files) {
   for(;!std::cin.eof();) {
     FileData fd; std::cin >> fd;
 //    if((fd.pfn.length() != 0) && (fd.pfn != "/")) {
-    if(fd.pfn.length() != 0) {  /* returns zero length only if empty std::string */
+    if(fd.pfn.length() != 0) { // returns zero length only if empty std::string
       files.push_back(fd);
     };
   };
   return true;
 }
+*/
 
 bool job_Xput_read_file(const std::string &fname,std::list<FileData> &files) {
   std::ifstream f(fname.c_str());
@@ -1177,7 +1193,7 @@ bool job_Xput_read_file(const std::string &fname,std::list<FileData> &files) {
   for(;!f.eof();) {
     FileData fd; f >> fd;
 //    if((fd.pfn.length() != 0) && (fd.pfn != "/")) {
-    if(fd.pfn.length() != 0) {  /* returns zero length only if empty std::string */
+    if(fd.pfn.length() != 0) { // returns zero length only if empty std::string
       files.push_back(fd);
     };
   };
@@ -1233,9 +1249,11 @@ bool job_clean_deleted(const JobDescription &desc,const JobUser &user,std::list<
   std::list<FileData> flist;
   std::string dname = user.SessionRoot(id)+"/"+id;
   if(user.StrictSession()) {
-    JobUser tmp_user(user.Env(),user.get_uid()==0?desc.get_uid():user.get_uid());
+    uid_t uid = user.get_uid()==0?desc.get_uid():user.get_uid();
+    uid_t gid = user.get_uid()==0?desc.get_gid():user.get_gid();
+    JobUser tmp_user(user.Env(),uid,gid);
     job_dir_remove_t arg; arg.dname=&dname; arg.flist=&flist;
-    return (RunFunction::run(tmp_user,"job_clean_deleted",&job_dir_remove_callback,&arg,10) == 0);
+    return (RunFunction::run(tmp_user,"job_clean_deleted",&job_dir_remove_callback,&arg,-1) == 0);
   } else {
     delete_all_files(dname,flist,true);
     remove(dname.c_str());
