@@ -2,17 +2,13 @@
 #include <fcntl.h>
 
 #include <arc/FileUtils.h>
+#include <arc/data/FileCache.h>
 
 #include <arc/data-staging/Scheduler.h>
 
-#include "../files/info_files.h"
 #include "../files/delete.h"
 #include "../conf/conf_map.h"
 
-// need to import FileCache after Scheduler since define of round()
-// conflicts with math.h. TODO: fix this!
-#include <arc/data/FileCache.h>
-
 #include "dtr_generator.h"
 
 Arc::Logger DTRInfo::logger(Arc::Logger::getRootLogger(), "DTRInfo");
@@ -29,6 +25,10 @@ void DTRInfo::receiveDTR(DataStaging::DTR& dtr) {
 
 Arc::Logger DTRGenerator::logger(Arc::Logger::getRootLogger(), "Generator");
 
+bool compare_job_description(JobDescription first, JobDescription second) {
+  return first.get_local()->priority > second.get_local()->priority;
+}
+
 void DTRGenerator::main_thread(void* arg) {
   ((DTRGenerator*)arg)->thread();
 }
@@ -61,9 +61,7 @@ void DTRGenerator::thread() {
       processReceivedDTR(*it_dtrs);
       event_lock.lock();
       // delete DTR Logger and LogDestinations
-      const std::list<Arc::LogDestination*> log_dests = it_dtrs->get_logger()->getDestinations();
-      for (std::list<Arc::LogDestination*>::const_iterator i = log_dests.begin(); i != log_dests.end(); ++i)
-        delete *i;
+      it_dtrs->get_logger()->deleteDestinations();
       delete it_dtrs->get_logger();
 
       it_dtrs = dtrs_received.erase(it_dtrs);
@@ -71,18 +69,26 @@ void DTRGenerator::thread() {
 
     // finally new jobs
     std::list<JobDescription>::iterator it_jobs = jobs_received.begin();
-    while (it_jobs != jobs_received.end()) {
+    // it can happen that the list grows faster than the jobs are processed
+    // so here we only process for a small time to avoid blocking other
+    // jobs finishing
+    Arc::Time limit(Arc::Time() + Arc::Period(30));
+
+    // sort the list by job priority
+    jobs_received.sort(compare_job_description);
+    while (it_jobs != jobs_received.end() && Arc::Time() < limit) {
       event_lock.unlock();
       processReceivedJob(*it_jobs);
       event_lock.lock();
       it_jobs = jobs_received.erase(it_jobs);
     }
     event_lock.unlock();
-    usleep(50000);
+    Glib::usleep(50000);
   }
   // stop scheduler - cancels all DTRs and waits for them to complete
   scheduler.stop();
   run_condition.signal();
+  logger.msg(Arc::INFO, "Exiting Generator thread");
 }
 
 DTRGenerator::DTRGenerator(const JobUsers& users,
@@ -98,7 +104,11 @@ DTRGenerator::DTRGenerator(const JobUsers& users,
   for (JobUsers::const_iterator i = users.begin(); i != users.end(); ++i) {
     jobusers[i->get_uid()] = &(*i);
   }
-//  DataStaging::DTR::registerCallback(&DTRGenerator::receiveDTR);
+  // set the DTR dump file to the first control dir registered
+  if (!jobusers.empty())
+    scheduler.SetDumpLocation(std::string(jobusers.begin()->second->ControlDir()+"/dtrstate.log"));
+
+  //  DataStaging::DTR::registerCallback(&DTRGenerator::receiveDTR);
 
   JobsListConfig& jcfg = users.Env().jobs_cfg();
 
@@ -110,12 +120,9 @@ DTRGenerator::DTRGenerator(const JobUsers& users,
   int max_processing_emergency;
   int max_downloads;
   jcfg.GetMaxJobsLoad(max_processing,max_processing_emergency,max_downloads);
-  if(max_processing <= 0) max_processing = 1;
-  if(max_processing_emergency <= 0) max_processing_emergency = 0;
-  if(max_downloads <= 0) max_downloads = 1;
-  max_processing *= max_downloads;
-  max_processing_emergency *= max_downloads;
-  scheduler.SetSlots(max_processing*2,max_processing*2,max_processing,max_processing_emergency);
+  if (max_processing > 0 && max_downloads > 0) max_processing *= max_downloads;
+  if (max_processing_emergency > 0 && max_downloads > 0) max_processing_emergency *= max_downloads;
+  scheduler.SetSlots(max_processing,max_processing,max_processing,max_processing_emergency);
 
   // transfer shares
   DataStaging::TransferShares shares;
@@ -141,6 +148,9 @@ DTRGenerator::DTRGenerator(const JobUsers& users,
   // preferred pattern
   scheduler.SetPreferredPattern(jcfg.GetPreferredPattern());
 
+  // delivery services
+  scheduler.SetDeliveryServices(jcfg.GetDeliveryServices());
+
   scheduler.start();
 
   Arc::CreateThreadFunction(&main_thread, this);
@@ -149,11 +159,10 @@ DTRGenerator::DTRGenerator(const JobUsers& users,
 
 DTRGenerator::~DTRGenerator() {
   if (generator_state != DataStaging::RUNNING)
-    return; // false;
+    return;
   generator_state = DataStaging::TO_STOP;
   run_condition.wait();
   generator_state = DataStaging::STOPPED;
-  return; // true;
 }
 
 void DTRGenerator::receiveDTR(DataStaging::DTR& dtr) {
@@ -173,10 +182,11 @@ void DTRGenerator::receiveDTR(DataStaging::DTR& dtr) {
 void DTRGenerator::receiveJob(const JobDescription& job) {
 
   if (generator_state != DataStaging::RUNNING) {
-    logger.msg(Arc::ERROR, "DTRGenerator is not running!");
-    return;
+    logger.msg(Arc::WARNING, "DTRGenerator is not running!");
   }
 
+  // Add to jobs list even if Generator is stopped, so that A-REX doesn't
+  // think that staging has finished.
   event_lock.lock();
   jobs_received.push_back(job);
   event_lock.unlock();
@@ -185,9 +195,9 @@ void DTRGenerator::receiveJob(const JobDescription& job) {
 void DTRGenerator::cancelJob(const JobDescription& job) {
 
   if (generator_state != DataStaging::RUNNING) {
-    logger.msg(Arc::ERROR, "DTRGenerator is not running!");
-    return;
+    logger.msg(Arc::WARNING, "DTRGenerator is not running!");
   }
+
   event_lock.lock();
   jobs_cancelled.push_back(job.get_id());
   event_lock.unlock();
@@ -271,13 +281,13 @@ bool DTRGenerator::processReceivedDTR(DataStaging::DTR& dtr) {
     logger.msg(Arc::ERROR, "%s: DTR %s to copy file %s failed",
                jobid, dtr.get_id(), dtr.get_source()->str());
 
+    lock.lock();
     if (!dtr.get_source()->Local() && finished_jobs.find(jobid) == finished_jobs.end()) { // download
       // cancel other DTRs and erase from our list unless error was already reported
       logger.msg(Arc::INFO, "%s: Cancelling other DTRs", jobid);
       scheduler.cancelDTRs(jobid);
     }
     // add error to finished jobs
-    lock.lock();
     finished_jobs[jobid] += std::string("Failed in data staging: " + dtr.get_error_status().GetDesc() + '\n');
     lock.unlock();
   }
@@ -440,7 +450,6 @@ bool DTRGenerator::processReceivedDTR(DataStaging::DTR& dtr) {
   }
   // add to finished jobs (without overwriting existing error)
   finished_jobs[jobid] += "";
-  lock.unlock();
 
   // log summary to DTR log
   const std::list<Arc::LogDestination*> log_dests = Arc::Logger::getRootLogger().getDestinations();
@@ -454,6 +463,7 @@ bool DTRGenerator::processReceivedDTR(DataStaging::DTR& dtr) {
     dtr.get_logger()->msg(Arc::INFO, "Some %s failed",
                           dtr.get_source()->Local() ? "uploads":"downloads");
 
+  lock.unlock();
   Arc::Logger::getRootLogger().addDestinations(log_dests);
 
   logger.msg(Arc::INFO, "%s: Data staging finished", jobid);
@@ -481,6 +491,7 @@ bool DTRGenerator::processReceivedJob(const JobDescription& job) {
       lock.lock();
       finished_jobs[jobid] = std::string("Internal configuration error in data staging");
       lock.unlock();
+      if (kicker_func) (*kicker_func)(kicker_arg);
       return false;
     }
   }
@@ -504,6 +515,7 @@ bool DTRGenerator::processReceivedJob(const JobDescription& job) {
     lock.lock();
     finished_jobs[jobid] = std::string("Failed to read list of output files");
     lock.unlock();
+    if (kicker_func) (*kicker_func)(kicker_arg);
     return false;
   }
 
@@ -513,6 +525,7 @@ bool DTRGenerator::processReceivedJob(const JobDescription& job) {
       lock.lock();
       finished_jobs[jobid] = std::string("Failed to read list of input files");
       lock.unlock();
+      if (kicker_func) (*kicker_func)(kicker_arg);
       return false;
     }
     // check for duplicates (see bug 1285)
@@ -523,6 +536,7 @@ bool DTRGenerator::processReceivedJob(const JobDescription& job) {
           lock.lock();
           finished_jobs[jobid] = std::string("Duplicate file in list of input files: " + i->pfn);
           lock.unlock();
+          if (kicker_func) (*kicker_func)(kicker_arg);
           return false;
         }
       }
@@ -547,6 +561,7 @@ bool DTRGenerator::processReceivedJob(const JobDescription& job) {
       lock.lock();
       finished_jobs[jobid] = std::string("Failed to clean up session dir before downloading inputs");
       lock.unlock();
+      if (kicker_func) (*kicker_func)(kicker_arg);
       return false;
     }
   }
@@ -563,6 +578,7 @@ bool DTRGenerator::processReceivedJob(const JobDescription& job) {
           lock.lock();
           finished_jobs[jobid] = std::string("Error reading user generated output file list");
           lock.unlock();
+          if (kicker_func) (*kicker_func)(kicker_arg);
           return false;
         }
         it->pfn.erase(1, 1);
@@ -579,6 +595,7 @@ bool DTRGenerator::processReceivedJob(const JobDescription& job) {
             lock.lock();
             finished_jobs[jobid] = std::string("Two identical output destinations: " + it->lfn);
             lock.unlock();
+            if (kicker_func) (*kicker_func)(kicker_arg);
             return false;
           }
           Arc::URL u_it(it->lfn);
@@ -590,6 +607,7 @@ bool DTRGenerator::processReceivedJob(const JobDescription& job) {
               lock.lock();
               finished_jobs[jobid] = std::string("Cannot upload two different files to same LFN: " + it->lfn);
               lock.unlock();
+              if (kicker_func) (*kicker_func)(kicker_arg);
               return false;
             }
             replication = true;
@@ -607,6 +625,7 @@ bool DTRGenerator::processReceivedJob(const JobDescription& job) {
       lock.lock();
       finished_jobs[jobid] = std::string("Failed to clean up session dir before uploading outputs");
       lock.unlock();
+      if (kicker_func) (*kicker_func)(kicker_arg);
       return false;
     }
   }
@@ -616,6 +635,7 @@ bool DTRGenerator::processReceivedJob(const JobDescription& job) {
     lock.lock();
     finished_jobs[jobid] = std::string("Logic error: DTR Generator received job in a bad state");
     lock.unlock();
+    if (kicker_func) (*kicker_func)(kicker_arg);
     return false;
   }
   Arc::initializeCredentialsType cred_type(Arc::initializeCredentialsType::SkipCredentials);
@@ -628,6 +648,12 @@ bool DTRGenerator::processReceivedJob(const JobDescription& job) {
   // create job.id.errors file with correct permissions to add to Logger
   job_errors_mark_put(job, *jobuser);
 
+  if (files.empty()) {
+    // nothing to do so wake up GM thread and return
+    if (kicker_func) (*kicker_func)(kicker_arg);
+    return true;
+  }
+
   for (std::list<FileData>::iterator i = files.begin(); i != files.end(); ++i) {
     if (i->lfn.find(":") == std::string::npos)
       continue; // user down/uploadable file
@@ -659,6 +685,9 @@ bool DTRGenerator::processReceivedJob(const JobDescription& job) {
     // set priority as given in job description
     if (job.get_local())
       dtr.set_priority(job.get_local()->priority);
+    // use A-REX host certificate for remote delivery services
+    dtr.host_cert_for_remote_delivery(true);
+
     DataStaging::CacheParameters cache_parameters;
     cache_parameters.cache_dirs = jobuser->CacheParams().getCacheDirs();
     cache_parameters.remote_cache_dirs = jobuser->CacheParams().getRemoteCacheDirs();
@@ -676,6 +705,19 @@ bool DTRGenerator::processReceivedJob(const JobDescription& job) {
     Arc::Logger::getRootLogger().removeDestinations();
     dtr.push(DataStaging::SCHEDULER);
     Arc::Logger::getRootLogger().addDestinations(log_dests);
+
+    // update .local with transfer share
+    JobLocalDescription *job_desc = new JobLocalDescription;
+    if (!job_local_read_file(jobid, *jobuser, *job_desc)) {
+      logger.msg(Arc::ERROR, "%s: Failed reading local information", jobid);
+      delete job_desc;
+      continue;
+    }
+    job_desc->transfershare = dtr.get_transfer_share();
+    if (!job_local_write_file(job, *jobuser, *job_desc)) {
+      logger.msg(Arc::ERROR, "%s: Failed writing local information", jobid);
+    }
+    delete job_desc;
   }
   return true;
 }
@@ -688,7 +730,6 @@ bool DTRGenerator::processCancelledJob(const std::string& jobid) {
   return true;
 }
 
-/* taken straight from old downloader - could be re-written better */
 int DTRGenerator::checkUploadedFiles(JobDescription& job) {
 
   JobId jobid(job.get_id());
@@ -715,161 +756,160 @@ int DTRGenerator::checkUploadedFiles(JobDescription& job) {
     return 1;
   }
   int res = 0;
-  // read input_status files
-  std::list<std::string> uploaded_files;
-  std::list<std::string>* uploaded_files_ = NULL;
-  if (job_input_status_read_file(jobid, *jobuser, uploaded_files))
-    uploaded_files_ = &uploaded_files;
+
   // loop through each file and check
   for (FileData::iterator i = input_files.begin(); i != input_files.end();) {
-    if (i->lfn.find(":") == std::string::npos) { // not remote file
-      logger.msg(Arc::VERBOSE, "%s: Check user uploadable file: %s", jobid, i->pfn);
-      std::string error;
-      int err = user_file_exists(*i, session_dir, uploaded_files_, &error);
-      if (err == 0) { /* file is uploaded */
-        logger.msg(Arc::VERBOSE, "%s: User has uploaded file %s", jobid, i->pfn);
-        // remove from input list
-        i = input_files.erase(i);
-        input_files_.clear();
-        for (FileData::iterator it = input_files.begin(); it != input_files.end(); ++it)
-          input_files_.push_back(*it);
-        if (!job_input_write_file(job, *jobuser, input_files_)) {
-          logger.msg(Arc::WARNING, "%s: Failed writing changed input file.", jobid);
-        }
-      }
-      else if (err == 1) { /* critical failure */
-        logger.msg(Arc::ERROR, "%s: Critical error for uploadable file %s", jobid, i->pfn);
-        job.AddFailure("User file: "+i->pfn+" - "+error);
-        res = 1;
-        goto exit;
-      }
-      else {
-        res = 2;
-        ++i;
+    // all remote files should have been downloaded by this point
+    if (i->lfn.find(":") != std::string::npos) {
+      ++i;
+      continue;
+    }
+    logger.msg(Arc::VERBOSE, "%s: Checking user uploadable file: %s", jobid, i->pfn);
+    std::string error;
+    int err = user_file_exists(*i, session_dir, error);
+
+    if (err == 0) { // file is uploaded
+      logger.msg(Arc::VERBOSE, "%s: User has uploaded file %s", jobid, i->pfn);
+      // remove from input list
+      i = input_files.erase(i);
+      input_files_.clear();
+      for (FileData::iterator it = input_files.begin(); it != input_files.end(); ++it)
+        input_files_.push_back(*it);
+      if (!job_input_write_file(job, *jobuser, input_files_)) {
+        logger.msg(Arc::WARNING, "%s: Failed writing changed input file.", jobid);
       }
     }
-    else {
+    else if (err == 1) { // critical failure
+      logger.msg(Arc::ERROR, "%s: Critical error for uploadable file %s", jobid, i->pfn);
+      job.AddFailure("User file: "+i->pfn+" - "+error);
+      res = 1;
+      break;
+    }
+    else { // still waiting
+      res = 2;
       ++i;
     }
   }
   // check for timeout
   if (res == 2 && ((time(NULL) - job.GetStartTime()) > 600)) { // hard-coded timeout
     for (FileData::iterator i = input_files.begin(); i != input_files.end(); ++i) {
-      if (i->lfn.find(":") == std::string::npos) { /* is it lfn ? */
+      if (i->lfn.find(":") == std::string::npos) {
         job.AddFailure("User file: "+i->pfn+" - Timeout waiting");
-      };
-    };
+      }
+    }
     logger.msg(Arc::ERROR, "%s: Uploadable files timed out", jobid);
     res = 1;
-  };
-exit:
+  }
   // clean unfinished files here
-  input_files_.clear();
-  for (FileData::iterator i = input_files.begin(); i != input_files.end(); ++i)
-    input_files_.push_back(*i);
-  delete_all_files(session_dir, input_files_, false, true, false);
+  delete_all_files(session_dir, input_files, false, true, false);
   return res;
 }
 
-/*
-   Check for existence of user uploadable file
-   returns 0 if file exists
-           1 - it is not proper file or other error
-           2 - not here yet
-   TODO: Copied straight from old downloader - should be cleaned up
-*/
-int DTRGenerator::user_file_exists(FileData &dt,const std::string& session_dir,std::list<std::string>* have_files,std::string* error) {
+int DTRGenerator::user_file_exists(FileData &dt,
+                                   const std::string& session_dir,
+                                   std::string& error) {
   struct stat st;
-  const char *str = dt.lfn.c_str();
-  if(strcmp(str,"*.*") == 0) return 0; /* do not wait for this file */
-  std::string fname=session_dir + '/' + dt.pfn;
-  /* check if file does exist at all */
-  if(lstat(fname.c_str(),&st) != 0) return 2;
-  /* check for misconfiguration */
-  /* parse files information */
-  char *str_;
-  unsigned long long int fsize;
-  unsigned long long int fsum = (unsigned long long int)(-1);
+  std::string file_info(dt.lfn);
+  if (file_info == "*.*") return 0; // do not wait for this file
+
+  std::string fname = session_dir + '/' + dt.pfn;
+  // check if file exists at all
+  if (lstat(fname.c_str(), &st) != 0) return 2;
+
+  // if no size/checksum was supplied, return success
+  if (file_info.empty()) return 0;
+
+  if (S_ISDIR(st.st_mode)) {
+    error = "Expected file. Directory found.";
+    return 1;
+  }
+  if (!S_ISREG(st.st_mode)) {
+    error = "Expected ordinary file. Special object found.";
+    return 1;
+  }
+
+  long long int fsize;
+  long long int fsum;
   bool have_size = false;
   bool have_checksum = false;
-  errno = 0;
-  fsize = strtoull(str,&str_,10);
-  if((*str_) == '.') {
-    if(str_ != str) have_size=true;
-    str=str_+1;
-    fsum = strtoull(str,&str_,10);
-    if((*str_) != 0) {
-      logger.msg(Arc::ERROR, "Invalid checksum in %s for %s", dt.lfn, dt.pfn);
-      if(error) (*error)="Bad information about file: checksum can't be parsed.";
+
+  // parse format [size][.checksum]
+  if (file_info[0] == '.') { // checksum only
+    if (!Arc::stringto(file_info.substr(1), fsum)) {
+      logger.msg(Arc::ERROR, "Can't convert checksum %s to int for %s", file_info.substr(1), dt.pfn);
+      error = "Invalid checksum information";
       return 1;
-    };
-    have_checksum=true;
+    }
+    have_checksum = true;
+  } else if (file_info.find('.') == std::string::npos) { // size only
+    if (!Arc::stringto(file_info, fsize)) {
+      logger.msg(Arc::ERROR, "Can't convert filesize %s to int for %s", file_info, dt.pfn);
+      error = "Invalid file size information";
+      return 1;
+    }
+    have_size = true;
+  } else { // size and checksum
+    std::vector<std::string> file_attrs;
+    Arc::tokenize(dt.lfn, file_attrs, ".");
+    if (file_attrs.size() != 2) {
+      logger.msg(Arc::ERROR, "Invalid size/checksum information (%s) for %s", file_info, dt.pfn);
+      error = "Invalid size/checksum information";
+      return 1;
+    }
+    if (!Arc::stringto(file_attrs[0], fsize)) {
+      logger.msg(Arc::ERROR, "Can't convert filesize %s to int for %s", file_attrs[0], dt.pfn);
+      error = "Invalid file size information";
+      return 1;
+    }
+    if (!Arc::stringto(file_attrs[1], fsum)) {
+      logger.msg(Arc::ERROR, "Can't convert checksum %s to int for %s", file_attrs[1], dt.pfn);
+      error = "Invalid checksum information";
+      return 1;
+    }
+    have_size = true;
+    have_checksum = true;
   }
-  else {
-    if(str_ != str) have_size=true;
-    if((*str_) != 0) {
-      logger.msg(Arc::ERROR, "Invalid file size in %s for %s ", dt.lfn, dt.pfn);
-      if(error) (*error)="Bad information about file: size can't be parsed.";
-      return 1;
-    };
-  };
-  if(S_ISDIR(st.st_mode)) {
-    if(have_size || have_checksum) {
-      if(error) (*error)="Expected file. Directory found.";
-      return 1;
-    };
-    return 0;
-  };
-  if(!S_ISREG(st.st_mode)) {
-    if(error) (*error)="Expected ordinary file. Special object found.";
-    return 1;
-  };
-  /* now check if proper size */
-  if(have_size) {
-    if(st.st_size < fsize) return 2;
-    if(st.st_size > fsize) {
+
+  // now check if proper size
+  if (have_size) {
+    if (st.st_size < fsize) return 2;
+    if (st.st_size > fsize) {
       logger.msg(Arc::ERROR, "Invalid file: %s is too big.", dt.pfn);
-      if(error) (*error)="Delivered file is bigger than specified.";
-      return 1; /* too big file */
-    };
-  };
-  if(have_files) {
-    std::list<std::string>::iterator f = have_files->begin();
-    for(;f!=have_files->end();++f) {
-      if(dt.pfn == *f) break;
-    };
-    if(f == have_files->end()) return 2;
-  } else if(have_checksum) {
-    int h=::open(fname.c_str(),O_RDONLY);
-    if(h==-1) { /* if we can't read that file job won't too */
-      logger.msg(Arc::ERROR, "Error accessing file %s", dt.pfn);
-      if(error) (*error)="Delivered file is unreadable.";
+      error = "Delivered file is bigger than specified.";
       return 1;
-    };
+    }
+  }
+
+  if (have_checksum) { // calculate checksum
+    int h = ::open(fname.c_str(), O_RDONLY);
+    if (h == -1) { // if we can't read that file job won't too
+      logger.msg(Arc::ERROR, "Error accessing file %s", dt.pfn);
+      error = "Delivered file is unreadable.";
+      return 1;
+    }
     Arc::CRC32Sum crc;
     char buffer[1024];
     ssize_t l;
-    size_t ll = 0;
     for(;;) {
       if((l=read(h,buffer,1024)) == -1) {
         logger.msg(Arc::ERROR, "Error reading file %s", dt.pfn);
-        if(error) (*error)="Could not read file to compute checksum.";
+        error = "Could not read file to compute checksum.";
         return 1;
-      };
-      if(l==0) break; ll+=l;
+      }
+      if(l==0) break;
       crc.add(buffer,l);
-    };
+    }
     close(h);
     crc.end();
-    if(fsum != crc.crc()) {
-      if(have_size) { /* size was checked - it is an error to have wrong crc */
+    if (fsum != crc.crc()) {
+      if (have_size) { // size was checked and is ok
         logger.msg(Arc::ERROR, "File %s has wrong CRC.", dt.pfn);
-        if(error) (*error)="Delivered file has wrong checksum.";
+        error = "Delivered file has wrong checksum.";
         return 1;
-      };
-      return 2; /* just not uploaded yet */
-    };
-  };
-  return 0; /* all checks passed - file is ok */
+      }
+      return 2; // not uploaded yet
+    }
+  }
+  return 0; // all checks passed - file is ok
 }
 
