@@ -48,7 +48,7 @@ static Arc::Logger& logger = Arc::Logger::getRootLogger();
 JobsListConfig::JobsListConfig(void) {
   for(int n = 0;n<JOB_STATE_NUM;n++) jobs_num[n]=0;
   jobs_pending = 0;
-  max_jobs_processing=DEFAULT_MAX_JOBS;
+  max_jobs_processing=DEFAULT_MAX_LOAD;
   max_jobs_processing_emergency=1;
   max_jobs_running=-1;
   max_jobs_total=-1;
@@ -185,7 +185,7 @@ void JobsList::CalculateShares(){
     }
     else if (i->job_state == JOB_STATE_INLRMS) {
       // is job ready to move to finishing?
-      if (job_lrms_mark_check(i->job_id,*user) && i->next_retry <= time(NULL)) {
+      if ((job_lrms_mark_check(i->job_id,*user) || i->job_pending) && i->next_retry <= time(NULL)) {
         pre_finishing_job_share[i->transfer_share]++;
       }
     }
@@ -200,7 +200,8 @@ void JobsList::CalculateShares(){
   int privileged_jobs_processing = 0;
   int privileged_preparing_job_share = 0;
   int privileged_finishing_job_share = 0;
-  for (std::map<std::string, int>::iterator i = jcfg.limited_share.begin(); i != jcfg.limited_share.end(); i++) {
+  std::map<std::string, int> limited_shares = jcfg.limited_share;
+  for (std::map<std::string, int>::iterator i = limited_shares.begin(); i != limited_shares.end(); i++) {
     if (pre_preparing_job_share.find(i->first) != pre_preparing_job_share.end()) {
       privileged_preparing_job_share++;
       privileged_jobs_processing += i->second;
@@ -257,26 +258,34 @@ void JobsList::CalculateShares(){
     unprivileged_finishing_limit = unprivileged_finishing_limit < 2 ? 1 : unprivileged_finishing_limit/2;
   }
 
-  if (jcfg.max_jobs_processing > 0 && privileged_total_pre_preparing > privileged_jobs_processing/2 && privileged_total_pre_finishing > privileged_jobs_processing/2)
-  for (std::map<std::string, int>::iterator i = jcfg.limited_share.begin(); i != jcfg.limited_share.end(); i++)
-    i->second = i->second < 2 ? 1 : i->second/2; 
+  if (jcfg.max_jobs_processing > 0 && privileged_total_pre_preparing > privileged_jobs_processing/2 && privileged_total_pre_finishing > privileged_jobs_processing/2) {
+    for (std::map<std::string, int>::iterator i = limited_shares.begin(); i != limited_shares.end(); i++) {
+      i->second = i->second < 2 ? 1 : i->second/2;
+    }
+  }
       
   preparing_max_share = pre_preparing_job_share;
   finishing_max_share = pre_finishing_job_share;
   for (std::map<std::string, int>::iterator i = preparing_max_share.begin(); i != preparing_max_share.end(); i++){
-    if (jcfg.limited_share.find(i->first) != jcfg.limited_share.end())
-      i->second = jcfg.limited_share[i->first];
+    if (limited_shares.find(i->first) != limited_shares.end())
+      i->second = limited_shares[i->first];
     else
       i->second = unprivileged_preparing_limit;
   }
   for (std::map<std::string, int>::iterator i = finishing_max_share.begin(); i != finishing_max_share.end(); i++){
-    if (jcfg.limited_share.find(i->first) != jcfg.limited_share.end())
-      i->second = jcfg.limited_share[i->first];
+    if (limited_shares.find(i->first) != limited_shares.end())
+      i->second = limited_shares[i->first];
     else
       i->second = unprivileged_finishing_limit;
   }
 }
 
+void JobsList::PrepareToDestroy(void) {
+  for(iterator i=jobs.begin();i!=jobs.end();++i) {
+    i->PrepareToDestroy();
+  }
+}
+
 bool JobsList::ActJobs(void) {
   JobsListConfig& jcfg = user->Env().jobs_cfg();
 /*
@@ -567,7 +576,7 @@ bool JobsList::state_loading(const JobsList::iterator &i,bool &state_changed,boo
     // Job is in data staging - check progress
     if (dtr_generator->queryJobFinished(*i)) {
       // Finished - check for failure
-      if (!i->GetFailure().empty()) {
+      if (!i->GetFailure(*user).empty()) {
         JobFailStateRemember(i, (up ? JOB_STATE_FINISHING : JOB_STATE_PREPARING));
         return false;
       }
@@ -960,11 +969,14 @@ void JobsList::ActJobUndefined(JobsList::iterator &i,
               std::string user_proxy_file = job_proxy_filename(i->get_id(), *user).c_str();
               std::string cert_dir = "/etc/grid-security/certificates";
               std::string v = user->Env().cert_dir_loc();
-              if(! v.empty()) cert_dir = v;
-                Arc::Credential u(user_proxy_file,"",cert_dir,"");
-                const std::string share = get_property(u,jcfg.share_type);
-                i->set_share(share);
-                logger.msg(Arc::INFO, "%s: adding to transfer share %s",i->get_id(),i->transfer_share);
+              if(!v.empty()) cert_dir = v;
+              Arc::Credential u(user_proxy_file,"",cert_dir,"");
+              std::string voms_trust_chains = Arc::GetEnv("VOMS_TRUST_CHAINS");
+              std::vector<std::string> vomstrustlist;
+              Arc::tokenize(voms_trust_chains, vomstrustlist, "\n");
+              const std::string share = getCredentialProperty(u,jcfg.share_type,cert_dir,"",vomstrustlist);
+              i->set_share(share);
+              logger.msg(Arc::INFO, "%s: adding to transfer share %s",i->get_id(),i->transfer_share);
             }
             job_desc->transfershare = i->transfer_share;
             job_local_write_file(*i,*user,*job_desc);
@@ -992,10 +1004,13 @@ void JobsList::ActJobUndefined(JobsList::iterator &i,
               std::string cert_dir = "/etc/grid-security/certificates";
               std::string v = user->Env().cert_dir_loc();
               if(! v.empty()) cert_dir = v;
-                Arc::Credential u(user_proxy_file,"",cert_dir,"");
-                const std::string share = get_property(u,jcfg.share_type);
-                i->set_share(share);
-                logger.msg(Arc::INFO, "%s: adding to transfer share %s",i->get_id(),i->transfer_share);
+              Arc::Credential u(user_proxy_file,"",cert_dir,"");
+              std::string voms_trust_chains = Arc::GetEnv("VOMS_TRUST_CHAINS");
+              std::vector<std::string> vomstrustlist;
+              Arc::tokenize(voms_trust_chains, vomstrustlist, "\n");
+              const std::string share = getCredentialProperty(u,jcfg.share_type,cert_dir,"",vomstrustlist);
+              i->set_share(share);
+              logger.msg(Arc::INFO, "%s: adding to transfer share %s",i->get_id(),i->transfer_share);
             }
             job_local_read_file(i->job_id, *user, *job_desc);
             job_desc->transfershare = i->transfer_share;
@@ -1041,13 +1056,14 @@ void JobsList::ActJobAccepted(JobsList::iterator &i,
           job_error=true; 
           return; /* go to next job */
         };
-        if((jcfg.max_jobs_processing == -1) ||
-           (jcfg.use_local_transfer) ||
-           ((i->local->downloads == 0) && (i->local->rtes == 0)) ||
-           (jcfg.max_jobs_per_dn < 0 || jcfg.jobs_dn[i->local->DN] < jcfg.max_jobs_per_dn))
+        // check per-DN limit on processing jobs
+        if(jcfg.max_jobs_per_dn < 0 || jcfg.jobs_dn[i->local->DN] < jcfg.max_jobs_per_dn)
         {
           // apply limits for old data staging
           if (jcfg.use_new_data_staging ||
+              ((jcfg.max_jobs_processing == -1) && (i->next_retry <= time(NULL))) ||
+              (jcfg.use_local_transfer) ||
+              ((i->local->downloads == 0) && (i->local->rtes == 0)) ||
               (((JOB_NUM_PROCESSING < jcfg.max_jobs_processing) ||
                ((JOB_NUM_FINISHING >= jcfg.max_jobs_processing) &&
                 (JOB_NUM_PREPARING < jcfg.max_jobs_processing_emergency))) &&
@@ -1106,17 +1122,6 @@ void JobsList::ActJobPreparing(JobsList::iterator &i,
               return;
             };
             if(i->local->arguments.size()) {
-              // with new data staging, when a job fails or is cancelled we wait until all
-              // DTRs complete before continuing, so here we check for failure
-              if(jcfg.use_new_data_staging && dtr_generator) {
-                if(job_failed_mark_check(i->job_id, *user)) {
-                  state_changed=true; once_more=true;
-                  i->job_state = JOB_STATE_FINISHING;
-                  dtr_generator->receiveJob(*i);
-                  finishing_job_share[i->transfer_share]++;
-                  return;
-                };
-              };
               if((JOB_NUM_RUNNING<jcfg.max_jobs_running) || (jcfg.max_jobs_running==-1)) {
                 i->job_state = JOB_STATE_SUBMITTING;
                 state_changed=true; once_more=true;
@@ -1130,7 +1135,7 @@ void JobsList::ActJobPreparing(JobsList::iterator &i,
               i->job_state = JOB_STATE_FINISHING;
               dtr_generator->receiveJob(*i);
               finishing_job_share[i->transfer_share]++;
-            } else if((jcfg.max_jobs_processing == -1) ||
+            } else if(((jcfg.max_jobs_processing == -1) && (i->next_retry <= time(NULL))) ||
                       (jcfg.use_local_transfer) ||
                       (i->local->uploads == 0) ||
                       (((JOB_NUM_PROCESSING < jcfg.max_jobs_processing) ||
@@ -1176,7 +1181,7 @@ void JobsList::ActJobPreparing(JobsList::iterator &i,
           }; 
         } 
         else {
-          if(i->GetFailure().length() == 0)
+          if(i->GetFailure(*user).length() == 0)
             i->AddFailure("Data staging failed (pre-processing)");
           job_error=true;
           preparing_job_share[i->transfer_share]--;
@@ -1278,7 +1283,7 @@ void JobsList::ActJobInlrms(JobsList::iterator &i,
               dtr_generator->receiveJob(*i);
               finishing_job_share[i->transfer_share]++;
             }
-            else if ((jcfg.max_jobs_processing == -1) ||
+            else if (((jcfg.max_jobs_processing == -1) && (i->next_retry <= time(NULL))) ||
               (jcfg.use_local_transfer) ||
               (i->local->uploads == 0) ||
               (((JOB_NUM_PROCESSING < jcfg.max_jobs_processing) ||
@@ -1293,7 +1298,7 @@ void JobsList::ActJobInlrms(JobsList::iterator &i,
                  finishing_job_share[i->transfer_share]++;
             } else JobPending(i);
           }
-        } else if((jcfg.max_jobs_processing == -1) ||
+        } else if(((jcfg.max_jobs_processing == -1) && (i->next_retry <= time(NULL))) ||
             (jcfg.use_local_transfer) ||
             (i->local->uploads == 0) ||
             (((JOB_NUM_PROCESSING < jcfg.max_jobs_processing) ||
@@ -1354,7 +1359,7 @@ void JobsList::ActJobFinishing(JobsList::iterator &i,
           // i->job_state = JOB_STATE_FINISHED;
           state_changed=true; /* to send mail */
           once_more=true;
-          if(i->GetFailure().length() == 0)
+          if(i->GetFailure(*user).length() == 0)
             i->AddFailure("uploader failed (post-processing)");
           job_error=true;
           finishing_job_share[i->transfer_share]--;
@@ -1370,7 +1375,7 @@ void JobsList::ActJobFinishing(JobsList::iterator &i,
                                  i->job_id, i->job_uid, i->job_gid);
             cache.Release();
           }
-          catch (CacheConfigException e) {
+          catch (CacheConfigException& e) {
             logger.msg(Arc::WARNING, "Error with cache configuration: %s. Cannot clean up files for job %s", e.what(), i->job_id);
           }
         }
@@ -1454,7 +1459,7 @@ void JobsList::ActJobFinished(JobsList::iterator &i,
                 try {
                   cache_config = CacheConfig(user->Env());
                 }
-                catch (CacheConfigException e) {
+                catch (CacheConfigException& e) {
                   logger.msg(Arc::ERROR, "Error with cache configuration: %s", e.what());
                   job_clean_deleted(*i,*user);
                   i->job_state = JOB_STATE_DELETED;
@@ -1561,15 +1566,17 @@ bool JobsList::ActJob(JobsList::iterator &i) {
         if(i->job_state == JOB_STATE_INLRMS) {
           i->job_state = JOB_STATE_CANCELING;
         }
+        // if new data staging we wait to get back all DTRs
         else if(i->job_state == JOB_STATE_FINISHING) {
-          i->job_state = JOB_STATE_FINISHED;
-          if(GetLocalDescription(i)) {
-            if (--(jcfg.jobs_dn[i->local->DN]) <= 0)
-              jcfg.jobs_dn.erase(i->local->DN);
+          if (!jcfg.use_new_data_staging) {
+            i->job_state = JOB_STATE_FINISHED;
+            if(GetLocalDescription(i)) {
+              if (--(jcfg.jobs_dn[i->local->DN]) <= 0)
+                jcfg.jobs_dn.erase(i->local->DN);
+            }
           }
         }
         else if (!jcfg.use_new_data_staging || i->job_state != JOB_STATE_PREPARING) {
-          // if preparing with new data staging we wait to get back all DTRs
           i->job_state = JOB_STATE_FINISHING;
           finishing_job_share[i->transfer_share]++;
         };
@@ -1635,9 +1642,7 @@ bool JobsList::ActJob(JobsList::iterator &i) {
             }
             state_changed=true;
             once_more=true;
-          } else if (!jcfg.use_new_data_staging || i->job_state != JOB_STATE_PREPARING){
-            // If job fails in preparing with new data staging, wait for DTRs to finish
-            // Any other failure should cause transfer to go to FINISHING
+          } else {
             i->job_state = JOB_STATE_FINISHING;
             if (jcfg.use_new_data_staging && dtr_generator) {
               dtr_generator->receiveJob(*i);
@@ -1866,7 +1871,7 @@ bool JobsList::ScanJobs(const std::string& cdir,std::list<JobFDesc>& ids) {
       };
     };
   } catch(Glib::FileError& e) {
-    logger.msg(Arc::ERROR,"Failed reading control directory: %s",user->ControlDir());
+    logger.msg(Arc::ERROR,"Failed reading control directory: %s: %s",user->ControlDir(), e.what());
     return false;
   };
   return true;
