@@ -5,6 +5,7 @@ package ARC1ClusterInfo;
 
 use Storable;
 use FileHandle;
+use File::Temp;
 use POSIX qw(ceil);
 
 use strict;
@@ -85,6 +86,89 @@ sub glueState {
     return $status;
 }
 
+sub getGMStatus {
+    my ($controldir, $ID) = @_;
+    foreach my $gmjob_status ("$controldir/accepting/job.$ID.status", "$controldir/processing/job.$ID.status", "$controldir/finished/job.$ID.status") {
+        unless (open (GMJOB_STATUS, "<$gmjob_status")) {
+            next;
+        } else {
+            chomp (my ($first_line) = <GMJOB_STATUS>);
+            close GMJOB_STATUS;
+            unless ($first_line) {
+                next;
+            }
+            return $first_line;
+        }
+    }
+    return undef;
+}
+
+# Helper function that assists the GLUE2 XML renderer handle the 'splitjobs' option
+#   $config       - the config hash
+#   $jobid        - job id from GM
+#   $gmjob        - a job hash ref as returned by GMJobsInfo
+#   $xmlGenerator - a function ref that returns a string (the job's GLUE2 XML description)
+# Returns undef on error, 0 if the XML file was already up to date, 1 if it was written
+sub jobXmlFileWriter {
+    my ($config, $jobid, $gmjob, $xmlGenerator) = @_;
+    # If this is defined, then it's a job managed by local A-REX.
+    my $gmuser = $gmjob->{gmuser};
+    # Skip for now jobs managed by remote A-REX.
+    # These are still published in ldap-infosys. As long as
+    # distributing jobs to remote grid-managers is only
+    # implemented by gridftd, remote jobs are not of interest
+    # for the WS interface.
+    return 0 unless defined $gmuser;
+    my $controldir = $config->{control}{$gmuser}{controldir};
+    my $xml_file = $controldir . "/job." . $jobid . ".xml";
+
+    # Here goes simple optimisation - do not write new
+    # XML if status has not changed while in "slow" states
+    my $xml_time = (stat($xml_file))[9];
+    my $status_time = $gmjob->{statusmodified};
+    return 0 if defined $xml_time
+                and defined $status_time
+                and $status_time < $xml_time
+                and $gmjob->{status} =~ /ACCEPTED|FINISHED|FAILED|KILLED|DELETED/;
+
+    my $xmlstring = &$xmlGenerator();
+    return undef unless defined $xmlstring;
+
+    # tempfile croaks on error
+    my ($fh, $tmpnam) = File::Temp::tempfile("job.$jobid.xml.XXXXXXX", DIR => $controldir);
+    binmode $fh, ':encoding(utf8)';
+    print $fh $xmlstring and close $fh
+        or $log->warning("Error writing to temporary file $tmpnam: $!")
+        and close $fh
+        and unlink $tmpnam
+        and return undef;
+    rename $tmpnam, $xml_file
+        or $log->warning("Error moving $tmpnam to $xml_file: $!")
+        and unlink $tmpnam
+        and return undef;
+
+
+    # Avoid .xml files created after job is deleted
+    # Check if status file exists
+    if(not defined getGMStatus($controldir,$jobid)) {
+      unlink $xml_file;
+      return undef;
+    }
+
+    # Set timestamp to the time when the status file was read in.
+    # This is because the status file might have been updated by the time the
+    # XML file gets written. This step ensures that the XML will be updated on
+    # the next run of the infoprovider.
+    my $status_read = $gmjob->{statusread};
+    return undef unless defined $status_read;
+    utime(time(), $status_read, $xml_file)
+        or $log->warning("Couldn't touch $xml_file: $!")
+        and return undef;
+
+    # *.xml file was updated
+    return 1;
+};
+
 # Intersection of two arrays that completes in linear time.  The input arrays
 # are the keys of the two hashes passed as reference.  The intersection array
 # consists of the keys of the returned hash reference.
@@ -628,8 +712,9 @@ sub collect($) {
             $cep->{Technology} = 'webservice';
             $cep->{InterfaceName} = 'ogf.bes';
             $cep->{InterfaceVersion} = [ '1.0' ];
-            # InterfaceExtension should return the same as BESExtension attribute of BES-Factory 
-            #$cep->{InterfaceExtension} = [ 'http://www.nordugrid.org/schemas/a-rex' ];
+            # InterfaceExtension should return the same as BESExtension attribute of BES-Factory.
+            # value is taken from services/a-rex/get_factory_attributes_document.cpp, line 56.
+            $cep->{InterfaceExtension} = [ 'http://www.nordugrid.org/schemas/a-rex' ];
             $cep->{WSDL} = [ $config->{endpoint}."/?wsdl" ];
             # Wrong type, should be URI
             $cep->{SupportedProfile} = [ "http://www.ws-i.org/Profiles/BasicProfile-1.0.html",  # WS-I 1.0
@@ -637,7 +722,7 @@ sub collect($) {
                                        ];
             $cep->{Semantics} = [ "http://www.nordugrid.org/documents/arex.pdf" ];
             $cep->{Implementor} = "NorduGrid";
-            $cep->{ImplementationName} = "ARC";
+            $cep->{ImplementationName} = "ARC CE";
             $cep->{ImplementationVersion} = $config->{arcversion};
 
             $cep->{QualityLevel} = "development";
@@ -723,6 +808,8 @@ sub collect($) {
                     return $apol;
                 };
             }
+	
+	    $cep->{OtherInfo} = $host_info->{EMIversion} if ($host_info->{EMIversion}); # array
 
 
             # Computing Activities
@@ -741,10 +828,6 @@ sub collect($) {
                 my $share = $gmjob->{share};
                 my $gridid = $config->{endpoint}."/$jobid";
 
-                # for use by XML printer
-                $cact->{gmid} = $jobid;
-                $cact->{statustime} = $gmjob->{statustime};
-
                 $cact->{Type} = 'single';
                 $cact->{ID} = $cactIDs{$share}{$jobid};
                 $cact->{IDFromEndpoint} = $gmjob->{globalid} if $gmjob->{globalid};
@@ -826,29 +909,7 @@ sub collect($) {
                     $cact->{State} = glueState($gmjob->{status});
                 }
 
-                # Give some help to the XML renderer to handle splitjobs
-                my $getXmlFileHandle = sub {
-                    # If this is defined, then it's a job managed by local A-REX.
-                    my $gmuser = $gmjob->{gmuser};
-                    # Skip for now jobs managed by remote A-REX.
-                    # These are still published in ldap-infosys. As long as
-                    # distributing jobs to remote grid-managers is only
-                    # implemented by gridftd, remote jobs are not of interest
-                    # for the WS interface.
-                    return undef unless defined $gmuser;
-                    my $controldir = $config->{control}{$gmuser}{controldir};
-                    my $xml_file = $controldir . "/job." . $jobid . ".xml";
-                    # Here goes simple optimisation - do not write new
-                    # XML if status has not changed while in "slow" states
-                    my $xml_time = (stat($xml_file))[9];
-                    my $status_time = $gmjob->{statustime};
-                    return undef if defined $xml_time
-                                and defined $status_time
-                                and $status_time < $xml_time
-                                and $gmjob->{status} =~ /ACCEPTED|FINISHED|FAILED|KILLED|DELETED/;
-                    return FileHandle->new($xml_file, "w");
-                };
-                $cact->{xmlFileHandle} = $getXmlFileHandle;
+                $cact->{jobXmlFileWriter} = sub { jobXmlFileWriter($config, $jobid, $gmjob, @_) };
 
                 return $cact;
             };
@@ -1125,7 +1186,6 @@ sub collect($) {
             push @{$csha->{ExecutionEnvironmentID}}, $xenvIDs{$_} for @$xenvs;
 
             $csha->{ComputingEndpointID} = $cepID;
-            $csha->{ComputingActivityID} = [ values %{$cactIDs{$share}} ] unless $nojobs;
             $csha->{ComputingServiceID} = $csvID;
 
             return $csha;
