@@ -77,6 +77,7 @@ my $share_options = {
     MaxSlotsPerJob => '*',
     SchedulingPolicy => '*',
     Preemption => '*',
+    totalcpus => '*',
 };
 my $gmuser_options = {
     controldir => '',
@@ -686,6 +687,7 @@ sub build_config_from_inifile {
     if (%$cluster) {
         # Ignored: cluster_location, lrmsconfig
         rename_keys $cluster, $config, {arex_mount_point => 'endpoint'};
+        rename_keys $cluster, $config->{location}, { cluster_location => 'PostCode' };
         rename_keys $cluster, $config->{service}, {
                                  interactive_contactstring => 'InteractiveContactstring',
                                  cluster_owner => 'ClusterOwner', localse => 'LocalSE',
@@ -726,7 +728,6 @@ sub build_config_from_inifile {
         $log->error("NodeSelection option only allowed under ExecutionEnvironment section") if $queue->{NodeSelection};
         delete $queue->{NodeSelection};
 
-        # Ignored: totalcpus
         rename_keys $queue, $sconf, {scheduling_policy => 'SchedulingPolicy',
                                      nodememory => 'MaxVirtualMemory', comment => 'Description'};
         move_keys $queue, $sconf, [keys %$share_options, keys %$lrms_share_options];
