@@ -307,7 +307,7 @@ my $cache_free_min = $no_cache ? 0 : $cache_stats{freemin};
 my $cache_diskfree = $cache_stats{freesum};
 
 #NorduGrid middleware
-my ($ngversion) = '1.0.0';
+my ($ngversion) = '1.1.0';
 
 #Globus Toolkit version
 #globuslocation/share/doc/VERSION
@@ -368,7 +368,7 @@ for my $controldir (keys %controldirs) {
 
     close GMJOBSOUTPUT;
     my $status = $? >> 8;
-    error("$config{arc_location}/bin/gm-jobs returned with non-zero exit status $status")
+    error("$config{arc_location}/libexec/arc/gm-jobs returned with non-zero exit status $status")
         if $status;
 }
 
