@@ -9,6 +9,7 @@
 #include <unistd.h>
 #include <sys/mman.h>
 #include <fcntl.h>
+#include <errno.h>
 
 #include <glibmm.h>
 
@@ -25,8 +26,6 @@
 
 namespace ARex {
 
-static void GetGlueStates(Arc::XMLNode infodoc,std::map<std::string,std::string>& states);
-
 void ARexService::InformationCollector(void) {
   thread_count_.RegisterThread();
   for(;;) {
@@ -221,35 +220,6 @@ std::string ARexService::getID() {
   return "ARC:AREX";
 }
 
-static void GetGlueStates(Arc::XMLNode infodoc,std::map<std::string,std::string>& states) {
-  std::string path = "Domains/AdminDomain/Services/ComputingService/ComputingEndpoint/ComputingActivities/ComputingActivity";
-  states.clear();
-  // Obtaining all job descriptions
-  Arc::XMLNodeList nodes = infodoc.Path(path);
-  // Pulling ids and states
-  for(Arc::XMLNodeList::iterator node = nodes.begin();node!=nodes.end();++node) {
-    // Exract ID of job
-    std::string id = (*node)["IDFromEndpoint"];
-    if(id.empty()) id = (std::string)((*node)["ID"]);
-    if(id.empty()) continue;
-    std::string::size_type p = id.rfind('/');
-    if(p != std::string::npos) id.erase(0,p+1);
-    if(id.empty()) continue;
-    Arc::XMLNode state_node = (*node)["State"];
-    for(;(bool)state_node;++state_node) {
-      std::string state  = (std::string)state_node;
-      if(state.empty()) continue;
-      // Look for nordugrid prefix
-      if(strncmp("nordugrid:",state.c_str(),10) == 0) {
-        // Remove prefix
-        state.erase(0,10);
-        // Store state under id
-        states[id] = state;
-      };
-    };
-  };
-}
-
 class PrefixedFilePayload: public Arc::PayloadRawInterface {
  private:
   std::string prefix_;
@@ -449,5 +419,57 @@ void OptimizedInformationContainer::Assign(const std::string& xml) {
   };
 }
 
+#define ESFAULT(MSG) { \
+  logger_.msg(Arc::ERROR, std::string("ES:GetResourceInfo: ")+(MSG)); \
+  Arc::SOAPFault fault(out.Parent(),Arc::SOAPFault::Sender,(MSG)); \
+  ESInternalResourceInfoFault(fault,(MSG)); \
+  out.Destroy(); \
+  return Arc::MCC_Status(); \
+}
+
+Arc::MCC_Status ARexService::ESGetResourceInfo(ARexGMConfig& config,Arc::XMLNode in,Arc::XMLNode out) {
+  // WARNING. Suboptimal temporary solution.
+  int h = infodoc_.OpenDocument();
+  if(h == -1) ESFAULT("Failed to open resource information file");
+  ::lseek(h,0,SEEK_SET);
+  struct stat st;
+  if((::fstat(h,&st) != 0) || (st.st_size == 0)) ESFAULT("Failed to stat resource information file");
+  char* buf = (char*)::malloc(st.st_size+1);
+  if(!buf) ESFAULT("Failed to allocate memory for resoure information");
+  off_t p = 0;
+  for(;p<st.st_size;) {
+    ssize_t l = ::read(h,buf+p,st.st_size-p);
+    if(l == 0) break;
+    if(l == -1) {
+      if(errno != EAGAIN) {
+        ::free(buf);
+        ESFAULT("Failed to read resource information file");
+      };
+    };
+    p+=l;
+  };
+  buf[p] = 0;
+  Arc::XMLNode doc(buf);
+  if(!doc) {
+    ::free(buf);
+    ESFAULT("Failed to parse resource information file");
+  };
+  free(buf); buf=NULL;
+  Arc::XMLNode service = doc["Domains"]["AdminDomain"]["Services"]["ComputingService"];
+  if(!service) {
+    ESFAULT("Missing ComputingService in resource information");
+  };
+  service = out.NewChild(service);
+  //service.Name("glue2:ComputingService");
+  out.NewChild("glue2:ActivityManager");
+  return Arc::MCC_Status(Arc::STATUS_OK);
+}
+
+Arc::MCC_Status ARexService::ESQueryResourceInfo(ARexGMConfig& config,Arc::XMLNode in,Arc::XMLNode out) {
+  Arc::SOAPFault fault(out.Parent(),Arc::SOAPFault::Sender,"Operation not implemented yet");
+  out.Destroy();
+  return Arc::MCC_Status();
+}
+
 }
 
