@@ -56,7 +56,10 @@ ARexGMConfig::ARexGMConfig(const GMEnvironment& env,const std::string& uname,con
   bool strict_session;
   std::string gridftp_endpoint; // for gridftp interface
   std::string arex_endpoint; // our interface
-  if(!configure_user_dirs(uname,control_dir,session_roots,
+  std::string username = uname;
+  std::string::size_type p =username.find(':');
+  if(p != std::string::npos) username.resize(p);
+  if(!configure_user_dirs(username,control_dir,session_roots,
                           session_roots_non_draining_,
                           default_lrms,default_queue,queues_,
                           cont_plugins_,*cred_plugin,
@@ -282,7 +285,7 @@ ARexJob::ARexJob(Arc::XMLNode jsdl,ARexGMConfig& config,const std::string& crede
   // New job is created here
   // First get and acquire new id
   if(!make_job_id()) return;
-  // Turn JSDL into text
+  // Turn job description into text
   std::string job_desc_str;
   // Make full XML doc out of subtree
   {
@@ -298,7 +301,7 @@ ARexJob::ARexJob(Arc::XMLNode jsdl,ARexGMConfig& config,const std::string& crede
     failure_type_=ARexJobInternalError;
     return;
   };
-  // Analyze JSDL (checking, substituting, etc)
+  // Analyze job descrption (checking, substituting, etc)
   std::string acl("");
   if((failure_type_=setfail(parse_job_req(fname.c_str(),job_,&acl))) != ARexJobNoError) {
     if(failure_.empty()) {
