@@ -8,8 +8,10 @@
 #include <fcntl.h>
 #include <pwd.h>
 #include <grp.h>
+#include <signal.h>
 
 #include <arc/Logger.h>
+#include <arc/Utils.h>
 
 #include "conf.h"
 #include "environment.h"
@@ -21,7 +23,15 @@ namespace gridftpd {
 
   static Arc::Logger logger(Arc::Logger::getRootLogger(),"Daemon");
 
-  Daemon::Daemon(void):logfile_(""),logsize_(0),lognum_(5),uid_((uid_t)(-1)),gid_((gid_t)(-1)),daemon_(true),pidfile_(""),debug_(-1) {
+  static Arc::LogFile* sighup_dest = NULL;
+
+  static void sighup_handler(int) {
+    if(!sighup_dest) return;
+    sighup_dest->setReopen(true);
+    sighup_dest->setReopen(false);
+  }
+
+  Daemon::Daemon(void):logfile_(""),logsize_(0),lognum_(5),logreopen_(false),uid_((uid_t)(-1)),gid_((gid_t)(-1)),daemon_(true),pidfile_(""),debug_(-1) {
   }
 
   Daemon::~Daemon(void) {
@@ -83,21 +93,23 @@ namespace gridftpd {
 
   int Daemon::config(const std::string& cmd,std::string& rest) {
     if(cmd == "gridmap") {
-      setenv("GRIDMAP",rest.c_str(),1); return 0;
+      Arc::SetEnv("GRIDMAP",rest.c_str()); return 0;
     } else if(cmd == "hostname") {
-      setenv("GLOBUS_HOSTNAME",rest.c_str(),1); return 0;
+      Arc::SetEnv("GLOBUS_HOSTNAME",rest.c_str()); return 0;
     } else if(cmd == "globus_tcp_port_range") {
-      setenv("GLOBUS_TCP_PORT_RANGE",rest.c_str(),1); return 0;
+      Arc::SetEnv("GLOBUS_TCP_PORT_RANGE",rest.c_str()); return 0;
     } else if(cmd == "globus_udp_port_range") {
-      setenv("GLOBUS_UDP_PORT_RANGE",rest.c_str(),1); return 0;
+      Arc::SetEnv("GLOBUS_UDP_PORT_RANGE",rest.c_str()); return 0;
     } else if(cmd == "x509_user_key") {
-      setenv("X509_USER_KEY",rest.c_str(),1); return 0;
+      Arc::SetEnv("X509_USER_KEY",rest.c_str()); return 0;
     } else if(cmd == "x509_user_cert") {
-      setenv("X509_USER_CERT",rest.c_str(),1); return 0;
+      Arc::SetEnv("X509_USER_CERT",rest.c_str()); return 0;
     } else if(cmd == "x509_cert_dir") {
-      setenv("X509_CERT_DIR",rest.c_str(),1); return 0;
+      Arc::SetEnv("X509_CERT_DIR",rest.c_str()); return 0;
     } else if(cmd == "http_proxy") {
-      setenv("ARC_HTTP_PROXY",rest.c_str(),1); return 0;
+      Arc::SetEnv("ARC_HTTP_PROXY",rest.c_str()); return 0;
+    } else if(cmd == "voms_processing") {
+      Arc::SetEnv("VOMS_PROCESSING",rest.c_str()); return 0;
     };
     if(cmd == "daemon") {
       if(daemon_) {
@@ -137,6 +149,15 @@ namespace gridftpd {
           return -1;
         };
       };
+    } else if(cmd == "logreopen") {
+      std::string arg = config_next_arg(rest);
+      if(arg=="") {
+        logger.msg(Arc::ERROR, "Missing option for command logreopen");
+        return -1;
+      };
+      if(strcasecmp("yes",arg.c_str()) == 0) { logreopen_=true; }
+      else if(strcasecmp("no",arg.c_str()) == 0) { logreopen_=false; }
+      else { logger.msg(Arc::ERROR, "Wrong option in logreopen"); return -1; };
     } else if(cmd == "user") {
       if(uid_ == (uid_t)(-1)) {
         std::string username = config_next_arg(rest);
@@ -223,15 +244,18 @@ namespace gridftpd {
       logger.msg(Arc::ERROR, "Failed to open log file %s", logfile_);
       return 1;
     }
-    if (logsize_ > 0)
-      logger_file->setMaxSize(logsize_);
-    if (lognum_ > 0)
-      logger_file->setBackups(lognum_);
-    if (debug_ > 0)
+    if (logsize_ > 0) logger_file->setMaxSize(logsize_);
+    if (lognum_ > 0) logger_file->setBackups(lognum_);
+    logger_file->setReopen(logreopen_);
+    if (debug_ > 0) {
       Arc::Logger::getRootLogger().setThreshold(Arc::old_level_to_level((unsigned int)debug_));
+    };
     Arc::Logger::getRootLogger().removeDestinations();
     Arc::Logger::getRootLogger().addDestination(*logger_file);
-
+    if(!logreopen_) {
+      sighup_dest = logger_file;
+      signal(SIGHUP,&sighup_handler);
+    };
     if(close_fds) {
       struct rlimit lim;
       int max_files;
@@ -251,7 +275,7 @@ namespace gridftpd {
       };
     };
     const char* log = logfile_.c_str();
-    if(daemon_) { if(log[0] == 0) log="/dev/null"; };
+    if(daemon_) { /*if(log[0] == 0)*/ log="/dev/null"; };
     if(log[0] != 0) {
       close(1); close(2);
       int h=::open(log,O_WRONLY | O_CREAT | O_APPEND,S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
@@ -298,7 +322,10 @@ namespace gridftpd {
     };
     if(hp != -1) {
       char buf[30]; int l = snprintf(buf,29,"%u",getpid()); buf[l]=0;
-      ::write(hp,buf,l);
+      if(::write(hp,buf,l)<0) {
+        ::close(hp);
+        return 0;
+      };
       ::close(hp);
     };
     return 0;
