@@ -15,6 +15,8 @@
 #include <sys/vfs.h>
 #endif
 
+#include <arc/Utils.h>
+
 #include "fileplugin.h"
 #include "../conf/conf.h"
 #include "../userspec.h"
@@ -266,13 +268,13 @@ int DirectFilePlugin::makedir(std::string &dname) {
     if(i->unix_set(uid,gid) == 0) {
       if(::mkdir(fdname.c_str(),
               i->access.mkdir_perm_or & i->access.mkdir_perm_and) == 0) {
-        (void)::chmod(fdname.c_str(),
+        chmod(fdname.c_str(),
               i->access.mkdir_perm_or & i->access.mkdir_perm_and);
         i->unix_reset();
         uid_t u = i->access.mkdir_uid;
         gid_t g = i->access.mkdir_gid;
         if(u == ((uid_t)(-1))) u=uid; if(g == ((gid_t)(-1))) g=gid;
-        (void)::chown(fdname.c_str(),u,g);
+        if(chown(fdname.c_str(),u,g) != 0) {}
         continue;
       } else {
         i->unix_reset();
@@ -307,9 +309,18 @@ int DirectFilePlugin::removefile(std::string &name) {
   if(!(i->access.del)) return 1;
   std::string fname=real_name(name);
   int ur=i->unix_rights(fname,uid,gid);
+  if(ur == 0 && errno > 0) { // stat failed
+    error_description = Arc::StrError(errno);
+    return 1;
+  }
+  if(ur & S_IFDIR) {
+    error_description = "Is a directory";
+    return 1;
+  }
   if(!(ur & S_IFREG)) return 1;
   if(i->unix_set(uid,gid) != 0) return 1;
   if(::remove(fname.c_str()) != 0) {
+    error_description = Arc::StrError(errno);
     i->unix_reset();
     return 1;
   };
@@ -323,9 +334,17 @@ int DirectFilePlugin::removedir(std::string &dname) {
   if(!(i->access.del)) return 1;
   std::string fdname=real_name(dname);
   int ur=i->unix_rights(fdname,uid,gid);
-  if(!(ur & S_IFDIR)) return 1;
+  if(ur == 0 && errno > 0) { // stat failed
+    error_description = Arc::StrError(errno);
+    return 1;
+  }
+  if(!(ur & S_IFDIR)) {
+    error_description = "Not a directory";
+    return 1;
+  }
   if(i->unix_set(uid,gid) != 0) return 1;
   if(::remove(fdname.c_str()) != 0) {
+    error_description = Arc::StrError(errno);
     i->unix_reset();
     return 1;
   };
@@ -341,6 +360,10 @@ int DirectFilePlugin::open(const char* name,open_modes mode,unsigned long long i
     if(i==access.end()) return 1; /* error ? */
     if(i->access.read) {
       int ur=(*i).unix_rights(fname,uid,gid);
+      if(ur == 0 && errno > 0) { // stat failed
+        error_description = Arc::StrError(errno);
+        return 1;
+      }
       if((ur & S_IFREG) && (ur & S_IRUSR)) {
         /* so open it */
         if(i->unix_set(uid,gid) != 0) return 1;
@@ -390,10 +413,11 @@ int DirectFilePlugin::open(const char* name,open_modes mode,unsigned long long i
           if(data_file == -1) return 1;
           file_mode=file_access_overwrite;
           file_name=fname;
-          truncate(file_name.c_str(),0);
+          if(truncate(file_name.c_str(),0) != 0) {}
           return 0;
         };
       };
+      error_description="File exists, overwrite not allowed";
       return 1;
     }
     else if(ur & S_IFDIR) { /* it's a directory */
@@ -429,9 +453,9 @@ int DirectFilePlugin::open(const char* name,open_modes mode,unsigned long long i
           gid_t g = i->access.creat_gid;
           if(u == ((uid_t)(-1))) u=uid; if(g == ((gid_t)(-1))) g=gid;
           logger.msg(Arc::VERBOSE, "open: changing owner for %s, %i, %i", fname, u, gid);
-          (void)chown(fname.c_str(),u,g);
+          if(chown(fname.c_str(),u,g) != 0) {}
           /* adjust permissions because open uses umask */
-          (void)chmod(fname.c_str(),
+          chmod(fname.c_str(),
                 i->access.creat_perm_or & i->access.creat_perm_and);
           struct stat st;
           stat(fname.c_str(),&st);
@@ -481,9 +505,9 @@ int DirectFilePlugin::open_direct(const char* name,open_modes mode) {
     if(data_file == -1) return 1;
     file_mode=file_access_create;
     file_name=fname;
-    truncate(file_name.c_str(),0);
-    (void)chown(fname.c_str(),uid,gid);
-    (void)chmod(fname.c_str(),S_IRUSR | S_IWUSR);
+    if(truncate(file_name.c_str(),0) != 0) {}
+    if(chown(fname.c_str(),uid,gid) != 0) {}
+    chmod(fname.c_str(),S_IRUSR | S_IWUSR);
     return 0;
   }
   else {
@@ -604,8 +628,9 @@ int DirectAccess::unix_set(int uid,int gid) {
 }
 
 void DirectAccess::unix_reset(void) {
-    if(geteuid() != getuid()) seteuid(getuid());
-    if(getegid() != getgid()) setegid(getgid());
+  if(access.access == local_none_access) return;
+  if(geteuid() != getuid()) seteuid(getuid());
+  if(getegid() != getgid()) setegid(getgid());
 }
 
 std::list<DirectAccess>::iterator DirectFilePlugin::control_dir(const std::string &name,bool indir) {
@@ -686,13 +711,16 @@ int DirectFilePlugin::readdir(const char* name,std::list<DirEntry> &dir_list,Dir
   std::string fname = real_name(name);
   if(i->access.dirlist) {
     int ur=i->unix_rights(fname,uid,gid);
+    if(ur == 0 && errno > 0) { // stat failed
+      error_description = Arc::StrError(errno);
+      return 1;
+    }
     if((ur & S_IFDIR) && (ur & S_IRUSR) && (ur & S_IXUSR)) {
       /* allowed to list in configuration and by unix rights */
       /* following Linux semantics - need r-x for dirlist */
       /* now get real listing */
       if(i->unix_set(uid,gid) != 0) return 1;
       DIR* d=::opendir(fname.c_str());
-      i->unix_reset();
       if(d == NULL) { return 1; }; /* maybe return ? */
       struct dirent *de;
       for(;;) {
@@ -700,12 +728,15 @@ int DirectFilePlugin::readdir(const char* name,std::list<DirEntry> &dir_list,Dir
         if(de == NULL) break;
         if((!strcmp(de->d_name,".")) || (!strcmp(de->d_name,".."))) continue;
         DirEntry dent(true,de->d_name); // treat it as file by default
+        i->unix_reset();
         bool is_manageable = fill_object_info(dent,fname,ur,i,mode);
+        i->unix_set(uid,gid);
         if(is_manageable) {
           dir_list.push_back(dent);
         };
       };
       ::closedir(d);
+      i->unix_reset();
       return 0;
     }
     else if(ur & S_IFREG) {
@@ -729,6 +760,10 @@ int DirectFilePlugin::checkdir(std::string &dirname) {
   std::string fname = real_name(dirname);
   if(i->access.cd) {
     int ur=(*i).unix_rights(fname,uid,gid);
+    if(ur == 0 && errno > 0) { // stat failed
+      error_description = Arc::StrError(errno);
+      return 1;
+    }
     if((ur & S_IXUSR) && (ur & S_IFDIR)) {
   logger.msg(Arc::VERBOSE, "plugin: checkdir: access: allowed: %s", fname);
       return 0;
@@ -753,11 +788,16 @@ int DirectFilePlugin::checkfile(std::string &name,DirEntry &info,DirEntry::objec
   if(!(i->access.dirlist)) { return 1; };
   std::string fdname = real_name(dname);
   int ur=i->unix_rights(fdname,uid,gid);
+  if(ur == 0 && errno > 0) { // stat failed
+    error_description = Arc::StrError(errno);
+    return 1;
+  }
   if(!((ur & S_IXUSR) && (ur & S_IFDIR)))  { return 1; };
   std::string fname = real_name(name);
   DirEntry dent(true,get_last_name(fname.c_str()));
   bool is_manageable = fill_object_info(dent,fdname,ur,i,mode);
   if(!is_manageable) {
+    if (errno > 0) error_description = Arc::StrError(errno);
     return 1;
   };
   info=dent;
