@@ -41,13 +41,13 @@ int FileRoot::config(gridftpd::Daemon &daemon,ServerParams* params) {
     int r;
     r=daemon.config(command,rest);
     if(r == 0) continue;
-    if(r == -1) { gridftpd::config_close(cfile); if(cf) delete cf; return 1; };
+    if(r == -1) { gridftpd::config_close(cfile); delete cf; return 1; };
     if(command == "port") {
       if(params) {
         if(sscanf(rest.c_str(),"%u",&(params->port)) != 1) {
           logger.msg(Arc::ERROR, "Wrong port number in configuration");
           gridftpd::config_close(cfile);
-          if(cf) delete cf;
+          delete cf;
           return 1;
         };
       };
@@ -56,7 +56,7 @@ int FileRoot::config(gridftpd::Daemon &daemon,ServerParams* params) {
         if(sscanf(rest.c_str(),"%u",&(params->max_connections)) != 1) {
           logger.msg(Arc::ERROR, "Wrong maxconnections number in configuration");
           gridftpd::config_close(cfile);
-          if(cf) delete cf;
+          delete cf;
           return 1;
         };
       };
@@ -65,7 +65,7 @@ int FileRoot::config(gridftpd::Daemon &daemon,ServerParams* params) {
         if(sscanf(rest.c_str(),"%u",&(params->default_buffer)) != 1) {
           logger.msg(Arc::ERROR, "Wrong defaultbuffer number in configuration");
           gridftpd::config_close(cfile);
-          if(cf) delete cf;
+          delete cf;
           return 1;
         };
       };
@@ -74,7 +74,7 @@ int FileRoot::config(gridftpd::Daemon &daemon,ServerParams* params) {
         if(sscanf(rest.c_str(),"%u",&(params->max_buffer)) != 1) {
           logger.msg(Arc::ERROR, "Wrong maxbuffer number in configuration");
           gridftpd::config_close(cfile);
-          if(cf) delete cf;
+          delete cf;
           return 1;
         };
       };
@@ -114,7 +114,7 @@ int FileRoot::config(gridftpd::Daemon &daemon,ServerParams* params) {
             (host->h_addr_list[0] == NULL) ) {
           logger.msg(Arc::ERROR, "Can't resolve host %s", value);
           gridftpd::config_close(cfile);
-          if(cf) delete cf;
+          delete cf;
           return 1;
         };
         unsigned char* addr = (unsigned char*)(&(((struct in_addr*)(host->h_addr_list[0]))->s_addr));
@@ -125,8 +125,8 @@ int FileRoot::config(gridftpd::Daemon &daemon,ServerParams* params) {
       };
     };
   };
-    gridftpd::config_close(cfile);
-  if(cf) delete cf;
+  gridftpd::config_close(cfile);
+  delete cf;
   return 0;
 }
 
@@ -197,7 +197,11 @@ int FileRoot::config(std::ifstream &cfile,std::string &pluginpath) {
         if(rest == "end") break;
         if(decision == AAA_NO_MATCH) {
           decision = user.user.evaluate(rest.c_str());
-          if(decision == AAA_FAILURE) decision=AAA_NO_MATCH;
+          if(decision == AAA_FAILURE) {
+            //decision=AAA_NO_MATCH;
+            logger.msg(Arc::ERROR, "Failed processing authorization group %s",group_name);
+            return 1;
+          };
         };
       };
       if(decision == AAA_POSITIVE_MATCH) user.user.add_group(group_name);
@@ -465,7 +469,11 @@ int FileRoot::config(gridftpd::ConfigSections &cf,std::string &pluginpath) {
           if(group_decision == AAA_NO_MATCH) {
             rest=command+" "+rest;
             group_decision = user.user.evaluate(rest.c_str());
-            if(group_decision == AAA_FAILURE) group_decision=AAA_NO_MATCH;
+            if(group_decision == AAA_FAILURE) {
+              //group_decision=AAA_NO_MATCH;
+              logger.msg(Arc::ERROR, "Failed processing authorization group %s",group_name);
+              return 1;
+            };
           };
         };
       }; break;
@@ -503,7 +511,7 @@ int FileRoot::config(globus_ftp_control_auth_info_t *auth,
   /* open and read configuration file */
   std::ifstream cfile;
   gridftpd::ConfigSections* cf = NULL;
-  config_open_gridftp(cfile);
+  std::string config_file = config_open_gridftp(cfile);
   if(!cfile.is_open()) {
     logger.msg(Arc::ERROR, "configuration file not found");
     delete cf;
@@ -515,7 +523,8 @@ int FileRoot::config(globus_ftp_control_auth_info_t *auth,
   cf->AddSection("gridftpd");
   cf->AddSection("vo");
   /* keep information about user */
-  if(!user.fill(auth,handle)) {
+  if(!user.fill(auth,handle,config_file.c_str())) {
+    logger.msg(Arc::ERROR, "failed to process client identification");
     delete cf;
     return 1;
   };
