@@ -1,6 +1,8 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <arpa/inet.h>
+#include <unistd.h>
+#include <signal.h>
 
 #include <globus_common.h>
 #include <globus_ftp_control.h>
@@ -117,7 +119,7 @@ int main(int argc,char** argv) {
   Arc::Logger::getRootLogger().setThreshold(Arc::INFO);
 
   globus_module_deactivate_all();
-  setpgrp();
+  setpgid(0,0);
   sig_old_term=signal(SIGTERM,&sig_term);
   if(sig_old_term == SIG_ERR) {
     perror("");
@@ -157,7 +159,7 @@ int main_internal(int argc,char** argv) {
 #else
 void sig_term_fork(int /* signum */) {
   int static passed = 0;
-  if(passed) return;
+  if(passed) _exit(-1);
   server_done=1;
   passed=1;
   kill(0,SIGTERM);
@@ -167,7 +169,7 @@ int main(int argc,char** argv) {
 #ifndef __DONT_USE_FORK__
   globus_module_deactivate_all();
 #endif
-  setpgrp();
+  setpgid(0,0);
 #endif
 
   // temporary stderr destination until configuration is read and used in daemon.daemon()
